@@ -65,7 +65,6 @@ struct Blob {
 */
 #define BLOB_SEEK_SET 1
 #define BLOB_SEEK_CUR 2
-#define BLOB_SEEK_END 3
 
 #endif /* INTERFACE */
 
@@ -278,15 +277,40 @@ void blob_zero(Blob *pBlob){
 }
 
 /*
-** Append text or data to the end of a blob.
+** Append text or data to the end of a blob.  Or, if pBlob==NULL, send
+** the text to standard output in terminal mode, or to standard CGI output
+** in CGI mode.
+**
+** If nData<0 then output all of aData up to the first 0x00 byte.
+**
+** Use the blob_append() routine in all application code.  The blob_append()
+** routine is faster, but blob_append_full() handles all the corner cases.
+** The blob_append() routine automatically calls blob_append_full() if
+** necessary.
 */
-void blob_append(Blob *pBlob, const char *aData, int nData){
-  assert( aData!=0 || nData==0 );
-  blob_is_init(pBlob);
+static void blob_append_full(Blob *pBlob, const char *aData, int nData){
+  sqlite3_int64 nNew;
+  /* assert( aData!=0 || nData==0 ); // omitted for speed */
+  /* blob_is_init(pBlob); // omitted for speed */
   if( nData<0 ) nData = strlen(aData);
   if( nData==0 ) return;
-  if( pBlob->nUsed + nData >= pBlob->nAlloc ){
-    pBlob->xRealloc(pBlob, pBlob->nUsed + nData + pBlob->nAlloc + 100);
+  if( pBlob==0 ){
+    if( g.cgiOutput ){
+      pBlob = cgi_output_blob();
+    }else{
+      fossil_puts(aData, 0, nData);
+      return;
+    }
+  }
+  nNew = pBlob->nUsed;
+  nNew += nData;
+  if( nNew >= pBlob->nAlloc ){
+    nNew += pBlob->nAlloc;
+    nNew += 100;
+    if( nNew>=0x7fff0000 ){
+      blob_panic();
+    }
+    pBlob->xRealloc(pBlob, (int)nNew);
     if( pBlob->nUsed + nData >= pBlob->nAlloc ){
       blob_panic();
     }
@@ -295,22 +319,40 @@ void blob_append(Blob *pBlob, const char *aData, int nData){
   pBlob->nUsed += nData;
   pBlob->aData[pBlob->nUsed] = 0;   /* Blobs are always nul-terminated */
 }
-
-/*
-** Append a single character to the blob
-*/
-void blob_append_char(Blob *pBlob, char c){
-  if( pBlob->nUsed+1 >= pBlob->nAlloc ){
-    pBlob->xRealloc(pBlob, pBlob->nUsed + pBlob->nAlloc + 100);
-    if( pBlob->nUsed + 1 >= pBlob->nAlloc ){
-      blob_panic();
-    }
+void blob_append(Blob *pBlob, const char *aData, int nData){
+  sqlite3_int64 nUsed;
+  /* assert( aData!=0 || nData==0 ); // omitted for speed */
+  if( nData<=0 || pBlob==0 || pBlob->nUsed + nData >= pBlob->nAlloc ){
+    blob_append_full(pBlob, aData, nData);
+    return;
   }
-  pBlob->aData[pBlob->nUsed++] = c;    
+  nUsed = pBlob->nUsed;
+  pBlob->nUsed += nData;
+  pBlob->aData[pBlob->nUsed] = 0;
+  memcpy(&pBlob->aData[nUsed], aData, nData);
 }
 
 /*
-** Copy a blob
+** Append a string literal to a blob.
+*/
+#if INTERFACE
+#define blob_append_string(BLOB,STR) blob_append(BLOB,STR,sizeof(STR)-1)
+#endif
+
+/*
+** Append a single character to the blob.  If pBlob is zero then the
+** character is written directly to stdout.
+*/
+void blob_append_char(Blob *pBlob, char c){
+  if( pBlob==0 || pBlob->nUsed+1 >= pBlob->nAlloc ){
+    blob_append_full(pBlob, &c, 1);
+  }else{
+    pBlob->aData[pBlob->nUsed++] = c;
+  }
+}
+
+/*
+** Copy a blob.  pTo is reinitialized to be a copy of pFrom.
 */
 void blob_copy(Blob *pTo, Blob *pFrom){
   blob_is_init(pFrom);
@@ -318,6 +360,90 @@ void blob_copy(Blob *pTo, Blob *pFrom){
   blob_append(pTo, blob_buffer(pFrom), blob_size(pFrom));
 }
 
+/*
+** Append the second blob onto the end of the first blob and reset the
+** second blob.  If the first blob (pTo) is NULL, then the content
+** of the second blob is written to stdout or to CGI depending on if the
+** Fossil is running in terminal or CGI mode.
+*/
+void blob_append_xfer(Blob *pTo, Blob *pFrom){
+  blob_append(pTo, blob_buffer(pFrom), blob_size(pFrom));
+  blob_reset(pFrom);
+}
+
+/*
+** Write into pOut, a string literal representation for the first n bytes
+** of z[].  The string literal representation is compatible with C, TCL,
+** and JSON.  Double-quotes are added to both ends.  Double-quote and
+** backslash characters are escaped.
+*/
+void blob_append_tcl_literal(Blob *pOut, const char *z, int n){
+  int i;
+  blob_append_char(pOut, '"');
+  for(i=0; i<n; i++){
+    char c = z[i];
+    switch( c ){
+      case '\r':  c = 'r';
+      case '[':
+      case ']':
+      case '$':
+      case '"':
+      case '\\':
+        blob_append_char(pOut, '\\');
+      default:
+        blob_append_char(pOut, c);
+    }
+  }
+  blob_append_char(pOut, '"');
+}
+void blob_append_json_literal(Blob *pOut, const char *z, int n){
+  int i;
+  blob_append_char(pOut, '"');
+  for(i=0; i<n; i++){
+    char c = z[i];
+    switch( c ){
+      case 0x00:
+      case 0x01:
+      case 0x02:
+      case 0x03:
+      case 0x04:
+      case 0x05:
+      case 0x06:
+      case 0x07: c += '0' - 0x00; blob_append(pOut, "\\u000",5); break;
+      case 0x0b:
+      case 0x0e:
+      case 0x0f: c += 'a' - 0x0a; blob_append(pOut, "\\u000",5); break;
+      case 0x10:
+      case 0x11:
+      case 0x12:
+      case 0x13:
+      case 0x14:
+      case 0x15:
+      case 0x16:
+      case 0x17:
+      case 0x18:
+      case 0x19: c += '0' - 0x10; blob_append(pOut, "\\u001",5); break;
+      case 0x1a:
+      case 0x1b:
+      case 0x1c:
+      case 0x1d:
+      case 0x1e:
+      case 0x1f: c += 'a' - 0x1a; blob_append(pOut, "\\u001",5); break;
+      case '\b': c = 'b';         blob_append_char(pOut, '\\');  break;
+      case '\t': c = 't';         blob_append_char(pOut, '\\');  break;
+      case '\r': c = 'r';         blob_append_char(pOut, '\\');  break;
+      case '\n': c = 'n';         blob_append_char(pOut, '\\');  break;
+      case '\f': c = 'f';         blob_append_char(pOut, '\\');  break;
+      case '"':                   blob_append_char(pOut, '\\');  break;
+      case '\\':                  blob_append_char(pOut, '\\');  break;
+      default:                                                   break;
+    }
+    blob_append_char(pOut, c);
+  }
+  blob_append_char(pOut, '"');
+}
+
+
 /*
 ** Return a pointer to a null-terminated string for a blob.
 */
@@ -327,12 +453,25 @@ char *blob_str(Blob *p){
     blob_append_char(p, 0); /* NOTE: Changes nUsed. */
     p->nUsed = 0;
   }
-  if( p->aData[p->nUsed]!=0 ){
+  if( p->nUsed<p->nAlloc ){
+    p->aData[p->nUsed] = 0;
+  }else{
     blob_materialize(p);
   }
   return p->aData;
 }
 
+/*
+** Compute the string length of a Blob.  If there are embedded
+** nul characters, truncate the to blob at the first nul.
+*/
+int blob_strlen(Blob *p){
+  char *z = blob_str(p);
+  if( z==0 ) return 0;
+  p->nUsed = (int)strlen(p->aData);
+  return p->nUsed;
+}
+
 /*
 ** Return a pointer to a null-terminated string for a blob that has
 ** been created using blob_append_sql() and not blob_appendf().  If
@@ -417,7 +556,7 @@ int blob_constant_time_cmp(Blob *pA, Blob *pB){
 int blob_eq_str(Blob *pBlob, const char *z, int n){
   Blob t;
   blob_is_init(pBlob);
-  if( n<=0 ) n = strlen(z);
+  if( n<=0 ) n = (int)strlen(z);
   t.aData = (char*)z;
   t.nUsed = n;
   t.xRealloc = blobReallocStatic;
@@ -431,7 +570,7 @@ int blob_eq_str(Blob *pBlob, const char *z, int n){
 */
 #if INTERFACE
 # define blob_eq(B,S) \
-     ((B)->nUsed==sizeof(S)-1 && memcmp((B)->aData,S,sizeof(S)-1)==0)
+     ((B)->nUsed==sizeof(S"")-1 && memcmp((B)->aData,S,sizeof(S)-1)==0)
 #endif
 
 
@@ -445,6 +584,30 @@ void blob_resize(Blob *pBlob, unsigned int newSize){
   pBlob->aData[newSize] = 0;
 }
 
+/*
+** Ensures that the given blob has at least the given amount of memory
+** allocated to it. Does not modify pBlob->nUsed nor will it reduce
+** the currently-allocated amount of memory.
+**
+** For semantic compatibility with blob_append_full(), if newSize is
+** >=0x7fff000 (~2GB) then this function will trigger blob_panic(). If
+** it didn't, it would be possible to bypass that hard-coded limit via
+** this function.
+**
+** We've had at least one report:
+**   https://fossil-scm.org/forum/forumpost/b7bbd28db4
+** which implies that this is unconditionally failing on mingw 32-bit
+** builds.
+*/
+void blob_reserve(Blob *pBlob, unsigned int newSize){
+  if(newSize>=0x7fff0000 ){
+    blob_panic();
+  }else if(newSize>pBlob->nUsed){
+    pBlob->xRealloc(pBlob, newSize);
+    pBlob->aData[newSize] = 0;
+  }
+}
+
 /*
 ** Make sure a blob is nul-terminated and is not a pointer to unmanaged
 ** space.  Return a pointer to the data.
@@ -511,8 +674,6 @@ int blob_seek(Blob *p, int offset, int whence){
     p->iCursor = offset;
   }else if( whence==BLOB_SEEK_CUR ){
     p->iCursor += offset;
-  }else if( whence==BLOB_SEEK_END ){
-    p->iCursor = p->nUsed + offset - 1;
   }
   if( p->iCursor>p->nUsed ){
     p->iCursor = p->nUsed;
@@ -770,30 +931,27 @@ int blob_tokenize(Blob *pIn, Blob *aToken, int nToken){
 }
 
 /*
-** Do printf-style string rendering and append the results to a blob.
+** Do printf-style string rendering and append the results to a blob.  Or
+** if pBlob==0, do printf-style string rendering directly to stdout.
 **
 ** The blob_appendf() version sets the BLOBFLAG_NotSQL bit in Blob.blobFlags
 ** whereas blob_append_sql() does not.
 */
 void blob_appendf(Blob *pBlob, const char *zFormat, ...){
-  if( pBlob ){
-    va_list ap;
-    va_start(ap, zFormat);
-    vxprintf(pBlob, zFormat, ap);
-    va_end(ap);
-    pBlob->blobFlags |= BLOBFLAG_NotSQL;
-  }
+  va_list ap;
+  va_start(ap, zFormat);
+  vxprintf(pBlob, zFormat, ap);
+  va_end(ap);
+  if( pBlob ) pBlob->blobFlags |= BLOBFLAG_NotSQL;
 }
 void blob_append_sql(Blob *pBlob, const char *zFormat, ...){
-  if( pBlob ){
-    va_list ap;
-    va_start(ap, zFormat);
-    vxprintf(pBlob, zFormat, ap);
-    va_end(ap);
-  }
+  va_list ap;
+  va_start(ap, zFormat);
+  vxprintf(pBlob, zFormat, ap);
+  va_end(ap);
 }
 void blob_vappendf(Blob *pBlob, const char *zFormat, va_list ap){
-  if( pBlob ) vxprintf(pBlob, zFormat, ap);
+  vxprintf(pBlob, zFormat, ap);
 }
 
 /*
@@ -933,7 +1091,7 @@ int blob_write_to_file(Blob *pBlob, const char *zFilename){
     file_mkfolder(zFilename, ExtFILE, 1, 0);
     out = fossil_fopen(zFilename, "wb");
     if( out==0 ){
-#if _WIN32
+#if defined(_WIN32)
       const char *zReserved = file_is_win_reserved(zFilename);
       if( zReserved ){
         fossil_fatal("cannot open \"%s\" because \"%s\" is "
@@ -1134,7 +1292,6 @@ void test_cycle_compress(void){
   fossil_print("ok\n");
 }
 
-#if defined(_WIN32) || defined(__CYGWIN__)
 /*
 ** Convert every \n character in the given blob into \r\n.
 */
@@ -1158,7 +1315,6 @@ void blob_add_cr(Blob *p){
     }
   }
 }
-#endif
 
 /*
 ** Remove every \r character from the given blob, replacing each one with
@@ -1230,51 +1386,322 @@ void blob_cp1252_to_utf8(Blob *p){
   }
 }
 
+/*
+** ASCII (for reference):
+**    x0  x1  x2  x3  x4  x5  x6  x7  x8  x9  xa  xb  xc  xd  xe  xf 
+** 0x ^`  ^a  ^b  ^c  ^d  ^e  ^f  ^g  \b  \t  \n  ()  \f  \r  ^n  ^o 
+** 1x ^p  ^q  ^r  ^s  ^t  ^u  ^v  ^w  ^x  ^y  ^z  ^{  ^|  ^}  ^~  ^ 
+** 2x ()  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /  
+** 3x 0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?  
+** 4x @   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O  
+** 5x P   Q   R   S   T   U   V   W   X   Y   Z   [   \   ]   ^   _  
+** 6x `   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o  
+** 7x p   q   r   s   t   u   v   w   x   y   z   {   |   }   ~   ^_ 
+*/
+
+/*
+** Meanings for bytes in a filename:
+**
+**    0      Ordinary character.  No encoding required
+**    1      Needs to be escaped
+**    2      Illegal character.  Do not allow in a filename
+**    3      First byte of a 2-byte UTF-8
+**    4      First byte of a 3-byte UTF-8
+**    5      First byte of a 4-byte UTF-8
+*/
+static const char aSafeChar[256] = {
+#ifdef _WIN32
+/* Windows
+** Prohibit:  all control characters, including tab, \r and \n
+** Escape:    (space) " # $ % & ' ( ) * ; < > ? [ ] ^ ` { | }
+*/
+/*  x0  x1  x2  x3  x4  x5  x6  x7  x8  x9  xa  xb  xc  xd  xe  xf  */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0x */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 1x */
+     1,  0,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0, /* 2x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  1,  1, /* 3x */
+     1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 4x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  1,  1,  0, /* 5x */
+     1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 6x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  0,  1, /* 7x */
+#else
+/* Unix
+** Prohibit:  all control characters, including tab, \r and \n
+** Escape:    (space) ! " # $ % & ' ( ) * ; < > ? [ \ ] ^ ` { | }
+*/
+/*  x0  x1  x2  x3  x4  x5  x6  x7  x8  x9  xa  xb  xc  xd  xe  xf  */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0x */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 1x */
+     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0, /* 2x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  1,  1, /* 3x */
+     1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 4x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  0, /* 5x */
+     1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 6x */
+     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  0,  1, /* 7x */
+#endif
+    /* all bytes 0x80 through 0xbf are unescaped, being secondary
+    ** bytes to UTF8 characters.  Bytes 0xc0 through 0xff are the
+    ** first byte of a UTF8 character and do get escaped */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 8x */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 9x */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* ax */
+     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* bx */
+     3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3, /* cx */
+     3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3, /* dx */
+     4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4, /* ex */
+     5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5  /* fx */
+};
+
 /*
 ** pBlob is a shell command under construction.  This routine safely
-** appends argument zIn.
+** appends filename argument zIn.
 **
 ** The argument is escaped if it contains white space or other characters
 ** that need to be escaped for the shell.  If zIn contains characters
 ** that cannot be safely escaped, then throw a fatal error.
 **
-** The argument is expected to a filename of some kinds.  As shell commands
-** commonly have command-line options that begin with "-" and since we
-** do not want an attacker to be able to invoke these switches using
-** filenames that begin with "-", if zIn begins with "-", prepend
-** an additional "./".
+** If the isFilename argument is true, then the argument is expected
+** to be a filename.  As shell commands commonly have command-line
+** options that begin with "-" and since we do not want an attacker
+** to be able to invoke these switches using filenames that begin
+** with "-", if zIn begins with "-", prepend an additional "./"
+** (or ".\\" on Windows).
 */
-void blob_append_escaped_arg(Blob *pBlob, const char *zIn){
+void blob_append_escaped_arg(Blob *pBlob, const char *zIn, int isFilename){
   int i;
-  char c;
+  unsigned char c;
   int needEscape = 0;
   int n = blob_size(pBlob);
   char *z = blob_buffer(pBlob);
-#if defined(_WIN32)
-  const char cQuote = '"';    /* Use "..." quoting on windows */
-#else
-  const char cQuote = '\'';   /* Use '...' quoting on unix */
-#endif
 
-  for(i=0; (c = zIn[i])!=0; i++){
-    if( c==cQuote || c=='\\' || c<' ' || c==';' || c=='*' || c=='?' || c=='[') {
-      Blob bad;
-      blob_token(pBlob, &bad);
-      fossil_fatal("the [%s] argument to the \"%s\" command contains "
-                   "a character (ascii 0x%02x) that is a security risk",
-                   zIn, blob_str(&bad), c);
-    }
-    if( !needEscape && !fossil_isalnum(c) && c!='/' && c!='.' && c!='_' ){
+  /* Look for illegal byte-sequences and byte-sequences that require
+  ** escaping.  No control-characters are allowed.  All spaces and
+  ** non-ASCII unicode characters and some punctuation characters require
+  ** escaping. */
+  for(i=0; (c = (unsigned char)zIn[i])!=0; i++){
+    if( aSafeChar[c] ){
+      unsigned char x = aSafeChar[c];
       needEscape = 1;
-    }
+      if( x==2 ){
+        Blob bad;
+        blob_token(pBlob, &bad);
+        fossil_fatal("the [%s] argument to the \"%s\" command contains "
+                     "a character (ascii 0x%02x) that is not allowed in "
+                     "filename arguments",
+                     zIn, blob_str(&bad), c);
+      }else if( x>2 ){
+        if( (zIn[i+1]&0xc0)!=0x80
+         || (x>=4 && (zIn[i+2]&0xc0)!=0x80)
+         || (x==5 && (zIn[i+3]&0xc0)!=0x80)
+        ){
+          Blob bad;
+          blob_token(pBlob, &bad);
+          fossil_fatal("the [%s] argument to the \"%s\" command contains "
+                       "an illegal UTF-8 character",
+                       zIn, blob_str(&bad));
+        }
+        i += x-2;
+      }
+    } 
   }
+
+  /* Separate from the previous argument by a space */
   if( n>0 && !fossil_isspace(z[n-1]) ){
     blob_append_char(pBlob, ' ');
   }
-  if( needEscape ) blob_append_char(pBlob, cQuote);
-  if( zIn[0]=='-' ) blob_append(pBlob, "./", 2);
-  blob_append(pBlob, zIn, -1);
-  if( needEscape ) blob_append_char(pBlob, cQuote);
+
+  /* Check for characters that need quoting */
+  if( !needEscape ){
+    if( isFilename && zIn[0]=='-' ){
+      blob_append_char(pBlob, '.');
+#if defined(_WIN32)
+      blob_append_char(pBlob, '\\');
+#else
+      blob_append_char(pBlob, '/');
+#endif
+    }
+    blob_append(pBlob, zIn, -1);
+  }else{
+#if defined(_WIN32)
+    /* Quoting strategy for windows:
+    ** Put the entire name inside of "...".  Any " characters within
+    ** the name get doubled.
+    */
+    blob_append_char(pBlob, '"');
+    if( isFilename && zIn[0]=='-' ){
+      blob_append_char(pBlob, '.');
+      blob_append_char(pBlob, '\\');
+    }else if( zIn[0]=='/' ){
+      blob_append_char(pBlob, '.');
+    }
+    for(i=0; (c = (unsigned char)zIn[i])!=0; i++){
+      blob_append_char(pBlob, (char)c);
+      if( c=='"' ) blob_append_char(pBlob, '"');
+    }
+    blob_append_char(pBlob, '"');
+#else
+    /* Quoting strategy for unix:
+    ** If the name does not contain ', then surround the whole thing
+    ** with '...'.   If there is one or more ' characters within the
+    ** name, then put \ before each special character.
+    */
+    if( strchr(zIn,'\'') ){
+      if( isFilename && zIn[0]=='-' ){
+        blob_append_char(pBlob, '.');
+        blob_append_char(pBlob, '/');
+      }
+      for(i=0; (c = (unsigned char)zIn[i])!=0; i++){
+        if( aSafeChar[c] && aSafeChar[c]!=2 ) blob_append_char(pBlob, '\\');
+        blob_append_char(pBlob, (char)c);
+      }
+    }else{
+      blob_append_char(pBlob, '\'');
+      if( isFilename && zIn[0]=='-' ){
+        blob_append_char(pBlob, '.');
+        blob_append_char(pBlob, '/');
+      }
+      blob_append(pBlob, zIn, -1);
+      blob_append_char(pBlob, '\'');
+    }
+#endif
+  }
+}
+
+/*
+** COMMAND: test-escaped-arg
+**
+** Usage %fossil ARGS ...
+**
+** Run each argument through blob_append_escaped_arg() and show the
+** result.  Append each argument to "fossil test-echo" and run that
+** using fossil_system() to verify that it really does get escaped
+** correctly.
+**
+** Other options:
+**
+**    --filename-args BOOL      Subsequent arguments are assumed to be
+**                              filenames if BOOL is true, or not if BOOL
+**                              is false.  Defaults on.
+**
+**    --hex HEX                 Skip the --hex flag and instead decode HEX
+**                              into ascii.  This provides a way to insert
+**                              unusual characters as an argument for testing.
+**
+**    --compare HEX ASCII       Verify that argument ASCII is identical to
+**                              to decoded HEX.
+**
+**    --fuzz N                  Run N fuzz cases.  Each cases is a call
+**                              to "fossil test-escaped-arg --compare HEX ARG"
+**                              where HEX and ARG are the same argument.
+**                              The argument is chosen at random.
+*/
+void test_escaped_arg_command(void){
+  int i;
+  Blob x;
+  const char *zArg;
+  int isFilename = 1;
+  char zBuf[100];
+  blob_init(&x, 0, 0);
+  for(i=2; i<g.argc; i++){
+    zArg = g.argv[i];
+    if( fossil_strcmp(zArg, "--hex")==0 && i+1<g.argc ){
+      size_t n = strlen(g.argv[++i]);
+      if( n>=(sizeof(zBuf)-1)*2 ){
+        fossil_fatal("Argument to --hex is too big");
+      }
+      memset(zBuf, 0, sizeof(zBuf));
+      decode16((const unsigned char*)g.argv[i], (unsigned char*)zBuf, (int)n);
+      zArg = zBuf;
+    }else if( fossil_strcmp(zArg, "--compare")==0 && i+2<g.argc ){
+      size_t n = strlen(g.argv[++i]);
+      if( n>=(sizeof(zBuf)-1)*2 ){
+        fossil_fatal("HEX argument to --compare is too big");
+      }
+      memset(zBuf, 0, sizeof(zBuf));
+      if( decode16((const unsigned char*)g.argv[i], (unsigned char*)zBuf,
+                   (int)n) ){
+        fossil_fatal("HEX decode of %s failed", g.argv[i]);
+      }
+      zArg = g.argv[++i];
+      if( zArg[0]=='-' ){
+        fossil_fatal("filename argument \"%s\" begins with \"-\"", zArg);
+      }
+#ifdef _WIN32
+      if( zBuf[0]=='-' && zArg[0]=='.' && zArg[1]=='\\' ) zArg += 2;
+#else
+      if( zBuf[0]=='-' && zArg[0]=='.' && zArg[1]=='/' ) zArg += 2;
+#endif
+      if( strcmp(zBuf, zArg)!=0 ){
+        fossil_fatal("argument disagree: \"%s\" (%s) versus \"%s\"", 
+                     zBuf, g.argv[i-1], zArg);
+      }
+      continue;
+    }else if( fossil_strcmp(zArg, "--fuzz")==0 && i+1<g.argc ){
+      int n = atoi(g.argv[++i]);
+      int j;
+      for(j=0; j<n; j++){
+        unsigned char m, k;
+        int rc;
+        unsigned char zWord[100];
+        sqlite3_randomness(sizeof(m), &m);
+        m = (m%40)+5;
+        sqlite3_randomness(m, zWord); /* Between 5 and 45 bytes of randomness */
+        for(k=0; k<m; k++){
+          unsigned char cx = zWord[k];
+          if( cx<0x20 || cx>=0x7f ){
+            /* Translate illegal bytes into various non-ASCII unicode
+            ** characters in order to exercise those code paths */
+            unsigned int u;
+            if( cx>=0x7f ){
+              u = cx;
+            }else if( cx>=0x08 ){
+              u = 0x800 + cx;
+            }else{
+              u = 0x10000 + cx;
+            }
+            if( u<0x00080 ){
+              zWord[k] = u & 0xFF;
+            }else if( u<0x00800 ){
+              zWord[k++] = 0xC0 + (u8)((u>>6)&0x1F);
+              zWord[k] =   0x80 + (u8)(u & 0x3F);
+            }else if( u<0x10000 ){
+              zWord[k++] = 0xE0 + (u8)((u>>12)&0x0F);
+              zWord[k++] = 0x80 + (u8)((u>>6) & 0x3F);
+              zWord[k] =   0x80 + (u8)(u & 0x3F);
+            }else{
+              zWord[k++] = 0xF0 + (u8)((u>>18) & 0x07);
+              zWord[k++] = 0x80 + (u8)((u>>12) & 0x3F);
+              zWord[k++] = 0x80 + (u8)((u>>6) & 0x3F);
+              zWord[k]   = 0x80 + (u8)(u & 0x3F);
+            }
+          }
+        }
+        zWord[k] = 0;
+        encode16(zWord, (unsigned char*)zBuf, (int)k);
+        blob_appendf(&x, "%$ test-escaped-arg --compare %s %$",
+                         g.nameOfExe, zBuf,zWord);
+        rc = fossil_system(blob_str(&x));
+        if( rc ) fossil_fatal("failed test (%d): %s\n", rc, blob_str(&x));
+        blob_reset(&x);
+      }
+      continue;
+    }else if( fossil_strcmp(zArg, "--filename-args")==0 ){
+       if( i+1<g.argc ){
+         i++;
+         isFilename = is_truth(g.argv[i]);
+       }
+       continue;
+    }
+    fossil_print("%3d [%s]: ", i, zArg);
+    if( isFilename ){
+      blob_appendf(&x, "%$ test-echo %$", g.nameOfExe, zArg);
+    }else{
+      blob_appendf(&x, "%$ test-echo %!$", g.nameOfExe, zArg);
+    }
+    fossil_print("%s\n", blob_str(&x));
+    fossil_system(blob_str(&x));
+    blob_reset(&x);
+  }
 }
 
 /*
@@ -1337,7 +1764,7 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
     if( bomReverse ){
       /* Found BOM, but with reversed bytes */
       unsigned int i = blob_size(pBlob);
-      while( i>0 ){
+      while( i>1 ){
         /* swap bytes of unicode representation */
         char zTemp = zUtf8[--i];
         zUtf8[i] = zUtf8[i-1];
@@ -1345,9 +1772,10 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
       }
     }
     /* Make sure the blob contains two terminating 0-bytes */
-    blob_append_char(pBlob, 0);
+    blob_append(pBlob, "\000\000", 3);
     zUtf8 = blob_str(pBlob) + bomSize;
     zUtf8 = fossil_unicode_to_utf8(zUtf8);
+    blob_reset(pBlob);
     blob_set_dynamic(pBlob, zUtf8);
   }else if( useMbcs && invalid_utf8(pBlob) ){
 #if defined(_WIN32) || defined(__CYGWIN__)
