@@ -15,11 +15,16 @@
 **
 *******************************************************************************
 **
-** This file contains C functions and procedures that provide useful
-** services to CGI programs.  There are procedures for parsing and
-** dispensing QUERY_STRING parameters and cookies, the "mprintf()"
-** formatting function and its cousins, and routines to encode and
-** decode strings in HTML or HTTP.
+** This file contains C functions and procedures used by CGI programs
+** (Fossil launched as CGI) to interpret CGI environment variables,
+** gather the results, and send they reply back to the CGI server.
+** This file also contains routines for running a simple web-server
+** (the "fossil ui" or "fossil server" command) and launching subprocesses
+** to handle each inbound HTTP request using CGI.
+**
+** This file contains routines used by Fossil when it is acting as a
+** CGI client.  For the code used by Fossil when it is acting as a
+** CGI server (for the /ext webpage) see the "extcgi.c" source file.
 */
 #include "config.h"
 #ifdef _WIN32
@@ -61,6 +66,17 @@
 #define PCK(x)      cgi_parameter_checked(x,1)
 #define PIF(x,y)    cgi_parameter_checked(x,y)
 
+/*
+** Shortcut for the cgi_printf() routine.  Instead of using the
+**
+**    @ ...
+**
+** notation provided by the translate.c utility, you can also
+** optionally use:
+**
+**    CX(...)
+*/
+#define CX cgi_printf
 
 /*
 ** Destinations for output text.
@@ -142,6 +158,13 @@ Blob *cgi_output_blob(void){
   return pContent;
 }
 
+/*
+** Return complete text of the output header
+*/
+const char *cgi_header(void){
+  return blob_str(&cgiContent[0]);
+}
+
 /*
 ** Combine the header and body of the CGI into a single string.
 */
@@ -164,16 +187,18 @@ char *cgi_extract_content(void){
 /*
 ** Additional information used to form the HTTP reply
 */
-static char *zContentType = "text/html";     /* Content type of the reply */
-static char *zReplyStatus = "OK";            /* Reply status description */
+static const char *zContentType = "text/html";   /* Content type of the reply */
+static const char *zReplyStatus = "OK";          /* Reply status description */
 static int iReplyStatus = 200;               /* Reply status code */
 static Blob extraHeader = BLOB_INITIALIZER;  /* Extra header text */
+static int rangeStart = 0;                   /* Start of Range: */
+static int rangeEnd = 0;                     /* End of Range: plus 1 */
 
 /*
 ** Set the reply content type
 */
 void cgi_set_content_type(const char *zType){
-  zContentType = mprintf("%s", zType);
+  zContentType = fossil_strdup(zType);
 }
 
 /*
@@ -190,7 +215,7 @@ void cgi_set_content(Blob *pNewContent){
 ** Set the reply status code
 */
 void cgi_set_status(int iStat, const char *zStat){
-  zReplyStatus = mprintf("%s", zStat);
+  zReplyStatus = fossil_strdup(zStat);
   iReplyStatus = iStat;
 }
 
@@ -200,12 +225,19 @@ void cgi_set_status(int iStat, const char *zStat){
 void cgi_append_header(const char *zLine){
   blob_append(&extraHeader, zLine, -1);
 }
+void cgi_printf_header(const char *zLine, ...){
+  va_list ap;
+  va_start(ap, zLine);
+  blob_vappendf(&extraHeader, zLine, ap);
+  va_end(ap);
+}
 
 /*
 ** Set a cookie by queuing up the appropriate HTTP header output. If
 ** !g.isHTTP, this is a no-op.
 **
-** Zero lifetime implies a session cookie.
+** Zero lifetime implies a session cookie. A negative one expires
+** the cookie immediately.
 */
 void cgi_set_cookie(
   const char *zName,    /* Name of the cookie */
@@ -222,13 +254,15 @@ void cgi_set_cookie(
   if( g.zBaseURL!=0 && strncmp(g.zBaseURL, "https:", 6)==0 ){
     zSecure = " secure;";
   }
-  if( lifetime>0 ){
+  if( lifetime!=0 ){
     blob_appendf(&extraHeader,
-       "Set-Cookie: %s=%t; Path=%s; max-age=%d; HttpOnly;%s Version=1\r\n",
-        zName, zValue, zPath, lifetime, zSecure);
+       "Set-Cookie: %s=%t; Path=%s; max-age=%d; HttpOnly; "
+       "%s Version=1\r\n",
+       zName, lifetime>0 ? zValue : "null", zPath, lifetime, zSecure);
   }else{
     blob_appendf(&extraHeader,
-       "Set-Cookie: %s=%t; Path=%s; HttpOnly;%s Version=1\r\n",
+       "Set-Cookie: %s=%t; Path=%s; HttpOnly; "
+       "%s Version=1\r\n",
        zName, zValue, zPath, zSecure);
   }
 }
@@ -256,27 +290,35 @@ void cgi_reply(void){
   }
 
   if( g.fullHttpReply ){
+    if( rangeEnd>0
+     && iReplyStatus==200 
+     && fossil_strcmp(P("REQUEST_METHOD"),"GET")==0
+    ){
+      iReplyStatus = 206;
+      zReplyStatus = "Partial Content";
+    }
     fprintf(g.httpOut, "HTTP/1.0 %d %s\r\n", iReplyStatus, zReplyStatus);
     fprintf(g.httpOut, "Date: %s\r\n", cgi_rfc822_datestamp(time(0)));
     fprintf(g.httpOut, "Connection: close\r\n");
     fprintf(g.httpOut, "X-UA-Compatible: IE=edge\r\n");
   }else{
+    assert( rangeEnd==0 );
     fprintf(g.httpOut, "Status: %d %s\r\n", iReplyStatus, zReplyStatus);
   }
-  if( g.isConst ){
-    /* isConst means that the reply is guaranteed to be invariant, even
-    ** after configuration changes and/or Fossil binary recompiles. */
-    fprintf(g.httpOut, "Cache-Control: max-age=31536000\r\n");
-  }else if( etag_tag()!=0 ){
+  if( etag_tag()[0]!=0 ){
     fprintf(g.httpOut, "ETag: %s\r\n", etag_tag());
     fprintf(g.httpOut, "Cache-Control: max-age=%d\r\n", etag_maxage());
+    if( etag_mtime()>0 ){
+      fprintf(g.httpOut, "Last-Modified: %s\r\n",
+              cgi_rfc822_datestamp(etag_mtime()));
+    }
+  }else if( g.isConst ){
+    /* isConst means that the reply is guaranteed to be invariant, even
+    ** after configuration changes and/or Fossil binary recompiles. */
+    fprintf(g.httpOut, "Cache-Control: max-age=315360000, immutable\r\n");
   }else{
     fprintf(g.httpOut, "Cache-control: no-cache\r\n");
   }
-  if( etag_mtime()>0 ){
-    fprintf(g.httpOut, "Last-Modified: %s\r\n",
-            cgi_rfc822_datestamp(etag_mtime()));
-  }
 
   if( blob_size(&extraHeader)>0 ){
     fprintf(g.httpOut, "%s", blob_buffer(&extraHeader));
@@ -303,14 +345,14 @@ void cgi_reply(void){
   /* Content intended for logged in users should only be cached in
   ** the browser, not some shared location.
   */
-  fprintf(g.httpOut, "Content-Type: %s; charset=utf-8\r\n", zContentType);
-  if( fossil_strcmp(zContentType,"application/x-fossil")==0 ){
-    cgi_combine_header_and_body();
-    blob_compress(&cgiContent[0], &cgiContent[0]);
-  }
+  if( iReplyStatus!=304 ) {
+    fprintf(g.httpOut, "Content-Type: %s; charset=utf-8\r\n", zContentType);
+    if( fossil_strcmp(zContentType,"application/x-fossil")==0 ){
+      cgi_combine_header_and_body();
+      blob_compress(&cgiContent[0], &cgiContent[0]);
+    }
 
-  if( iReplyStatus != 304 ) {
-    if( is_gzippable() ){
+    if( is_gzippable() && iReplyStatus!=206 ){
       int i;
       gzip_begin(0);
       for( i=0; i<2; i++ ){
@@ -323,24 +365,38 @@ void cgi_reply(void){
       fprintf(g.httpOut, "Vary: Accept-Encoding\r\n");
     }
     total_size = blob_size(&cgiContent[0]) + blob_size(&cgiContent[1]);
+    if( iReplyStatus==206 ){
+      fprintf(g.httpOut, "Content-Range: bytes %d-%d/%d\r\n",
+              rangeStart, rangeEnd-1, total_size);
+      total_size = rangeEnd - rangeStart; 
+    }
     fprintf(g.httpOut, "Content-Length: %d\r\n", total_size);
   }else{
     total_size = 0;
   }
   fprintf(g.httpOut, "\r\n");
-  if( total_size>0 && iReplyStatus != 304
+  if( total_size>0
+   && iReplyStatus!=304
    && fossil_strcmp(P("REQUEST_METHOD"),"HEAD")!=0
   ){
     int i, size;
     for(i=0; i<2; i++){
       size = blob_size(&cgiContent[i]);
-      if( size>0 ){
-        fwrite(blob_buffer(&cgiContent[i]), 1, size, g.httpOut);
+      if( size<=rangeStart ){
+        rangeStart -= size;
+      }else{
+        int n = size - rangeStart;
+        if( n>total_size ){
+          n = total_size;
+        }
+        fwrite(blob_buffer(&cgiContent[i])+rangeStart, 1, n, g.httpOut);
+        rangeStart = 0;
+        total_size -= n;
       }
     }
   }
   fflush(g.httpOut);
-  CGIDEBUG(("DONE\n"));
+  CGIDEBUG(("-------- END cgi ---------\n"));
 
   /* After the webpage has been sent, do any useful background
   ** processing.
@@ -356,7 +412,7 @@ void cgi_reply(void){
 **
 ** The URL must be relative to the base of the fossil server.
 */
-NORETURN static void cgi_redirect_with_status(
+NORETURN void cgi_redirect_with_status(
   const char *zURL,
   int iStat,
   const char *zStat
@@ -394,6 +450,27 @@ NORETURN void cgi_redirectf(const char *zFormat, ...){
   va_end(ap);
 }
 
+/*
+** Add a "Content-disposition: attachment; filename=%s" header to the reply.
+*/
+void cgi_content_disposition_filename(const char *zFilename){
+  char *z;
+  int i, n;
+
+           /*  0123456789 123456789 123456789 123456789 123456*/
+  z = mprintf("Content-Disposition: attachment; filename=\"%s\";\r\n",
+                    file_tail(zFilename));
+  n = (int)strlen(z);
+  for(i=43; i<n-4; i++){
+    char c = z[i];
+    if( fossil_isalnum(c) ) continue;
+    if( c=='.' || c=='-' || c=='/' ) continue;
+    z[i] = '_';
+  }
+  cgi_append_header(z);
+  fossil_free(z);
+}
+
 /*
 ** Return the URL for the caller.  This is obtained from either the
 ** referer CGI parameter, if it exists, or the HTTP_REFERER HTTP parameter.
@@ -476,6 +553,27 @@ void cgi_set_parameter_nocopy(const char *zName, const char *zValue, int isQP){
   sortQP = 1;
 }
 
+/*
+** Add another query parameter or cookie to the parameter set.
+** zName is the name of the query parameter or cookie and zValue
+** is its fully decoded value.  zName will be modified to be an
+** all lowercase string.
+**
+** zName and zValue are not copied and must not change or be
+** deallocated after this routine returns.  This routine changes
+** all ASCII alphabetic characters in zName to lower case.  The
+** caller must not change them back.
+*/
+void cgi_set_parameter_nocopy_tolower(
+  char *zName,
+  const char *zValue,
+  int isQP
+){
+  int i;
+  for(i=0; zName[i]; i++){ zName[i] = fossil_tolower(zName[i]); }
+  cgi_set_parameter_nocopy(zName, zValue, isQP);
+}
+
 /*
 ** Add another query parameter or cookie to the parameter set.
 ** zName is the name of the query parameter or cookie and zValue
@@ -484,10 +582,10 @@ void cgi_set_parameter_nocopy(const char *zName, const char *zValue, int isQP){
 ** Copies are made of both the zName and zValue parameters.
 */
 void cgi_set_parameter(const char *zName, const char *zValue){
-  cgi_set_parameter_nocopy(mprintf("%s",zName), mprintf("%s",zValue), 0);
+  cgi_set_parameter_nocopy(fossil_strdup(zName),fossil_strdup(zValue), 0);
 }
 void cgi_set_query_parameter(const char *zName, const char *zValue){
-  cgi_set_parameter_nocopy(mprintf("%s",zName), mprintf("%s",zValue), 1);
+  cgi_set_parameter_nocopy(fossil_strdup(zName),fossil_strdup(zValue), 1);
 }
 
 /*
@@ -514,6 +612,11 @@ void cgi_replace_query_parameter(const char *zName, const char *zValue){
   }
   cgi_set_parameter_nocopy(zName, zValue, 1);
 }
+void cgi_replace_query_parameter_tolower(char *zName, const char *zValue){
+  int i;
+  for(i=0; zName[i]; i++){ zName[i] = fossil_tolower(zName[i]); }
+  cgi_replace_query_parameter(zName, zValue);
+}
 
 /*
 ** Delete a parameter.
@@ -549,10 +652,9 @@ void cgi_delete_query_parameter(const char *zName){
 ** must be made of zValue.
 */
 void cgi_setenv(const char *zName, const char *zValue){
-  cgi_set_parameter_nocopy(zName, mprintf("%s",zValue), 0);
+  cgi_set_parameter_nocopy(zName, fossil_strdup(zValue), 0);
 }
 
-
 /*
 ** Add a list of query parameters or cookies to the parameter set.
 **
@@ -605,8 +707,12 @@ static void add_param_list(char *z, int terminator){
       if( *z ){ *z++ = 0; }
       zValue = "";
     }
-    if( fossil_islower(zName[0]) && fossil_no_strange_characters(zName+1) ){
-      cgi_set_parameter_nocopy(zName, zValue, isQP);
+    if( zName[0] && fossil_no_strange_characters(zName+1) ){
+      if( fossil_islower(zName[0]) ){
+        cgi_set_parameter_nocopy(zName, zValue, isQP);
+      }else if( fossil_isupper(zName[0]) ){
+        cgi_set_parameter_nocopy_tolower(zName, zValue, isQP);
+      }
     }
 #ifdef FOSSIL_ENABLE_JSON
     json_setenv( zName, cson_value_new_string(zValue,strlen(zValue)) );
@@ -643,32 +749,32 @@ static char *get_line_from_string(char **pz, int *pLen){
 
 /*
 ** The input *pz points to content that is terminated by a "\r\n"
-** followed by the boundry marker zBoundry.  An extra "--" may or
-** may not be appended to the boundry marker.  There are *pLen characters
+** followed by the boundary marker zBoundary.  An extra "--" may or
+** may not be appended to the boundary marker.  There are *pLen characters
 ** in *pz.
 **
 ** This routine adds a "\000" to the end of the content (overwriting
 ** the "\r\n") and returns a pointer to the content.  The *pz input
-** is adjusted to point to the first line following the boundry.
+** is adjusted to point to the first line following the boundary.
 ** The length of the content is stored in *pnContent.
 */
 static char *get_bounded_content(
   char **pz,         /* Content taken from here */
   int *pLen,         /* Number of bytes of data in (*pz)[] */
-  char *zBoundry,    /* Boundry text marking the end of content */
+  char *zBoundary,    /* Boundary text marking the end of content */
   int *pnContent     /* Write the size of the content here */
 ){
   char *z = *pz;
   int len = *pLen;
   int i;
-  int nBoundry = strlen(zBoundry);
+  int nBoundary = strlen(zBoundary);
   *pnContent = len;
   for(i=0; i<len; i++){
-    if( z[i]=='\n' && strncmp(zBoundry, &z[i+1], nBoundry)==0 ){
+    if( z[i]=='\n' && strncmp(zBoundary, &z[i+1], nBoundary)==0 ){
       if( i>0 && z[i-1]=='\r' ) i--;
       z[i] = 0;
       *pnContent = i;
-      i += nBoundry;
+      i += nBoundary;
       break;
     }
   }
@@ -737,23 +843,31 @@ static int tokenize_line(char *z, int mxArg, char **azArg){
 static void process_multipart_form_data(char *z, int len){
   char *zLine;
   int nArg, i;
-  char *zBoundry;
+  char *zBoundary;
   char *zValue;
   char *zName = 0;
   int showBytes = 0;
   char *azArg[50];
 
-  zBoundry = get_line_from_string(&z, &len);
-  if( zBoundry==0 ) return;
+  zBoundary = get_line_from_string(&z, &len);
+  if( zBoundary==0 ) return;
   while( (zLine = get_line_from_string(&z, &len))!=0 ){
     if( zLine[0]==0 ){
       int nContent = 0;
-      zValue = get_bounded_content(&z, &len, zBoundry, &nContent);
-      if( zName && zValue && fossil_islower(zName[0]) ){
-        cgi_set_parameter_nocopy(zName, zValue, 1);
-        if( showBytes ){
-          cgi_set_parameter_nocopy(mprintf("%s:bytes", zName),
-               mprintf("%d",nContent), 1);
+      zValue = get_bounded_content(&z, &len, zBoundary, &nContent);
+      if( zName && zValue ){
+        if( fossil_islower(zName[0]) ){
+          cgi_set_parameter_nocopy(zName, zValue, 1);
+          if( showBytes ){
+            cgi_set_parameter_nocopy(mprintf("%s:bytes", zName),
+                 mprintf("%d",nContent), 1);
+          }
+        }else if( fossil_isupper(zName[0]) ){
+          cgi_set_parameter_nocopy_tolower(zName, zValue, 1);
+          if( showBytes ){
+            cgi_set_parameter_nocopy_tolower(mprintf("%s:bytes", zName),
+                 mprintf("%d",nContent), 1);
+          }
         }
       }
       zName = 0;
@@ -769,14 +883,24 @@ static void process_multipart_form_data(char *z, int len){
           zName = azArg[++i];
         }else if( c=='f' && sqlite3_strnicmp(azArg[i],"filename=",n)==0 ){
           char *z = azArg[++i];
-          if( zName && z && fossil_islower(zName[0]) ){
-            cgi_set_parameter_nocopy(mprintf("%s:filename",zName), z, 1);
+          if( zName && z ){
+            if( fossil_islower(zName[0]) ){
+              cgi_set_parameter_nocopy(mprintf("%s:filename",zName), z, 1);
+            }else if( fossil_isupper(zName[0]) ){
+              cgi_set_parameter_nocopy_tolower(mprintf("%s:filename",zName),
+                                               z, 1);
+            }
           }
           showBytes = 1;
         }else if( c=='c' && sqlite3_strnicmp(azArg[i],"content-type:",n)==0 ){
           char *z = azArg[++i];
-          if( zName && z && fossil_islower(zName[0]) ){
-            cgi_set_parameter_nocopy(mprintf("%s:mimetype",zName), z, 1);
+          if( zName && z ){
+            if( fossil_islower(zName[0]) ){
+              cgi_set_parameter_nocopy(mprintf("%s:mimetype",zName), z, 1);
+            }else if( fossil_isupper(zName[0]) ){
+              cgi_set_parameter_nocopy_tolower(mprintf("%s:mimetype",zName),
+                                               z, 1);
+            }
           }
         }
       }
@@ -839,7 +963,7 @@ void cgi_parse_POST_JSON( FILE * zIn, unsigned int contentLen ){
   CgiPostReadState state;
   cson_parse_opt popt = cson_parse_opt_empty;
   cson_parse_info pinfo = cson_parse_info_empty;
-  assert(g.json.gc.a && "json_main_bootstrap() was not called!");
+  assert(g.json.gc.a && "json_bootstrap_early() was not called!");
   popt.maxDepth = 15;
   state.fh = zIn;
   state.len = contentLen;
@@ -929,23 +1053,73 @@ static NORETURN void malformed_request(const char *zMsg);
 ** assume that PATH_INFO is an empty string and set REQUEST_URI equal
 ** to PATH_INFO.
 **
+** Sometimes PATH_INFO is missing and SCRIPT_NAME is not a prefix of
+** REQUEST_URI.  (See https://fossil-scm.org/forum/forumpost/049e8650ed)
+** In that case, truncate SCRIPT_NAME so that it is a proper prefix
+** of REQUEST_URI.
+**
 ** SCGI typically omits PATH_INFO.  CGI sometimes omits REQUEST_URI and
 ** PATH_INFO when it is empty.
+**
+** CGI Parameter quick reference:
+**
+**                                      REQUEST_URI
+**                               _____________|____________
+**                              /                          \
+**    https://www.fossil-scm.org/forum/info/12736b30c072551a?t=c
+**            \________________/\____/\____________________/ \_/
+**                    |            |             |            |
+**               HTTP_HOST         |        PATH_INFO     QUERY_STRING
+**                            SCRIPT_NAME
 */
 void cgi_init(void){
   char *z;
   const char *zType;
+  char *zSemi;
   int len;
   const char *zRequestUri = cgi_parameter("REQUEST_URI",0);
   const char *zScriptName = cgi_parameter("SCRIPT_NAME",0);
   const char *zPathInfo = cgi_parameter("PATH_INFO",0);
+#ifdef _WIN32
+  const char *zServerSoftware = cgi_parameter("SERVER_SOFTWARE",0);
+#endif
 
 #ifdef FOSSIL_ENABLE_JSON
-  json_main_bootstrap();
+  const int noJson = P("no_json")!=0;
 #endif
   g.isHTTP = 1;
   cgi_destination(CGI_BODY);
-  if( zScriptName==0 ) malformed_request("missing SCRIPT_NAME");
+
+  /* We must have SCRIPT_NAME. If the web server did not supply it, try
+  ** to compute it from REQUEST_URI and PATH_INFO. */
+  if( zScriptName==0 ){
+    size_t nRU, nPI;
+    if( zRequestUri==0 || zPathInfo==0 ){
+      malformed_request("missing SCRIPT_NAME");  /* Does not return */
+    }
+    nRU = strlen(zRequestUri);
+    nPI = strlen(zPathInfo);
+    if( nRU<nPI ){
+      malformed_request("PATH_INFO is longer than REQUEST_URI");
+    }
+    zScriptName = fossil_strndup(zRequestUri,(int)(nRU-nPI));
+    cgi_set_parameter("SCRIPT_NAME", zScriptName);
+  }
+
+#ifdef _WIN32
+  /* The Microsoft IIS web server does not define REQUEST_URI, instead it uses
+  ** PATH_INFO for virtually the same purpose.  Define REQUEST_URI the same as
+  ** PATH_INFO and redefine PATH_INFO with SCRIPT_NAME removed from the 
+  ** beginning. */
+  if( zServerSoftware && strstr(zServerSoftware, "Microsoft-IIS") ){
+    int i, j;
+    cgi_set_parameter("REQUEST_URI", zPathInfo);
+    for(i=0; zPathInfo[i]==zScriptName[i] && zPathInfo[i]; i++){}
+    for(j=i; zPathInfo[j] && zPathInfo[j]!='?'; j++){}
+    zPathInfo = fossil_strndup(zPathInfo+i, j-i);
+    cgi_replace_parameter("PATH_INFO", zPathInfo);
+  }
+#endif
   if( zRequestUri==0 ){
     const char *z = zPathInfo;
     if( zPathInfo==0 ){
@@ -959,75 +1133,111 @@ void cgi_init(void){
     int i, j;
     for(i=0; zRequestUri[i]==zScriptName[i] && zRequestUri[i]; i++){}
     for(j=i; zRequestUri[j] && zRequestUri[j]!='?'; j++){}
-    cgi_set_parameter("PATH_INFO", mprintf("%.*s", j-i, zRequestUri+i));
+    zPathInfo = fossil_strndup(zRequestUri+i, j-i);
+    cgi_set_parameter_nocopy("PATH_INFO", zPathInfo, 0);
+    if( j>i && zScriptName[i]!=0 ){
+      /* If SCRIPT_NAME is not a prefix of REQUEST_URI, truncate it so
+      ** that it is.  See https://fossil-scm.org/forum/forumpost/049e8650ed
+      */
+      char *zNew = fossil_strndup(zScriptName, i);
+      cgi_replace_parameter("SCRIPT_NAME", zNew);
+    }
   }
-
+#ifdef FOSSIL_ENABLE_JSON
+  if(noJson==0 && json_request_is_json_api(zPathInfo)){
+    /* We need to change some following behaviour depending on whether
+    ** we are operating in JSON mode or not. We cannot, however, be
+    ** certain whether we should/need to be in JSON mode until the
+    ** PATH_INFO is set up.
+    */
+    g.json.isJsonMode = 1;
+    json_bootstrap_early();
+  }else{
+    assert(!g.json.isJsonMode &&
+           "Internal misconfiguration of g.json.isJsonMode");
+  }
+#endif
   z = (char*)P("HTTP_COOKIE");
   if( z ){
-    z = mprintf("%s",z);
+    z = fossil_strdup(z);
     add_param_list(z, ';');
+    z = (char*)cookie_value("skin",0);
+    if(z){
+      skin_use_alternative(z, 2);
+    }
   }
 
   z = (char*)P("QUERY_STRING");
   if( z ){
-    z = mprintf("%s",z);
+    z = fossil_strdup(z);
     add_param_list(z, '&');
+    z = (char*)P("skin");
+    if(z){
+      char *zErr = skin_use_alternative(z, 2);
+      if(!zErr && !P("once")){
+        cookie_write_parameter("skin","skin",z);
+      }
+      fossil_free(zErr);
+    }
   }
 
   z = (char*)P("REMOTE_ADDR");
   if( z ){
-    g.zIpAddr = mprintf("%s", z);
+    g.zIpAddr = fossil_strdup(z);
   }
 
   len = atoi(PD("CONTENT_LENGTH", "0"));
-  g.zContentType = zType = P("CONTENT_TYPE");
+  zType = P("CONTENT_TYPE");
+  zSemi = zType ? strchr(zType, ';') : 0;
+  if( zSemi ){
+    g.zContentType = fossil_strndup(zType, (int)(zSemi-zType));
+    zType = g.zContentType;
+  }else{
+    g.zContentType = zType;
+  }
   blob_zero(&g.cgiIn);
   if( len>0 && zType ){
-    if( fossil_strcmp(zType,"application/x-www-form-urlencoded")==0
-         || strncmp(zType,"multipart/form-data",19)==0 ){
-      z = fossil_malloc( len+1 );
-      len = fread(z, 1, len, g.httpIn);
-      z[len] = 0;
-      cgi_trace(z);
-      if( zType[0]=='a' ){
-        add_param_list(z, '&');
-      }else{
-        process_multipart_form_data(z, len);
-      }
-    }else if( fossil_strcmp(zType, "application/x-fossil")==0 ){
+    if( fossil_strcmp(zType, "application/x-fossil")==0 ){
       blob_read_from_channel(&g.cgiIn, g.httpIn, len);
       blob_uncompress(&g.cgiIn, &g.cgiIn);
-    }else if( fossil_strcmp(zType, "application/x-fossil-debug")==0 ){
-      blob_read_from_channel(&g.cgiIn, g.httpIn, len);
-    }else if( fossil_strcmp(zType, "application/x-fossil-uncompressed")==0 ){
-      blob_read_from_channel(&g.cgiIn, g.httpIn, len);
     }
 #ifdef FOSSIL_ENABLE_JSON
-    else if( fossil_strcmp(zType, "application/json")
-              || fossil_strcmp(zType,"text/plain")/*assume this MIGHT be JSON*/
-              || fossil_strcmp(zType,"application/javascript")){
-      g.json.isJsonMode = 1;
+    else if( noJson==0 && g.json.isJsonMode!=0 
+             && json_can_consume_content_type(zType)!=0 ){
       cgi_parse_POST_JSON(g.httpIn, (unsigned int)len);
-      /* FIXMEs:
+      /*
+       Potential TODOs:
 
-      - See if fossil really needs g.cgiIn to be set for this purpose
-      (i don't think it does). If it does then fill g.cgiIn and
-      refactor to parse the JSON from there.
-
-      - After parsing POST JSON, copy the "first layer" of keys/values
-      to cgi_setenv(), honoring the upper-case distinction used
-      in add_param_list(). However...
-
-      - If we do that then we might get a disconnect in precedence of
-      GET/POST arguments. i prefer for GET entries to take precedence
-      over like-named POST entries, but in order for that to happen we
-      need to process QUERY_STRING _after_ reading the POST data.
+       1) If parsing fails, immediately return an error response
+       without dispatching the ostensibly-upcoming JSON API.
       */
       cgi_set_content_type(json_guess_content_type());
     }
 #endif /* FOSSIL_ENABLE_JSON */
+    else{
+      blob_read_from_channel(&g.cgiIn, g.httpIn, len);
+    }
   }
+}
 
+/*
+** Decode POST parameter information in the cgiIn content, if any.
+*/
+void cgi_decode_post_parameters(void){
+  int len = blob_size(&g.cgiIn);
+  if( len==0 ) return;
+  if( fossil_strcmp(g.zContentType,"application/x-www-form-urlencoded")==0
+   || strncmp(g.zContentType,"multipart/form-data",19)==0
+  ){
+    char *z = blob_str(&g.cgiIn);
+    cgi_trace(z);
+    if( g.zContentType[0]=='a' ){
+      add_param_list(z, '&');
+    }else{
+      process_multipart_form_data(z, len);
+    }
+    blob_init(&g.cgiIn, 0, 0);
+  }
 }
 
 /*
@@ -1078,6 +1288,10 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
     nUsedQP = j;
   }
 
+  /* Invoking with a NULL zName is just a way to cause the parameters
+  ** to be sorted.  So go ahead and bail out in that case */
+  if( zName==0 || zName[0]==0 ) return 0;
+
   /* Do a binary search for a matching query parameter */
   lo = 0;
   hi = nUsedQP-1;
@@ -1098,7 +1312,7 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
   ** letter, then check to see if there is an environment variable
   ** with the given name.
   */
-  if( zName && fossil_isupper(zName[0]) ){
+  if( fossil_isupper(zName[0]) ){
     const char *zValue = fossil_getenv(zName);
     if( zValue ){
       cgi_set_parameter_nocopy(zName, zValue, 0);
@@ -1110,6 +1324,25 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
   return zDefault;
 }
 
+/*
+** Return the value of the first defined query parameter or cookie whose
+** name appears in the list of arguments.  Or if no parameter is found,
+** return NULL.
+*/
+const char *cgi_coalesce(const char *zName, ...){
+  va_list ap;
+  const char *z;
+  const char *zX;
+  if( zName==0 ) return 0;
+  z = cgi_parameter(zName, 0);
+  va_start(ap, zName);
+  while( z==0 && (zX = va_arg(ap,const char*))!=0 ){
+    z = cgi_parameter(zX, 0);
+  }
+  va_end(ap);
+  return z;
+}
+
 /*
 ** Return the value of a CGI parameter with leading and trailing
 ** spaces removed and with internal \r\n changed to just \n
@@ -1223,12 +1456,48 @@ int cgi_all(const char *z, ...){
 }
 
 /*
-** Print all query parameters on standard output.  Format the
-** parameters as HTML.  This is used for testing and debugging.
+** Load all relevant environment variables into the parameter buffer.
+** Invoke this routine prior to calling cgi_print_all() in order to see
+** the full CGI environment.  This routine intended for debugging purposes
+** only.
+*/
+void cgi_load_environment(void){
+  /* The following is a list of environment variables that Fossil considers
+  ** to be "relevant". */
+  static const char *const azCgiVars[] = {
+    "COMSPEC", "DOCUMENT_ROOT", "GATEWAY_INTERFACE", "SCGI",
+    "HTTP_ACCEPT", "HTTP_ACCEPT_CHARSET", "HTTP_ACCEPT_ENCODING",
+    "HTTP_ACCEPT_LANGUAGE", "HTTP_AUTHENICATION",
+    "HTTP_CONNECTION", "HTTP_HOST",
+    "HTTP_IF_NONE_MATCH", "HTTP_IF_MODIFIED_SINCE",
+    "HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
+    "QUERY_STRING", "REMOTE_ADDR", "REMOTE_PORT",
+    "REMOTE_USER", "REQUEST_METHOD", "REQUEST_SCHEME",
+    "REQUEST_URI", "SCRIPT_FILENAME", "SCRIPT_NAME", "SERVER_NAME",
+    "SERVER_PROTOCOL", "HOME", "FOSSIL_HOME", "USERNAME", "USER",
+    "FOSSIL_USER", "SQLITE_TMPDIR", "TMPDIR",
+    "TEMP", "TMP", "FOSSIL_VFS",
+    "FOSSIL_FORCE_TICKET_MODERATION", "FOSSIL_FORCE_WIKI_MODERATION",
+    "FOSSIL_TCL_PATH", "TH1_DELETE_INTERP", "TH1_ENABLE_DOCS",
+    "TH1_ENABLE_HOOKS", "TH1_ENABLE_TCL", "REMOTE_HOST",
+  };
+  int i;
+  for(i=0; i<count(azCgiVars); i++) (void)P(azCgiVars[i]);
+}
+
+/*
+** Print all query parameters on standard output.
+** This is used for testing and debugging.
 **
 ** Omit the values of the cookies unless showAll is true.
+**
+** The eDest parameter determines where the output is shown:
+**
+**     eDest==0:    Rendering as HTML into the CGI reply
+**     eDest==1:    Written to stderr
+**     eDest==2:    Written to cgi_debug
 */
-void cgi_print_all(int showAll, int onConsole){
+void cgi_print_all(int showAll, unsigned int eDest){
   int i;
   cgi_parameter("","");  /* Force the parameters into sorted order */
   for(i=0; i<nUsedQP; i++){
@@ -1237,14 +1506,46 @@ void cgi_print_all(int showAll, int onConsole){
       if( fossil_stricmp("HTTP_COOKIE",zName)==0 ) continue;
       if( fossil_strnicmp("fossil-",zName,7)==0 ) continue;
     }
-    if( onConsole ){
-      fossil_trace("%s = %s\n", zName, aParamQP[i].zValue);
-    }else{
-      cgi_printf("%h = %h  <br />\n", zName, aParamQP[i].zValue);
+    switch( eDest ){
+      case 0: {
+        cgi_printf("%h = %h  <br />\n", zName, aParamQP[i].zValue);
+        break;
+      }
+      case 1: {  
+        fossil_trace("%s = %s\n", zName, aParamQP[i].zValue);
+        break;
+      }
+      case 2: {
+        cgi_debug("%s = %s\n", zName, aParamQP[i].zValue);
+        break;
+      }
     }
   }
 }
 
+/*
+** Put information about the N-th parameter into arguments.
+** Return non-zero on success, and return 0 if there is no N-th parameter.
+*/
+int cgi_param_info(
+  int N,
+  const char **pzName,
+  const char **pzValue,
+  int *pbIsQP
+){
+  if( N>=0 && N<nUsedQP ){
+    *pzName = aParamQP[N].zName;
+    *pzValue = aParamQP[N].zValue;
+    *pbIsQP = aParamQP[N].isQP;
+    return 1;
+  }else{
+    *pzName = 0;
+    *pzValue = 0;
+    *pbIsQP = 0;
+    return 0;
+  }
+}
+
 /*
 ** Export all untagged query parameters (but not cookies or environment
 ** variables) as hidden values of a form.
@@ -1429,6 +1730,7 @@ char *cgi_remote_ip(int fd){
 void cgi_handle_http_request(const char *zIpAddr){
   char *z, *zToken;
   int i;
+  const char *zScheme = "http";
   char zLine[2000];     /* A single line of input. */
   g.fullHttpReply = 1;
   if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
@@ -1461,9 +1763,10 @@ void cgi_handle_http_request(const char *zIpAddr){
   }
   if( zIpAddr ){
     cgi_setenv("REMOTE_ADDR", zIpAddr);
-    g.zIpAddr = mprintf("%s", zIpAddr);
+    g.zIpAddr = fossil_strdup(zIpAddr);
   }
 
+
   /* Get all the optional fields that follow the first line.
   */
   while( fgets(zLine,sizeof(zLine),g.httpIn) ){
@@ -1491,8 +1794,13 @@ void cgi_handle_http_request(const char *zIpAddr){
       cgi_setenv("HTTP_COOKIE", zVal);
     }else if( fossil_strcmp(zFieldName,"https:")==0 ){
       cgi_setenv("HTTPS", zVal);
+      zScheme = "https";
     }else if( fossil_strcmp(zFieldName,"host:")==0 ){
+      char *z;
       cgi_setenv("HTTP_HOST", zVal);
+      z = strchr(zVal, ':');
+      if( z ) z[0] = 0;
+      cgi_setenv("SERVER_NAME", zVal);
     }else if( fossil_strcmp(zFieldName,"if-none-match:")==0 ){
       cgi_setenv("HTTP_IF_NONE_MATCH", zVal);
     }else if( fossil_strcmp(zFieldName,"if-modified-since:")==0 ){
@@ -1506,11 +1814,19 @@ void cgi_handle_http_request(const char *zIpAddr){
     }else if( fossil_strcmp(zFieldName,"x-forwarded-for:")==0 ){
       const char *zIpAddr = cgi_accept_forwarded_for(zVal);
       if( zIpAddr!=0 ){
-        g.zIpAddr = mprintf("%s", zIpAddr);
+        g.zIpAddr = fossil_strdup(zIpAddr);
         cgi_replace_parameter("REMOTE_ADDR", g.zIpAddr);
       }
+    }else if( fossil_strcmp(zFieldName,"range:")==0 ){
+      int x1 = 0;
+      int x2 = 0;
+      if( sscanf(zVal,"bytes=%d-%d",&x1,&x2)==2 && x1>=0 && x1<=x2 ){
+        rangeStart = x1;
+        rangeEnd = x2+1;
+      }
     }
   }
+  cgi_setenv("REQUEST_SCHEME",zScheme);
   cgi_init();
   cgi_trace(0);
 }
@@ -1532,13 +1848,16 @@ void cgi_handle_ssh_http_request(const char *zIpAddr){
   int i, content_length = 0;
   char zLine[2000];     /* A single line of input. */
 
+#ifdef FOSSIL_ENABLE_JSON
+  if( nCycles==0 ){ json_bootstrap_early(); }
+#endif
   if( zIpAddr ){
     if( nCycles==0 ){
       cgi_setenv("REMOTE_ADDR", zIpAddr);
-      g.zIpAddr = mprintf("%s", zIpAddr);
+      g.zIpAddr = fossil_strdup(zIpAddr);
     }
   }else{
-    fossil_panic("missing SSH IP address");
+    fossil_fatal("missing SSH IP address");
   }
   if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
     malformed_request("missing HTTP header");
@@ -1597,7 +1916,7 @@ void cgi_handle_ssh_http_request(const char *zIpAddr){
   if( nCycles==0 ){
     cgi_setenv("PATH_INFO", zToken);
   }else{
-    cgi_replace_parameter("PATH_INFO", mprintf("%s",zToken));
+    cgi_replace_parameter("PATH_INFO", fossil_strdup(zToken));
   }
 
   /* Get all the optional fields that follow the first line.
@@ -1619,7 +1938,7 @@ void cgi_handle_ssh_http_request(const char *zIpAddr){
     if( fossil_strcmp(zFieldName,"content-length:")==0 ){
       content_length = atoi(zVal);
     }else if( fossil_strcmp(zFieldName,"content-type:")==0 ){
-      g.zContentType = zType = mprintf("%s", zVal);
+      g.zContentType = zType = fossil_strdup(zVal);
     }else if( fossil_strcmp(zFieldName,"host:")==0 ){
       if( nCycles==0 ){
         cgi_setenv("HTTP_HOST", zVal);
@@ -1696,7 +2015,7 @@ char *cgi_handle_ssh_probes(char *zLine, int zSize, char *z, char *zToken){
   ** so return the command that was requested
   */
   g.fSshClient |= CGI_SSH_COMPAT;
-  return mprintf("%s", zToken);
+  return fossil_strdup(zToken);
 }
 
 /*
@@ -1886,7 +2205,7 @@ int cgi_http_server(
       }
     }else
 #endif
-    if( system(zBrowser)<0 ){
+    if( fossil_system(zBrowser)<0 ){
       fossil_warning("cannot start browser: %s\n", zBrowser);
     }
   }
@@ -1945,7 +2264,7 @@ int cgi_http_server(
         }
         nchildren--;
       }
-    }  
+    }
   }
   /* NOT REACHED */
   fossil_exit(1);
@@ -1986,6 +2305,26 @@ char *cgi_rfc822_datestamp(time_t now){
   }
 }
 
+/*
+** Returns an ISO8601-formatted time string suitable for debugging
+** purposes.
+**
+** The value returned is always a string obtained from mprintf() and must
+** be freed using fossil_free() to avoid a memory leak.
+*/
+char *cgi_iso8601_datestamp(void){
+  struct tm *pTm;
+  time_t now = time(0);
+  pTm = gmtime(&now);
+  if( pTm==0 ){
+    return mprintf("");
+  }else{
+    return mprintf("%04d-%02d-%02d %02d:%02d:%02d",
+                   pTm->tm_year+1900, pTm->tm_mon, pTm->tm_mday,
+                   pTm->tm_hour, pTm->tm_min, pTm->tm_sec);
+  }
+}
+
 /*
 ** Parse an RFC822-formatted timestamp as we'd expect from HTTP and return
 ** a Unix epoch time. <= zero is returned on failure.
@@ -2052,7 +2391,7 @@ const char *cgi_ssh_remote_addr(const char *zDefault){
   const char *zSshConn = fossil_getenv("SSH_CONNECTION");
 
   if( zSshConn && zSshConn[0] ){
-    char *zSshClient = mprintf("%s",zSshConn);
+    char *zSshClient = fossil_strdup(zSshConn);
     if( (zIndex = strchr(zSshClient,' '))!=0 ){
       zSshClient[zIndex-zSshClient] = '\0';
       return zSshClient;
@@ -2069,3 +2408,16 @@ int cgi_is_loopback(const char *zIpAddr){
          fossil_strcmp(zIpAddr, "::ffff:127.0.0.1")==0 ||
          fossil_strcmp(zIpAddr, "::1")==0;
 }
+
+/*
+** Return true if the HTTP request is likely to be from a small-screen
+** mobile device.
+**
+** The returned value is a guess.  Use it only for setting up defaults.
+*/
+int cgi_from_mobile(void){
+  const char *zAgent = P("HTTP_USER_AGENT");
+  if( zAgent==0 ) return 0;
+  if( sqlite3_strglob("*iPad*", zAgent)==0 ) return 0;
+  return sqlite3_strlike("%mobile%", zAgent, 0)==0;
+}
