@@ -39,7 +39,8 @@
 #define CONFIGSET_XFER      0x000080     /* Transfer configuration */
 #define CONFIGSET_ALIAS     0x000100     /* URL Aliases */
 #define CONFIGSET_SCRIBER   0x000200     /* Email subscribers */
-#define CONFIGSET_ALL       0x0003ff     /* Everything */
+#define CONFIGSET_IWIKI     0x000400     /* Interwiki codes */
+#define CONFIGSET_ALL       0x0007ff     /* Everything */
 
 #define CONFIGSET_OVERWRITE 0x100000     /* Causes overwrite instead of merge */
 
@@ -60,18 +61,19 @@ static struct {
   int groupMask;       /* Mask for that configuration set */
   const char *zHelp;   /* What it does */
 } aGroupName[] = {
-  { "/email",       CONFIGSET_ADDR,  "Concealed email addresses in tickets" },
-  { "/project",     CONFIGSET_PROJ,  "Project name and description"         },
+  { "/email",       CONFIGSET_ADDR,    "Concealed email addresses in tickets" },
+  { "/project",     CONFIGSET_PROJ,    "Project name and description"         },
   { "/skin",        CONFIGSET_SKIN | CONFIGSET_CSS,
-                                     "Web interface appearance settings"    },
-  { "/css",         CONFIGSET_CSS,   "Style sheet"                          },
-  { "/shun",        CONFIGSET_SHUN,  "List of shunned artifacts"            },
-  { "/ticket",      CONFIGSET_TKT,   "Ticket setup",                        },
-  { "/user",        CONFIGSET_USER,  "Users and privilege settings"         },
-  { "/xfer",        CONFIGSET_XFER,  "Transfer setup",                      },
-  { "/alias",       CONFIGSET_ALIAS, "URL Aliases",                         },
-  { "/subscriber",  CONFIGSET_SCRIBER,"Email notification subscriber list"  },
-  { "/all",         CONFIGSET_ALL,   "All of the above"                     },
+                                       "Web interface appearance settings"    },
+  { "/css",         CONFIGSET_CSS,     "Style sheet"                          },
+  { "/shun",        CONFIGSET_SHUN,    "List of shunned artifacts"            },
+  { "/ticket",      CONFIGSET_TKT,     "Ticket setup",                        },
+  { "/user",        CONFIGSET_USER,    "Users and privilege settings"         },
+  { "/xfer",        CONFIGSET_XFER,    "Transfer setup",                      },
+  { "/alias",       CONFIGSET_ALIAS,   "URL Aliases",                         },
+  { "/subscriber",  CONFIGSET_SCRIBER, "Email notification subscriber list"   },
+  { "/interwiki",   CONFIGSET_IWIKI,   "Inter-wiki link prefixes"             },
+  { "/all",         CONFIGSET_ALL,     "All of the above"                     },
 };
 
 
@@ -89,22 +91,32 @@ static struct {
 } aConfig[] = {
   { "css",                    CONFIGSET_CSS  },
   { "header",                 CONFIGSET_SKIN },
+  { "mainmenu",               CONFIGSET_SKIN },
   { "footer",                 CONFIGSET_SKIN },
   { "details",                CONFIGSET_SKIN },
+  { "js",                     CONFIGSET_SKIN },
   { "logo-mimetype",          CONFIGSET_SKIN },
   { "logo-image",             CONFIGSET_SKIN },
   { "background-mimetype",    CONFIGSET_SKIN },
   { "background-image",       CONFIGSET_SKIN },
+  { "icon-mimetype",          CONFIGSET_SKIN },
+  { "icon-image",             CONFIGSET_SKIN },
   { "timeline-block-markup",  CONFIGSET_SKIN },
+  { "timeline-date-format",   CONFIGSET_SKIN },
+  { "timeline-default-style", CONFIGSET_SKIN },
+  { "timeline-dwelltime",     CONFIGSET_SKIN },
+  { "timeline-closetime",     CONFIGSET_SKIN },
   { "timeline-max-comment",   CONFIGSET_SKIN },
   { "timeline-plaintext",     CONFIGSET_SKIN },
+  { "timeline-truncate-at-blank", CONFIGSET_SKIN },
+  { "timeline-tslink-info",   CONFIGSET_SKIN },
+  { "timeline-utc",           CONFIGSET_SKIN },
   { "adunit",                 CONFIGSET_SKIN },
   { "adunit-omit-if-admin",   CONFIGSET_SKIN },
   { "adunit-omit-if-user",    CONFIGSET_SKIN },
-  { "sitemap-docidx",         CONFIGSET_SKIN },
-  { "sitemap-download",       CONFIGSET_SKIN },
-  { "sitemap-license",        CONFIGSET_SKIN },
-  { "sitemap-contact",        CONFIGSET_SKIN },
+  { "default-csp",            CONFIGSET_SKIN },
+  { "sitemap-extra",          CONFIGSET_SKIN },
+  { "safe-html",              CONFIGSET_SKIN },
 
 #ifdef FOSSIL_ENABLE_TH1_DOCS
   { "th1-docs",               CONFIGSET_TH1 },
@@ -133,16 +145,14 @@ static struct {
   { "crnl-glob",              CONFIGSET_PROJ },
   { "encoding-glob",          CONFIGSET_PROJ },
   { "empty-dirs",             CONFIGSET_PROJ },
-  { "allow-symlinks",         CONFIGSET_PROJ },
   { "dotfiles",               CONFIGSET_PROJ },
   { "parent-project-code",    CONFIGSET_PROJ },
   { "parent-project-name",    CONFIGSET_PROJ },
   { "hash-policy",            CONFIGSET_PROJ },
-
-#ifdef FOSSIL_ENABLE_LEGACY_MV_RM
+  { "comment-format",         CONFIGSET_PROJ },
+  { "mimetypes",              CONFIGSET_PROJ },
+  { "forbid-delta-manifests", CONFIGSET_PROJ },
   { "mv-rm-files",            CONFIGSET_PROJ },
-#endif
-
   { "ticket-table",           CONFIGSET_TKT  },
   { "ticket-common",          CONFIGSET_TKT  },
   { "ticket-change",          CONFIGSET_TKT  },
@@ -157,6 +167,7 @@ static struct {
   { "@reportfmt",             CONFIGSET_TKT  },
 
   { "@user",                  CONFIGSET_USER },
+  { "user-color-map",         CONFIGSET_USER },
 
   { "@concealed",             CONFIGSET_ADDR },
 
@@ -166,6 +177,8 @@ static struct {
 
   { "@subscriber",            CONFIGSET_SCRIBER },
 
+  { "@interwiki",             CONFIGSET_IWIKI },
+
   { "xfer-common-script",     CONFIGSET_XFER },
   { "xfer-push-script",       CONFIGSET_XFER },
   { "xfer-commit-script",     CONFIGSET_XFER },
@@ -249,6 +262,9 @@ int configure_is_exportable(const char *zName){
   if( strncmp(zName, "walias:/", 8)==0 ){
     return CONFIGSET_ALIAS;
   }
+  if( strncmp(zName, "interwiki:", 10)==0 ){
+    return CONFIGSET_IWIKI;
+  }
   return 0;
 }
 
@@ -421,7 +437,7 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
     }
     blob_append_sql(&sql, "\"%w\"(\"%w\",mtime",
          &zName[1], aType[ii].zPrimKey);
-    if( fossil_stricmp(zName,"/subscriber") ) alert_schema(0);
+    if( fossil_stricmp(zName,"/subscriber")==0 ) alert_schema(0);
     for(jj=2; jj<nToken; jj+=2){
        blob_append_sql(&sql, ",\"%w\"", azToken[jj]);
     }
@@ -430,6 +446,7 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
     for(jj=2; jj<nToken; jj+=2){
        blob_append_sql(&sql, ",%s", azToken[jj+1] /*safe-for-%s*/);
     }
+    db_protect_only(PROTECT_SENSITIVE);
     db_multi_exec("%s)", blob_sql_text(&sql));
     if( db_changes()==0 ){
       blob_reset(&sql);
@@ -444,6 +461,7 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
                    azToken[0]/*safe-for-%s*/);
       db_multi_exec("%s", blob_sql_text(&sql));
     }
+    db_protect_pop();
     blob_reset(&sql);
     rebuildMask |= thisMask;
   }
@@ -586,6 +604,22 @@ int configure_send_group(
     }
     db_finalize(&q);
   }
+  if( groupMask & CONFIGSET_IWIKI ){
+    db_prepare(&q, "SELECT mtime, quote(name), quote(value) FROM config"
+                   " WHERE name GLOB 'interwiki:*' AND mtime>=%lld", iStart);
+    while( db_step(&q)==SQLITE_ROW ){
+      blob_appendf(&rec,"%s %s value %s",
+        db_column_text(&q, 0),
+        db_column_text(&q, 1),
+        db_column_text(&q, 2)
+      );
+      blob_appendf(pOut, "config /config %d\n%s\n",
+                   blob_size(&rec), blob_str(&rec));
+      nCard++;
+      blob_reset(&rec);
+    }
+    db_finalize(&q);
+  }
   if( (groupMask & CONFIGSET_SCRIBER)!=0
    && db_table_exists("repository","subscriber")
   ){
@@ -694,51 +728,52 @@ static void export_config(
 ** Where METHOD is one of: export import merge pull push reset.  All methods
 ** accept the -R or --repository option to specify a repository.
 **
-**    %fossil configuration export AREA FILENAME
+** >  fossil configuration export AREA FILENAME
 **
 **         Write to FILENAME exported configuration information for AREA.
 **         AREA can be one of:
 **
-**             all email project shun skin ticket user alias subscriber
+**             all email interwiki project shun skin
+**             ticket user alias subscriber
 **
-**    %fossil configuration import FILENAME
+** >  fossil configuration import FILENAME
 **
 **         Read a configuration from FILENAME, overwriting the current
 **         configuration.
 **
-**    %fossil configuration merge FILENAME
+** >  fossil configuration merge FILENAME
 **
 **         Read a configuration from FILENAME and merge its values into
 **         the current configuration.  Existing values take priority over
 **         values read from FILENAME.
 **
-**    %fossil configuration pull AREA ?URL?
+** >  fossil configuration pull AREA ?URL?
 **
 **         Pull and install the configuration from a different server
 **         identified by URL.  If no URL is specified, then the default
 **         server is used.  Use the --overwrite flag to completely
 **         replace local settings with content received from URL.
 **
-**    %fossil configuration push AREA ?URL?
+** >  fossil configuration push AREA ?URL?
 **
 **         Push the local configuration into the remote server identified
 **         by URL.  Admin privilege is required on the remote server for
 **         this to work.  When the same record exists both locally and on
 **         the remote end, the one that was most recently changed wins.
 **
-**    %fossil configuration reset AREA
+** >  fossil configuration reset AREA
 **
 **         Restore the configuration to the default.  AREA as above.
 **
-**    %fossil configuration sync AREA ?URL?
+** >  fossil configuration sync AREA ?URL?
 **
 **         Synchronize configuration changes in the local repository with
 **         the remote repository at URL.
 **
 ** Options:
-**    -R|--repository FILE       Extract info from repository FILE
+**    -R|--repository REPO       Extract info from repository REPO
 **
-** See also: settings, unset
+** See also: [[settings]], [[unset]]
 */
 void configuration_cmd(void){
   int n;
@@ -780,7 +815,9 @@ void configuration_cmd(void){
     }else{
       groupMask = CONFIGSET_ALL;
     }
+    db_unprotect(PROTECT_USER);
     configure_receive_all(&in, groupMask);
+    db_protect_pop();
     db_end_transaction(0);
   }else
   if( strncmp(zMethod, "pull", n)==0
@@ -808,11 +845,11 @@ void configuration_cmd(void){
     url_enable_proxy("via proxy: ");
     if( overwriteFlag ) mask |= CONFIGSET_OVERWRITE;
     if( strncmp(zMethod, "push", n)==0 ){
-      client_sync(0,0,(unsigned)mask);
+      client_sync(0,0,(unsigned)mask,0);
     }else if( strncmp(zMethod, "pull", n)==0 ){
-      client_sync(0,(unsigned)mask,0);
+      client_sync(0,(unsigned)mask,0,0);
     }else{
-      client_sync(0,(unsigned)mask,(unsigned)mask);
+      client_sync(0,(unsigned)mask,(unsigned)mask,0);
     }
   }else
   if( strncmp(zMethod, "reset", n)==0 ){
@@ -828,9 +865,13 @@ void configuration_cmd(void){
       const char *zName = aConfig[i].zName;
       if( (aConfig[i].groupMask & mask)==0 ) continue;
       if( zName[0]!='@' ){
+        db_unprotect(PROTECT_CONFIG);
         db_multi_exec("DELETE FROM config WHERE name=%Q", zName);
+        db_protect_pop();
       }else if( fossil_strcmp(zName,"@user")==0 ){
+        db_unprotect(PROTECT_USER);
         db_multi_exec("DELETE FROM user");
+        db_protect_pop();
         db_create_default_users(0, 0);
       }else if( fossil_strcmp(zName,"@concealed")==0 ){
         db_multi_exec("DELETE FROM concealed");
@@ -1042,6 +1083,7 @@ void test_var_set_cmd(void){
   }else{
     blob_init(&x,g.argv[3],-1);
   }
+  db_unprotect(PROTECT_CONFIG);
   db_prepare(&ins,
      "REPLACE INTO config(name,value,mtime)"
      "VALUES(%Q,:val,now())", zVar);
@@ -1052,5 +1094,6 @@ void test_var_set_cmd(void){
   }
   db_step(&ins);
   db_finalize(&ins);
+  db_protect_pop();
   blob_reset(&x);
 }
