@@ -82,7 +82,7 @@ void transport_stats(i64 *pnSent, i64 *pnRcvd, int resetFlag){
 ** like "/bin/rm".
 */
 static int is_safe_fossil_command(const char *zFossil){
-  static const char *azSafe[] = { "*/fossil", "*/echo" };
+  static const char *const azSafe[] = { "*/fossil", "*/fossil.exe", "*/echo" };
   int i;
   for(i=0; i<sizeof(azSafe)/sizeof(azSafe[0]); i++){
     if( sqlite3_strglob(azSafe[i], zFossil)==0 ) return 1;
@@ -94,12 +94,21 @@ static int is_safe_fossil_command(const char *zFossil){
 /*
 ** Default SSH command
 */
-#ifdef _WIN32
-static const char zDefaultSshCmd[] = "plink -ssh -T";
+#if 0 /* was: defined(_WIN32).  Windows generally has ssh now. */
+static const char zDefaultSshCmd[] = "plink -ssh";
 #else
-static const char zDefaultSshCmd[] = "ssh -e none -T";
+static const char zDefaultSshCmd[] = "ssh -e none";
 #endif
 
+/*
+** Initialize a Blob to the name of the configured SSH command.
+*/
+void transport_ssh_command(Blob *p){
+  char *zSsh;        /* The base SSH command */
+  zSsh = db_get("ssh-command", zDefaultSshCmd);
+  blob_init(p, zSsh, -1);
+}
+
 /*
 ** SSH initialization of the transport layer
 */
@@ -107,42 +116,37 @@ int transport_ssh_open(UrlData *pUrlData){
   /* For SSH we need to create and run SSH fossil http
   ** to talk to the remote machine.
   */
-  char *zSsh;        /* The base SSH command */
   Blob zCmd;         /* The SSH command */
   char *zHost;       /* The host name to contact */
 
   socket_ssh_resolve_addr(pUrlData);
-  zSsh = db_get("ssh-command", zDefaultSshCmd);
-  blob_init(&zCmd, zSsh, -1);
+  transport_ssh_command(&zCmd);
   if( pUrlData->port!=pUrlData->dfltPort && pUrlData->port ){
-#ifdef _WIN32
-    blob_appendf(&zCmd, " -P %d", pUrlData->port);
-#else
     blob_appendf(&zCmd, " -p %d", pUrlData->port);
-#endif
   }
+  blob_appendf(&zCmd, " -T --");  /* End of switches */
   if( pUrlData->user && pUrlData->user[0] ){
     zHost = mprintf("%s@%s", pUrlData->user, pUrlData->name);
-    blob_append_escaped_arg(&zCmd, zHost);
+    blob_append_escaped_arg(&zCmd, zHost, 0);
     fossil_free(zHost);
   }else{
-    blob_append_escaped_arg(&zCmd, pUrlData->name);
+    blob_append_escaped_arg(&zCmd, pUrlData->name, 0);
   }
   if( !is_safe_fossil_command(pUrlData->fossil) ){
     fossil_fatal("the ssh:// URL is asking to run an unsafe command [%s] on "
                  "the server.", pUrlData->fossil);
   }
-  blob_append_escaped_arg(&zCmd, pUrlData->fossil);
+  blob_append_escaped_arg(&zCmd, pUrlData->fossil, 1);
   blob_append(&zCmd, " test-http", 10);
   if( pUrlData->path && pUrlData->path[0] ){
-    blob_append_escaped_arg(&zCmd, pUrlData->path);
+    blob_append_escaped_arg(&zCmd, pUrlData->path, 1);
   }else{
     fossil_fatal("ssh:// URI does not specify a path to the repository");
   }
   if( g.fSshTrace ){
     fossil_print("%s\n", blob_str(&zCmd));  /* Show the whole SSH command */
   }
-  popen2(blob_str(&zCmd), &sshIn, &sshOut, &sshPid);
+  popen2(blob_str(&zCmd), &sshIn, &sshOut, &sshPid, 0);
   if( sshPid==0 ){
     socket_set_errmsg("cannot start ssh tunnel using [%b]", &zCmd);
   }
@@ -154,7 +158,7 @@ int transport_ssh_open(UrlData *pUrlData){
 ** Open a connection to the server.  The server is defined by the following
 ** variables:
 **
-**   pUrlData->name        Name of the server.  Ex: www.fossil-scm.org
+**   pUrlData->name        Name of the server.  Ex: fossil-scm.org
 **   pUrlData->port        TCP/IP port.  Ex: 80
 **   pUrlData->isHttps     Use TLS for the connection
 **
@@ -167,13 +171,13 @@ int transport_open(UrlData *pUrlData){
       rc = transport_ssh_open(pUrlData);
       if( rc==0 ) transport.isOpen = 1;
     }else if( pUrlData->isHttps ){
-      #ifdef FOSSIL_ENABLE_SSL
+#ifdef FOSSIL_ENABLE_SSL
       rc = ssl_open(pUrlData);
       if( rc==0 ) transport.isOpen = 1;
-      #else
+#else
       socket_set_errmsg("HTTPS: Fossil has been compiled without SSL support");
       rc = 1;
-      #endif
+#endif
     }else if( pUrlData->isFile ){
       sqlite3_uint64 iRandId;
       sqlite3_randomness(sizeof(iRandId), &iRandId);
@@ -241,7 +245,7 @@ void transport_send(UrlData *pUrlData, Blob *toSend){
     fwrite(z, 1, n, sshOut);
     fflush(sshOut);
   }else if( pUrlData->isHttps ){
-    #ifdef FOSSIL_ENABLE_SSL
+#ifdef FOSSIL_ENABLE_SSL
     int sent;
     while( n>0 ){
       sent = ssl_send(0, z, n);
@@ -249,7 +253,7 @@ void transport_send(UrlData *pUrlData, Blob *toSend){
       if( sent<=0 ) break;
       n -= sent;
     }
-    #endif
+#endif
   }else if( pUrlData->isFile ){
     fwrite(z, 1, n, transport.pFile);
   }else{
@@ -265,13 +269,14 @@ void transport_send(UrlData *pUrlData, Blob *toSend){
 
 /*
 ** This routine is called when the outbound message is complete and
-** it is time to being receiving a reply.
+** it is time to begin receiving a reply.
 */
 void transport_flip(UrlData *pUrlData){
   if( pUrlData->isFile ){
     char *zCmd;
     fclose(transport.pFile);
-    zCmd = mprintf("\"%s\" http \"%s\" \"%s\" 127.0.0.1 \"%s\" --localauth",
+    zCmd = mprintf("%$ http --in %$ --out %$ --ipaddr 127.0.0.1"
+                   " %$ --localauth",
        g.nameOfExe, transport.zOutFile, transport.zInFile, pUrlData->name
     );
     fossil_system(zCmd);
@@ -327,7 +332,7 @@ static int transport_fetch(UrlData *pUrlData, char *zBuf, int N){
   }else if( pUrlData->isFile ){
     got = fread(zBuf, 1, N, transport.pFile);
   }else{
-    got = socket_receive(0, zBuf, N);
+    got = socket_receive(0, zBuf, N, 0);
   }
   /* printf("received %d of %d bytes\n", got, N); fflush(stdout); */
   if( transport.pLog ){
