@@ -118,7 +118,7 @@ cson_value * json_get_wiki_page_by_rid(int rid, int contentFormat){
         zFormat = "html";
         if(zBody && *zBody){
           const char *zMimetype = pWiki->zMimetype;
-          if( zMimetype==0 ) zMimetype = "text/plain";
+          if( zMimetype==0 ) zMimetype = "text/x-fossil-wiki";
           zMimetype = wiki_filter_mimetypes(zMimetype);
           blob_append(&raw,zBody,-1);
           if( fossil_strcmp(zMimetype, "text/x-fossil-wiki")==0 ){
@@ -157,7 +157,7 @@ cson_value * json_get_wiki_page_by_rid(int rid, int contentFormat){
     /*TODO: add the 'A' card (file attachment) entries?*/
     manifest_destroy(pWiki);
     return cson_object_value(pay);
-  }  
+  }
 }
 
 /*
@@ -172,7 +172,7 @@ cson_value * json_get_wiki_page_by_name(char const * zPageName, int contentForma
                " WHERE x.tagid=t.tagid AND t.tagname='wiki-%q' "
                " AND b.rid=x.rid"
                " ORDER BY x.mtime DESC LIMIT 1",
-               zPageName 
+               zPageName
              );
   if( 0==rid ){
     json_set_err( FSL_JSON_E_RESOURCE_NOT_FOUND, "Wiki page not found: %s",
@@ -258,7 +258,7 @@ static cson_value * json_wiki_get(){
   zPageName = json_find_option_cstr2("name",NULL,"n",g.json.dispatchDepth+1);
 
   zSymName = json_find_option_cstr("uuid",NULL,"u");
-  
+
   if((!zPageName||!*zPageName) && (!zSymName || !*zSymName)){
     json_set_err(FSL_JSON_E_MISSING_ARGS,
                  "At least one of the 'name' or 'uuid' arguments must be provided.");
@@ -268,7 +268,7 @@ static cson_value * json_wiki_get(){
   /* TODO: see if we have a page named zPageName. If not, try to resolve
      zPageName as a UUID.
   */
-  
+
   contentFormat = json_wiki_get_content_format_flag(contentFormat);
   return json_wiki_get_by_name_or_symname( zPageName, zSymName, contentFormat );
 }
@@ -460,8 +460,11 @@ static cson_value * json_wiki_list(){
     return NULL;
   }
   blob_append(&sql,"SELECT"
-              " substr(tagname,6) as name"
-              " FROM tag WHERE tagname GLOB 'wiki-*'",
+              " DISTINCT substr(tagname,6) as name"
+              " FROM tag JOIN tagxref USING('tagid')"
+              " WHERE tagname GLOB 'wiki-*'"
+              " AND TYPEOF(tagxref.value+0)='integer'",
+              /* ^^^ elide wiki- tags which are not wiki pages */
               -1);
   zGlob = json_find_option_cstr("glob",NULL,"g");
   if(zGlob && *zGlob){
@@ -518,7 +521,7 @@ static cson_value * json_wiki_diff(){
   Manifest * pW1 = NULL, *pW2 = NULL;
   Blob w1 = empty_blob, w2 = empty_blob, d = empty_blob;
   char const * zErrTag = NULL;
-  u64 diffFlags;
+  DiffConfig DCfg;
   char * zUuid = NULL;
   if( !g.perm.Hyperlink ){
     json_set_err(FSL_JSON_E_DENIED,
@@ -526,7 +529,7 @@ static cson_value * json_wiki_diff(){
     return NULL;
   }
 
-  
+
   zV1 = json_find_option_cstr2( "v1",NULL, NULL, ++argPos );
   zV2 = json_find_option_cstr2( "v2",NULL, NULL, ++argPos );
   if(!zV1 || !*zV1 || !zV2 || !*zV2) {
@@ -534,7 +537,7 @@ static cson_value * json_wiki_diff(){
                  "Requires both 'v1' and 'v2' arguments.");
     return NULL;
   }
-  
+
   r1 = symbolic_name_to_rid( zV1, "w" );
   zErrTag = zV1;
   if(r1<0){
@@ -566,13 +569,13 @@ static cson_value * json_wiki_diff(){
   blob_zero(&w2);
   blob_init(&w2, pW2->zWiki, -1);
   blob_zero(&d);
-  diffFlags = DIFF_IGNORE_EOLWS | DIFF_STRIP_EOLCR;
-  text_diff(&w1, &w2, &d, 0, diffFlags);
+  diff_config_init(&DCfg, DIFF_IGNORE_EOLWS | DIFF_STRIP_EOLCR);
+  text_diff(&w1, &w2, &d, &DCfg);
   blob_reset(&w1);
   blob_reset(&w2);
 
   pay = cson_new_object();
-  
+
   zUuid = json_wiki_get_uuid_for_rid( pW1->rid );
   cson_object_set(pay, "v1", json_new_string(zUuid) );
   free(zUuid);
@@ -587,7 +590,7 @@ static cson_value * json_wiki_diff(){
   cson_object_set(pay, "diff",
                   cson_value_new_string( blob_str(&d),
                                          (unsigned int)blob_size(&d)));
-  
+
   return cson_object_value(pay);
 
   manifest:
