@@ -1,21 +1,32 @@
 #!/usr/bin/tclsh
 #
-# Run this Tcl script to generate the various makefiles for a variety
-# of platforms.  Files generated include:
+#    ### Run this Tcl script EVERY time you modify it in any way! ###
+#
+# This Tcl script generates make files for various platforms. The makefiles
+# then need to be committed.
+#
+# If you modify this file then:
+#
+#     1. cd src; tclsh makemake.tcl
+#
+#     2. if errors are reported, fix them and go to step 1
+#
+#     3. if "fossil diff" reports changes in any of the generated
+#        files, commit the changed files to the repo
+#
+# Files generated include:
 #
 #     src/main.mk           # makefile for all unix systems
 #     win/Makefile.mingw    # makefile for mingw on windows
 #     win/Makefile.*        # makefiles for other windows compilers
 #
-# Run this script while in the "src" subdirectory.  Like this:
-#
-#      tclsh makemake.tcl
-#
 # Add new source files by listing the files (without their .c suffix)
 # in the "src" variable.  Add new resource files to the "extra_files"
 # variable.  There are other variables that you can alter, down to
 # the "STOP HERE" comment.  The stuff below "STOP HERE" should rarely need
-# to change.
+# to change. After modification, go to step 1 above.
+#
+# Delete unused source files in the "src" variable, then go to step 1 above.
 #
 #############################################################################
 
@@ -28,9 +39,11 @@
 #
 set src {
   add
+  ajax
   alerts
   allrepo
   attach
+  backlink
   backoffice
   bag
   bisect
@@ -43,10 +56,12 @@ set src {
   capabilities
   captcha
   cgi
+  chat
   checkin
   checkout
   clearsign
   clone
+  color
   comformat
   configure
   content
@@ -54,6 +69,7 @@ set src {
   db
   delta
   deltacmd
+  deltafunc
   descendants
   diff
   diffcmd
@@ -62,22 +78,27 @@ set src {
   encode
   etag
   event
+  extcgi
   export
   file
+  fileedit
   finfo
   foci
   forum
   fshell
   fusefs
+  fuzz
   glob
   graph
   gzip
   hname
+  hook
   http
   http_socket
   http_transport
   import
   info
+  interwiki
   json
   json_artifact
   json_branch
@@ -106,8 +127,11 @@ set src {
   merge3
   moderate
   name
+  patch
   path
   piechart
+  pikchr
+  pikchrshow
   pivot
   popen
   pqueue
@@ -116,6 +140,7 @@ set src {
   purge
   rebuild
   regexp
+  repolist
   report
   rss
   schema
@@ -138,6 +163,7 @@ set src {
   sync
   tag
   tar
+  terminal
   th_main
   timeline
   tkt
@@ -152,12 +178,10 @@ set src {
   util
   verify
   vfile
-  webmail
   wiki
   wikiformat
   winfile
   winhttp
-  wysiwyg
   xfer
   xfersetup
   zip
@@ -171,20 +195,24 @@ set extra_files {
   markdown.md
   wiki.wiki
   *.js
+  default.css
+  style.*.css
   ../skins/*/*.txt
+  sounds/*.wav
+  alerts/*.wav
 }
 
 # Options used to compile the included SQLite library.
 #
 set SQLITE_OPTIONS {
   -DNDEBUG=1
+  -DSQLITE_DQS=0
   -DSQLITE_THREADSAFE=0
   -DSQLITE_DEFAULT_MEMSTATUS=0
   -DSQLITE_DEFAULT_WAL_SYNCHRONOUS=1
   -DSQLITE_LIKE_DOESNT_MATCH_BLOBS
   -DSQLITE_OMIT_DECLTYPE
   -DSQLITE_OMIT_DEPRECATED
-  -DSQLITE_OMIT_GET_TABLE
   -DSQLITE_OMIT_PROGRESS_CALLBACK
   -DSQLITE_OMIT_SHARED_CACHE
   -DSQLITE_OMIT_LOAD_EXTENSION
@@ -194,7 +222,6 @@ set SQLITE_OPTIONS {
   -DSQLITE_DEFAULT_FILE_FORMAT=4
   -DSQLITE_ENABLE_EXPLAIN_COMMENTS
   -DSQLITE_ENABLE_FTS4
-  -DSQLITE_ENABLE_FTS3_PARENTHESIS
   -DSQLITE_ENABLE_DBSTAT_VTAB
   -DSQLITE_ENABLE_JSON1
   -DSQLITE_ENABLE_FTS5
@@ -202,6 +229,7 @@ set SQLITE_OPTIONS {
   -DSQLITE_HAVE_ZLIB
   -DSQLITE_INTROSPECTION_PRAGMAS
   -DSQLITE_ENABLE_DBPAGE_VTAB
+  -DSQLITE_TRUSTED_SCHEMA=0
 }
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_FTS3=1
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_STAT4
@@ -309,9 +337,6 @@ writeln -nonewline "OBJ ="
 foreach s [lsort $src] {
   writeln -nonewline " \\\n \$(OBJDIR)/$s.o"
 }
-writeln "\n"
-writeln "APPNAME = $name\$(E)"
-writeln "\n"
 
 writeln [string map [list \
     <<<SQLITE_OPTIONS>>> [join $SQLITE_OPTIONS " \\\n                 "] \
@@ -319,7 +344,7 @@ writeln [string map [list \
     <<<MINIZ_OPTIONS>>> [join $MINIZ_OPTIONS " \\\n                "]] {
 all:	$(OBJDIR) $(APPNAME)
 
-install:	$(APPNAME)
+install:	all
 	mkdir -p $(INSTALLDIR)
 	cp $(APPNAME) $(INSTALLDIR)
 
@@ -344,9 +369,6 @@ $(OBJDIR)/mkbuiltin:	$(SRCDIR)/mkbuiltin.c
 $(OBJDIR)/mkversion:	$(SRCDIR)/mkversion.c
 	$(XBCC) -o $(OBJDIR)/mkversion $(SRCDIR)/mkversion.c
 
-$(OBJDIR)/mkcss:	$(SRCDIR)/mkcss.c
-	$(XBCC) -o $(OBJDIR)/mkcss $(SRCDIR)/mkcss.c
-
 $(OBJDIR)/codecheck1:	$(SRCDIR)/codecheck1.c
 	$(XBCC) -o $(OBJDIR)/codecheck1 $(SRCDIR)/codecheck1.c
 
@@ -366,13 +388,13 @@ $(OBJDIR)/codecheck1:	$(SRCDIR)/codecheck1.c
 test:	$(OBJDIR) $(APPNAME)
 	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME) $(TESTFLAGS)
 
-$(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION $(OBJDIR)/mkversion
+$(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION $(OBJDIR)/mkversion $(OBJDIR)/phony.h
 	$(OBJDIR)/mkversion $(SRCDIR)/../manifest.uuid \
 		$(SRCDIR)/../manifest \
 		$(SRCDIR)/../VERSION >$(OBJDIR)/VERSION.h
 
-$(OBJDIR)/default_css.h:	$(SRCDIR)/default_css.txt $(OBJDIR)/mkcss
-	$(OBJDIR)/mkcss $(SRCDIR)/default_css.txt $(OBJDIR)/default_css.h
+$(OBJDIR)/phony.h:
+	# Force rebuild of VERSION.h every time we run "make"
 
 # Setup the options used to compile the included SQLite library.
 SQLITE_OPTIONS = <<<SQLITE_OPTIONS>>>
@@ -439,9 +461,9 @@ EXTRAOBJ = <<<NEXT_LINE>>>
 }]
 
 writeln {
-$(APPNAME):	$(OBJDIR)/headers $(OBJDIR)/codecheck1 $(OBJ) $(EXTRAOBJ)
+$(APPNAME):	$(OBJDIR)/headers $(OBJDIR)/codecheck1 $(EXTRAOBJ) $(OBJ)
 	$(OBJDIR)/codecheck1 $(TRANS_SRC)
-	$(TCC) -o $(APPNAME) $(OBJ) $(EXTRAOBJ) $(LIB)
+	$(TCC) $(TCCFLAGS) -o $(APPNAME) $(EXTRAOBJ) $(OBJ) $(LIB)
 
 # This rule prevents make from using its default rules to try build
 # an executable named "manifest" out of the file named "manifest.c"
@@ -470,15 +492,15 @@ writeln "\t\$(OBJDIR)/mkindex \$(TRANS_SRC) >\$@\n"
 writeln "\$(OBJDIR)/builtin_data.h: \$(OBJDIR)/mkbuiltin \$(EXTRA_FILES)"
 writeln "\t\$(OBJDIR)/mkbuiltin --prefix \$(SRCDIR)/ \$(EXTRA_FILES) >\$@\n"
 
-writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/default_css.h \$(OBJDIR)/makeheaders \$(OBJDIR)/VERSION.h"
+writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/makeheaders \$(OBJDIR)/VERSION.h"
 writeln "\t\$(OBJDIR)/makeheaders $mhargs"
 writeln "\ttouch \$(OBJDIR)/headers"
 writeln "\$(OBJDIR)/headers: Makefile"
 writeln "\$(OBJDIR)/json.o \$(OBJDIR)/json_artifact.o \$(OBJDIR)/json_branch.o \$(OBJDIR)/json_config.o \$(OBJDIR)/json_diff.o \$(OBJDIR)/json_dir.o \$(OBJDIR)/json_finfo.o \$(OBJDIR)/json_login.o \$(OBJDIR)/json_query.o \$(OBJDIR)/json_report.o \$(OBJDIR)/json_status.o \$(OBJDIR)/json_tag.o \$(OBJDIR)/json_timeline.o \$(OBJDIR)/json_user.o \$(OBJDIR)/json_wiki.o : \$(SRCDIR)/json_detail.h"
+
 writeln "Makefile:"
 set extra_h(dispatch) " \$(OBJDIR)/page_index.h "
 set extra_h(builtin) " \$(OBJDIR)/builtin_data.h "
-set extra_h(style) " \$(OBJDIR)/default_css.h "
 
 foreach s [lsort $src] {
   writeln "\$(OBJDIR)/${s}_.c:\t\$(SRCDIR)/$s.c \$(OBJDIR)/translate"
@@ -607,10 +629,6 @@ BCC = $(BCCEXE)
 #
 # FOSSIL_ENABLE_EXEC_REL_PATHS = 1
 
-#### Enable legacy treatment of mv/rm (skip checkout files)
-#
-FOSSIL_ENABLE_LEGACY_MV_RM = 1
-
 #### Enable TH1 scripts in embedded documentation files
 #
 # FOSSIL_ENABLE_TH1_DOCS = 1
@@ -696,9 +714,9 @@ ZLIBTARGETS =
 endif
 
 #### Disable creation of the OpenSSL shared libraries.  Also, disable support
-#    for both SSLv2 and SSLv3 (i.e. thereby forcing the use of TLS).
+#    for SSLv3 (i.e. thereby forcing the use of TLS).
 #
-SSLCONFIG += no-ssl2 no-ssl3 no-weak-ssl-ciphers no-shared
+SSLCONFIG += no-ssl3 no-weak-ssl-ciphers no-shared
 
 #### When using zlib, make sure that OpenSSL is configured to use the zlib
 #    that Fossil knows about (i.e. the one within the source tree).
@@ -708,11 +726,8 @@ SSLCONFIG +=  --with-zlib-lib=$(PWD)/$(ZLIBDIR) --with-zlib-include=$(PWD)/$(ZLI
 endif
 
 #### The directories where the OpenSSL include and library files are located.
-#    The recommended usage here is to use the Sysinternals junction tool
-#    to create a hard link between an "openssl-1.x" sub-directory of the
-#    Fossil source code directory and the target OpenSSL source directory.
 #
-OPENSSLDIR = $(SRCDIR)/../compat/openssl-1.0.2p
+OPENSSLDIR = $(SRCDIR)/../compat/openssl
 OPENSSLINCDIR = $(OPENSSLDIR)/include
 OPENSSLLIBDIR = $(OPENSSLDIR)
 
@@ -834,12 +849,6 @@ TCC += -DFOSSIL_ENABLE_EXEC_REL_PATHS=1
 RCC += -DFOSSIL_ENABLE_EXEC_REL_PATHS=1
 endif
 
-# With legacy treatment of mv/rm
-ifdef FOSSIL_ENABLE_LEGACY_MV_RM
-TCC += -DFOSSIL_ENABLE_LEGACY_MV_RM=1
-RCC += -DFOSSIL_ENABLE_LEGACY_MV_RM=1
-endif
-
 # With TH1 embedded docs support
 ifdef FOSSIL_ENABLE_TH1_DOCS
 TCC += -DFOSSIL_ENABLE_TH1_DOCS=1
@@ -1009,7 +1018,6 @@ MAKEHEADERS = $(subst /,\,$(OBJDIR)/makeheaders.exe)
 MKINDEX     = $(subst /,\,$(OBJDIR)/mkindex.exe)
 MKBUILTIN   = $(subst /,\,$(OBJDIR)/mkbuiltin.exe)
 MKVERSION   = $(subst /,\,$(OBJDIR)/mkversion.exe)
-MKCSS       = $(subst /,\,$(OBJDIR)/mkcss.exe)
 CODECHECK1  = $(subst /,\,$(OBJDIR)/codecheck1.exe)
 CAT         = type
 CP          = copy
@@ -1024,7 +1032,6 @@ MAKEHEADERS = $(OBJDIR)/makeheaders.exe
 MKINDEX     = $(OBJDIR)/mkindex.exe
 MKBUILTIN   = $(OBJDIR)/mkbuiltin.exe
 MKVERSION   = $(OBJDIR)/mkversion.exe
-MKCSS       = $(OBJDIR)/mkcss.exe
 CODECHECK1  = $(OBJDIR)/codecheck1.exe
 CAT         = cat
 CP          = cp
@@ -1038,7 +1045,7 @@ endif}
 writeln {
 all:	$(OBJDIR) $(APPNAME)
 
-$(OBJDIR)/fossil.o:	$(SRCDIR)/../win/fossil.rc $(OBJDIR)/VERSION.h $(OBJDIR)/default_css.h
+$(OBJDIR)/fossil.o:	$(SRCDIR)/../win/fossil.rc $(OBJDIR)/VERSION.h
 ifdef USE_WINDOWS
 	$(CAT) $(subst /,\,$(SRCDIR)\miniz.c) | $(GREP) "define MZ_VERSION" > $(subst /,\,$(OBJDIR)\minizver.h)
 	$(CP) $(subst /,\,$(SRCDIR)\..\win\fossil.rc) $(subst /,\,$(OBJDIR))
@@ -1083,9 +1090,6 @@ $(MKBUILTIN):	$(SRCDIR)/mkbuiltin.c
 $(MKVERSION): $(SRCDIR)/mkversion.c
 	$(XBCC) -o $@ $(SRCDIR)/mkversion.c
 
-$(MKCSS): $(SRCDIR)/mkcss.c
-	$(XBCC) -o $@ $(SRCDIR)/mkcss.c
-
 $(CODECHECK1):	$(SRCDIR)/codecheck1.c
 	$(XBCC) -o $@ $(SRCDIR)/codecheck1.c
 
@@ -1095,11 +1099,11 @@ $(CODECHECK1):	$(SRCDIR)/codecheck1.c
 test:	$(OBJDIR) $(APPNAME)
 	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME)
 
-$(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(MKVERSION)
+$(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(MKVERSION) $(OBJDIR)/phony.h
 	$(MKVERSION) $(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION >$@
 
-$(OBJDIR)/default_css.h:	$(SRCDIR)/default_css.txt $(MKCSS)
-	$(MKCSS) $(SRCDIR)/default_css.txt $@
+$(OBJDIR)/phony.h:
+	# Force rebuild of VERSION.h every time "make" is run
 
 # The USE_SYSTEM_SQLITE variable may be undefined, set to 0, or set
 # to 1. If it is set to 1, then there is no need to build or link
@@ -1166,6 +1170,7 @@ endif
 
 openssl:	$(BLDTARGETS)
 	cd $(OPENSSLLIBDIR);./Configure --cross-compile-prefix=$(PREFIX) $(SSLCONFIG)
+	sed -i -e 's/^PERL=C:\\.*$$/PERL=perl.exe/i' $(OPENSSLLIBDIR)/Makefile
 	$(MAKE) -C $(OPENSSLLIBDIR) PREFIX=$(PREFIX) CC=$(PREFIX)$(TCCEXE) build_libs
 
 clean-openssl:
@@ -1184,9 +1189,9 @@ ifdef FOSSIL_BUILD_SSL
 APPTARGETS += openssl
 endif
 
-$(APPNAME):	$(APPTARGETS) $(OBJDIR)/headers $(CODECHECK1) $(OBJ) $(EXTRAOBJ) $(OBJDIR)/fossil.o
+$(APPNAME):	$(APPTARGETS) $(OBJDIR)/headers $(CODECHECK1) $(EXTRAOBJ) $(OBJ) $(OBJDIR)/fossil.o
 	$(CODECHECK1) $(TRANS_SRC)
-	$(TCC) -o $@ $(OBJ) $(EXTRAOBJ) $(OBJDIR)/fossil.o $(LIB)
+	$(TCC) -o $@ $(EXTRAOBJ) $(OBJ) $(OBJDIR)/fossil.o $(LIB)
 
 # This rule prevents make from using its default rules to try build
 # an executable named "manifest" out of the file named "manifest.c"
@@ -1225,14 +1230,13 @@ writeln "\t\$(MKINDEX) \$(TRANS_SRC) >\$@\n"
 writeln "\$(OBJDIR)/builtin_data.h:\t\$(MKBUILTIN) \$(EXTRA_FILES)"
 writeln "\t\$(MKBUILTIN) --prefix \$(SRCDIR)/ \$(EXTRA_FILES) >\$@\n"
 
-writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/default_css.h \$(MAKEHEADERS) \$(OBJDIR)/VERSION.h"
+writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(MAKEHEADERS) \$(OBJDIR)/VERSION.h"
 writeln "\t\$(MAKEHEADERS) $mhargs"
 writeln "\techo Done >\$(OBJDIR)/headers\n"
 writeln "\$(OBJDIR)/headers: Makefile\n"
 writeln "Makefile:\n"
 set extra_h(main) " \$(OBJDIR)/page_index.h "
 set extra_h(builtin) " \$(OBJDIR)/builtin_data.h "
-set extra_h(style) " \$(OBJDIR)/default_css.h "
 
 foreach s [lsort $src] {
   writeln "\$(OBJDIR)/${s}_.c:\t\$(SRCDIR)/$s.c \$(TRANSLATE)"
@@ -1382,9 +1386,6 @@ mkbuiltin$E: $(SRCDIR)\mkbuiltin.c
 mkversion$E: $(SRCDIR)\mkversion.c
 	$(BCC) -o$@ $**
 
-mkcss$E: $(SRCDIR)\mkcss.c
-	$(BCC) -o$@ $**
-
 codecheck1$E: $(SRCDIR)\codecheck1.c
 	$(BCC) -o$@ $**
 
@@ -1406,9 +1407,6 @@ $(OBJDIR)\cson_amalgamation.h : $(SRCDIR)\cson_amalgamation.h
 VERSION.h : mkversion$E $B\manifest.uuid $B\manifest $B\VERSION
 	+$** > $@
 
-default_css.h : mkcss$E $B\src\default_css.txt
-	+$** $B\src\default_css.txt $@
-
 page_index.h: mkindex$E $(SRC)
 	+$** > $@
 
@@ -1420,7 +1418,7 @@ clean:
 	-del *.obj *_.c *.h *.map
 
 realclean:
-	-del $(APPNAME) translate$E mkindex$E makeheaders$E mkversion$E codecheck1$E mkbuiltin$E mkcss$E
+	-del $(APPNAME) translate$E mkindex$E makeheaders$E mkversion$E codecheck1$E mkbuiltin$E
 
 $(OBJDIR)\json$O : $(SRCDIR)\json_detail.h
 $(OBJDIR)\json_artifact$O : $(SRCDIR)\json_detail.h
@@ -1438,7 +1436,6 @@ $(OBJDIR)\json_timeline$O : $(SRCDIR)\json_detail.h
 $(OBJDIR)\json_user$O : $(SRCDIR)\json_detail.h
 $(OBJDIR)\json_wiki$O : $(SRCDIR)\json_detail.h
 
-
 }
 foreach s [lsort $src] {
   writeln "\$(OBJDIR)\\$s\$O : ${s}_.c ${s}.h"
@@ -1447,7 +1444,7 @@ foreach s [lsort $src] {
   writeln "\t+translate\$E \$** > \$@\n"
 }
 
-writeln -nonewline "headers: makeheaders\$E page_index.h builtin_data.h default_css.h VERSION.h\n\t +makeheaders\$E "
+writeln -nonewline "headers: makeheaders\$E page_index.h builtin_data.h VERSION.h\n\t +makeheaders\$E "
 foreach s [lsort $src] {
   writeln -nonewline "${s}_.c:$s.h "
 }
@@ -1471,36 +1468,55 @@ writeln {#
 # WARNING: DO NOT EDIT, AUTOMATICALLY GENERATED FILE (SEE "src/makemake.tcl")
 ##############################################################################
 #
-# This Makefile will only function correctly if used from a sub-directory
-# that is a direct child of the top-level directory for this project.
-#
-!if !exist("..\.fossil-settings")
-!error "Please change the current directory to the one containing this file."
-!endif
-
 #
 # This file is automatically generated.  Instead of editing this
 # file, edit "makemake.tcl" then run "tclsh makemake.tcl"
 # to regenerate this file.
 #
 B       = ..
-SRCDIR  = $B\src
-OBJDIR  = .
-OX      = .
+SRCDIR  = $(B)\src
+T       = .
+OBJDIR  = $(T)
+OX      = $(OBJDIR)
 O       = .obj
 E       = .exe
 P       = .pdb
+DBGOPTS = /Od
 
-# Perl is only necessary if OpenSSL support is enabled and it must
-# be built from source code.  The PERLDIR variable should point to
-# the directory containing the main Perl binary (i.e. "perl.exe").
-PERLDIR = C:\Perl\bin
+INSTALLDIR = .
+!ifdef DESTDIR
+INSTALLDIR = $(DESTDIR)\$(INSTALLDIR)
+!endif
+
+# When building out of source, this Makefile needs to know the path to the base
+# top-level directory for this project. Pass it on NMAKE command line via make
+# variable B:
+#   NMAKE /f "path\to\this\Makefile" B="path/to/fossil/root"
+#
+# NOTE: Make sure B path has no trailing backslash, UNIX-style path is OK too.
+#
+!if !exist("$(B)\.fossil-settings")
+!error Please specify path to project base directory: B="path/to/fossil"
+!endif
+
+# Perl is only necessary if OpenSSL support is enabled and it is built from
+# source code.  The PERLDIR environment variable, if it exists, should point
+# to the directory containing the main Perl executable specified here (i.e.
+# "perl.exe").
 PERL    = perl.exe
 
+# Enable use of available compiler optimizations?
+!ifndef OPTIMIZATIONS
+OPTIMIZATIONS = 2
+!endif
+
 # Enable debugging symbols?
 !ifndef DEBUG
 DEBUG = 0
 !endif
+!ifdef FOSSIL_DEBUG
+DEBUG = 1
+!endif
 
 # Build the OpenSSL libraries?
 !ifndef FOSSIL_BUILD_SSL
@@ -1527,11 +1543,6 @@ FOSSIL_ENABLE_EXEC_REL_PATHS = 0
 FOSSIL_ENABLE_JSON = 0
 !endif
 
-# Enable legacy treatment of the mv/rm commands?
-!ifndef FOSSIL_ENABLE_LEGACY_MV_RM
-FOSSIL_ENABLE_LEGACY_MV_RM = 1
-!endif
-
 # Enable use of miniz instead of zlib?
 !ifndef FOSSIL_ENABLE_MINIZ
 FOSSIL_ENABLE_MINIZ = 0
@@ -1568,75 +1579,39 @@ USE_SEE = 0
 !endif
 
 !if $(FOSSIL_ENABLE_SSL)!=0
-SSLDIR    = $(B)\compat\openssl-1.0.2p
-SSLINCDIR = $(SSLDIR)\inc32
+SSLDIR    = $(B)\compat\openssl
+SSLINCDIR = $(SSLDIR)\include
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLLIBDIR = $(SSLDIR)\out32dll
+SSLLIBDIR = $(SSLDIR)
 !else
-SSLLIBDIR = $(SSLDIR)\out32
+SSLLIBDIR = $(SSLDIR)
 !endif
 SSLLFLAGS = /nologo /opt:ref /debug
-SSLLIB    = ssleay32.lib libeay32.lib user32.lib gdi32.lib crypt32.lib
+SSLLIB    = libssl.lib libcrypto.lib user32.lib gdi32.lib crypt32.lib
 !if "$(PLATFORM)"=="amd64" || "$(PLATFORM)"=="x64"
 !message Using 'x64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64A no-asm no-ssl2 no-ssl3 no-weak-ssl-ciphers
-SSLCONFIG = VC-WIN64A no-asm
+SSLCONFIG = VC-WIN64A no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_win64a.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3 -DOPENSSL_NO_WEAK_SSL_CIPHERS
-!endif
 !elseif "$(PLATFORM)"=="ia64"
 !message Using 'ia64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64I no-asm no-ssl2 no-ssl3 no-weak-ssl-ciphers
-SSLCONFIG = VC-WIN64I no-asm
+SSLCONFIG = VC-WIN64I no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_win64i.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3 -DOPENSSL_NO_WEAK_SSL_CIPHERS
-!endif
 !else
 !message Assuming 'x86' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN32 no-asm no-ssl2 no-ssl3 no-weak-ssl-ciphers
-SSLCONFIG = VC-WIN32 no-asm
+SSLCONFIG = VC-WIN32 no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_ms.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3 -DOPENSSL_NO_WEAK_SSL_CIPHERS
-!endif
 !endif
 !endif
 
@@ -1656,23 +1631,26 @@ ZLIB      = zdll.lib
 ZLIB      = zlib.lib
 !endif
 
-INCL      = /I. /I$(SRCDIR) /I$B\win\include
+INCL      = /I. /I"$(OX)" /I"$(SRCDIR)" /I"$(B)\win\include"
 
 !if $(FOSSIL_ENABLE_MINIZ)==0
-INCL      = $(INCL) /I$(ZINCDIR)
+INCL      = $(INCL) /I"$(ZINCDIR)"
 !endif
 
 !if $(FOSSIL_ENABLE_SSL)!=0
-INCL      = $(INCL) /I$(SSLINCDIR)
+INCL      = $(INCL) /I"$(SSLINCDIR)"
 !endif
 
 !if $(FOSSIL_ENABLE_TCL)!=0
-INCL      = $(INCL) /I$(TCLINCDIR)
+INCL      = $(INCL) /I"$(TCLINCDIR)"
 !endif
 
 CFLAGS    = /nologo
 LDFLAGS   =
 
+CFLAGS    = $(CFLAGS) /D_CRT_SECURE_NO_DEPRECATE /D_CRT_SECURE_NO_WARNINGS
+CFLAGS    = $(CFLAGS) /D_CRT_NONSTDC_NO_DEPRECATE /D_CRT_NONSTDC_NO_WARNINGS
+
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 LDFLAGS   = $(LDFLAGS) /MANIFEST
 !else
@@ -1704,11 +1682,23 @@ CRTFLAGS = /MT
 !endif
 !endif
 
+!if $(OPTIMIZATIONS)>3
+RELOPTS = /Os
+!elseif $(OPTIMIZATIONS)>2
+RELOPTS = /Ox
+!elseif $(OPTIMIZATIONS)>1
+RELOPTS = /O2
+!elseif $(OPTIMIZATIONS)>0
+RELOPTS = /O1
+!else
+RELOPTS =
+!endif
+
 !if $(DEBUG)!=0
-CFLAGS    = $(CFLAGS) /Zi $(CRTFLAGS) /Od
+CFLAGS    = $(CFLAGS) /Zi $(CRTFLAGS) $(DBGOPTS) /DFOSSIL_DEBUG /DTH_MEMDEBUG
 LDFLAGS   = $(LDFLAGS) /DEBUG
 !else
-CFLAGS    = $(CFLAGS) $(CRTFLAGS) /O2
+CFLAGS    = $(CFLAGS) $(CRTFLAGS) $(RELOPTS)
 !endif
 
 BCC       = $(CC) $(CFLAGS)
@@ -1725,7 +1715,7 @@ RCC       = $(RCC) /DFOSSIL_DYNAMIC_BUILD=1
 
 !if $(FOSSIL_ENABLE_MINIZ)==0
 LIBS      = $(LIBS) $(ZLIB)
-LIBDIR    = $(LIBDIR) /LIBPATH:$(ZLIBDIR)
+LIBDIR    = $(LIBDIR) /LIBPATH:"$(ZLIBDIR)"
 !endif
 
 !if $(FOSSIL_ENABLE_MINIZ)!=0
@@ -1742,7 +1732,7 @@ RCC       = $(RCC) /DFOSSIL_ENABLE_JSON=1
 TCC       = $(TCC) /DFOSSIL_ENABLE_SSL=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_SSL=1
 LIBS      = $(LIBS) $(SSLLIB)
-LIBDIR    = $(LIBDIR) /LIBPATH:$(SSLLIBDIR)
+LIBDIR    = $(LIBDIR) /LIBPATH:"$(SSLLIBDIR)"
 !endif
 
 !if $(FOSSIL_ENABLE_EXEC_REL_PATHS)!=0
@@ -1750,11 +1740,6 @@ TCC       = $(TCC) /DFOSSIL_ENABLE_EXEC_REL_PATHS=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_EXEC_REL_PATHS=1
 !endif
 
-!if $(FOSSIL_ENABLE_LEGACY_MV_RM)!=0
-TCC       = $(TCC) /DFOSSIL_ENABLE_LEGACY_MV_RM=1
-RCC       = $(RCC) /DFOSSIL_ENABLE_LEGACY_MV_RM=1
-!endif
-
 !if $(FOSSIL_ENABLE_TH1_DOCS)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_TH1_DOCS=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_TH1_DOCS=1
@@ -1800,7 +1785,7 @@ foreach s [lsort $src] {
     writeln " \\"
     writeln -nonewline "        "
   }
-  writeln -nonewline "${s}_.c"; incr i
+  writeln -nonewline "\"\$(OX)\\${s}_.c\""; incr i
 }
 writeln "\n"
 writeln -nonewline "EXTRA_FILES   = "
@@ -1811,7 +1796,7 @@ foreach s [lsort $extra_files] {
     writeln -nonewline "        "
   }
   set s [regsub -all / $s \\]
-  writeln -nonewline "\$(SRCDIR)\\${s}"; incr i
+  writeln -nonewline "\"\$(SRCDIR)\\${s}\""; incr i
 }
 writeln "\n"
 set AdditionalObj [list shell sqlite3 th th_lang th_tcl cson_amalgamation]
@@ -1822,22 +1807,39 @@ foreach s [lsort [concat $src $AdditionalObj]] {
     writeln " \\"
     writeln -nonewline "        "
   }
-  writeln -nonewline "\$(OX)\\$s\$O"; incr i
+  writeln -nonewline "\"\$(OX)\\$s\$O\""; incr i
 }
 if {$i > 0} {
   writeln " \\"
 }
 writeln "!if \$(FOSSIL_ENABLE_MINIZ)!=0"
 writeln -nonewline "        "
-writeln "\$(OX)\\miniz\$O \\"; incr i
+writeln "\"\$(OX)\\miniz\$O\" \\"; incr i
 writeln "!endif"
-writeln -nonewline "        \$(OX)\\fossil.res\n\n"
+writeln -nonewline "        \"\$(OX)\\fossil.res\"\n\n"
 writeln [string map [list <<<NEXT_LINE>>> \\] {
-APPNAME    = $(OX)\fossil$(E)
-PDBNAME    = $(OX)\fossil$(P)
-APPTARGETS =
+!ifndef BASEAPPNAME
+BASEAPPNAME = fossil
+!endif
 
-all: $(OX) $(APPNAME)
+APPNAME     = $(OX)\$(BASEAPPNAME)$(E)
+PDBNAME     = $(OX)\$(BASEAPPNAME)$(P)
+APPTARGETS  =
+
+all: "$(OX)" "$(APPNAME)"
+
+$(BASEAPPNAME): "$(APPNAME)"
+
+$(BASEAPPNAME)$(E): "$(APPNAME)"
+
+install: "$(APPNAME)"
+	echo F | xcopy /Y "$(APPNAME)" "$(INSTALLDIR)"\*
+!if $(DEBUG)!=0
+	echo F | xcopy /Y "$(PDBNAME)" "$(INSTALLDIR)"\*
+!endif
+
+$(OX):
+	@-mkdir $@
 
 zlib:
 	@echo Building zlib from "$(ZLIBDIR)"...
@@ -1847,19 +1849,25 @@ zlib:
 	@pushd "$(ZLIBDIR)" && $(MAKE) /f win32\Makefile.msc $(ZLIB) && popd
 !endif
 
+clean-zlib:
+	@pushd "$(ZLIBDIR)" && $(MAKE) /f win32\Makefile.msc clean && popd
+
 !if $(FOSSIL_ENABLE_SSL)!=0
 openssl:
 	@echo Building OpenSSL from "$(SSLDIR)"...
-!if "$(PERLDIR)" != ""
-	@set PATH=$(PERLDIR);$(PATH)
-!endif
-	@pushd "$(SSLDIR)" && $(PERL) Configure $(SSLCONFIG) && popd
-	@pushd "$(SSLDIR)" && call $(SSLSETUP) && popd
-!if $(FOSSIL_ENABLE_WINXP)!=0
-	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS) $(XPCFLAGS)" "LFLAGS=$(SSLLFLAGS) $(XPLDFLAGS)" && popd
+!ifdef PERLDIR
+	@pushd "$(SSLDIR)" && "$(PERLDIR)\$(PERL)" Configure $(SSLCONFIG) && popd
 !else
-	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS)" && popd
+	@pushd "$(SSLDIR)" && "$(PERL)" Configure $(SSLCONFIG) && popd
 !endif
+!if $(FOSSIL_ENABLE_WINXP)!=0
+	@pushd "$(SSLDIR)" && $(MAKE) "CC=cl $(XPCFLAGS)" "LFLAGS=$(XPLDFLAGS)" && popd
+!else
+	@pushd "$(SSLDIR)" && $(MAKE) && popd
+!endif
+
+clean-openssl:
+	@pushd "$(SSLDIR)" && $(MAKE) clean && popd
 !endif
 
 !if $(FOSSIL_ENABLE_MINIZ)==0
@@ -1874,48 +1882,41 @@ APPTARGETS = $(APPTARGETS) openssl
 !endif
 !endif
 
-$(APPNAME) : $(APPTARGETS) translate$E mkindex$E codecheck1$E headers $(OBJ) $(OX)\linkopts
-	cd $(OX)
-	codecheck1$E $(SRC)
-	link $(LDFLAGS) /OUT:$@ $(LIBDIR) Wsetargv.obj fossil.res @linkopts
-	if exist $@.manifest <<<NEXT_LINE>>>
-		$(MTC) -nologo -manifest $@.manifest -outputresource:$@;1
+"$(APPNAME)" : $(APPTARGETS) "$(OBJDIR)\translate$E" "$(OBJDIR)\mkindex$E" "$(OBJDIR)\codecheck1$E" "$(OX)\headers" $(OBJ) "$(OX)\linkopts"
+	"$(OBJDIR)\codecheck1$E" $(SRC)
+	link $(LDFLAGS) /OUT:$@ /PDB:$(@D)\ $(LIBDIR) Wsetargv.obj "$(OX)\fossil.res" @"$(OX)\linkopts"
+	if exist "$(B)\win\fossil.exe.manifest" <<<NEXT_LINE>>>
+		$(MTC) -nologo -manifest "$(B)\win\fossil.exe.manifest" -outputresource:$@;1
 
-$(OX)\linkopts: $B\win\Makefile.msc}]
+"$(OX)\linkopts": "$(B)\win\Makefile.msc"}]
 set redir {>}
 foreach s [lsort [concat $src $AdditionalObj]] {
-  writeln "\techo \$(OX)\\$s.obj $redir \$@"
+  writeln "\techo \"\$(OX)\\$s.obj\" $redir \$@"
   set redir {>>}
 }
 set redir {>>}
 writeln "!if \$(FOSSIL_ENABLE_MINIZ)!=0"
-writeln "\techo \$(OX)\\miniz.obj $redir \$@"
+writeln "\techo \"\$(OX)\\miniz.obj\" $redir \$@"
 writeln "!endif"
 writeln "\techo \$(LIBS) $redir \$@"
 writeln {
-$(OX):
-	@-mkdir $@
+"$(OBJDIR)\translate$E": "$(SRCDIR)\translate.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
-translate$E: $(SRCDIR)\translate.c
-	$(BCC) $**
+"$(OBJDIR)\makeheaders$E": "$(SRCDIR)\makeheaders.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
-makeheaders$E: $(SRCDIR)\makeheaders.c
-	$(BCC) $**
+"$(OBJDIR)\mkindex$E": "$(SRCDIR)\mkindex.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
-mkindex$E: $(SRCDIR)\mkindex.c
-	$(BCC) $**
+"$(OBJDIR)\mkbuiltin$E": "$(SRCDIR)\mkbuiltin.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
-mkbuiltin$E: $(SRCDIR)\mkbuiltin.c
-	$(BCC) $**
+"$(OBJDIR)\mkversion$E": "$(SRCDIR)\mkversion.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
-mkversion$E: $(SRCDIR)\mkversion.c
-	$(BCC) $**
-
-mkcss$E: $(SRCDIR)\mkcss.c
-	$(BCC) $**
-
-codecheck1$E: $(SRCDIR)\codecheck1.c
-	$(BCC) $**
+"$(OBJDIR)\codecheck1$E": "$(SRCDIR)\codecheck1.c"
+	$(BCC) /Fe$@ /Fo$(@D)\ /Fd$(@D)\ $**
 
 !if $(USE_SEE)!=0
 SEE_FLAGS = /DSQLITE_HAS_CODEC=1 /DSQLITE_SHELL_DBKEY_PROC=fossil_key
@@ -1927,110 +1928,122 @@ SQLITE3_SHELL_SRC = $(SRCDIR)\shell.c
 SQLITE3_SRC = $(SRCDIR)\sqlite3.c
 !endif
 
-$(OX)\shell$O : $(SQLITE3_SHELL_SRC) $B\win\Makefile.msc
-	$(TCC) /Fo$@ $(SHELL_OPTIONS) $(SQLITE_OPTIONS) $(SHELL_CFLAGS) $(SEE_FLAGS) -c $(SQLITE3_SHELL_SRC)
+"$(OX)\shell$O" : "$(SQLITE3_SHELL_SRC)" "$(B)\win\Makefile.msc"
+	$(TCC) /Fo$@ /Fd$(@D)\ $(SHELL_OPTIONS) $(SQLITE_OPTIONS) $(SHELL_CFLAGS) $(SEE_FLAGS) -c "$(SQLITE3_SHELL_SRC)"
 
-$(OX)\sqlite3$O : $(SQLITE3_SRC) $B\win\Makefile.msc
-	$(TCC) /Fo$@ -c $(SQLITE_OPTIONS) $(SQLITE_CFLAGS) $(SEE_FLAGS) $(SQLITE3_SRC)
+"$(OX)\sqlite3$O" : "$(SQLITE3_SRC)" "$(B)\win\Makefile.msc"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $(SQLITE_OPTIONS) $(SQLITE_CFLAGS) $(SEE_FLAGS) "$(SQLITE3_SRC)"
 
-$(OX)\th$O : $(SRCDIR)\th.c
-	$(TCC) /Fo$@ -c $**
+"$(OX)\th$O" : "$(SRCDIR)\th.c"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $**
 
-$(OX)\th_lang$O : $(SRCDIR)\th_lang.c
-	$(TCC) /Fo$@ -c $**
+"$(OX)\th_lang$O" : "$(SRCDIR)\th_lang.c"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $**
 
-$(OX)\th_tcl$O : $(SRCDIR)\th_tcl.c
-	$(TCC) /Fo$@ -c $**
+"$(OX)\th_tcl$O" : "$(SRCDIR)\th_tcl.c"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $**
 
-$(OX)\miniz$O : $(SRCDIR)\miniz.c
-	$(TCC) /Fo$@ -c $(MINIZ_OPTIONS) $(SRCDIR)\miniz.c
+"$(OX)\miniz$O" : "$(SRCDIR)\miniz.c"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $(MINIZ_OPTIONS) $**
 
-VERSION.h : mkversion$E $B\manifest.uuid $B\manifest $B\VERSION
+"$(OX)\VERSION.h" : "$(OBJDIR)\mkversion$E" "$(B)\manifest.uuid" "$(B)\manifest" "$(B)\VERSION" "$(B)\phony.h"
+	"$(OBJDIR)\mkversion$E" "$(B)\manifest.uuid" "$(B)\manifest" "$(B)\VERSION" > $@
+
+"$(B)\phony.h" :
+	rem Force rebuild of VERSION.h whenever nmake is run
+
+"$(OX)\cson_amalgamation$O" : "$(SRCDIR)\cson_amalgamation.c"
+	$(TCC) /Fo$@ /Fd$(@D)\ -c $**
+
+"$(OX)\page_index.h": "$(OBJDIR)\mkindex$E" $(SRC)
 	$** > $@
 
-$(OX)\cson_amalgamation$O : $(SRCDIR)\cson_amalgamation.c
-	$(TCC) /Fo$@ /c $**
+"$(OX)\builtin_data.h":	"$(OBJDIR)\mkbuiltin$E" "$(OX)\builtin_data.reslist"
+	"$(OBJDIR)\mkbuiltin$E" --prefix "$(SRCDIR)/" --reslist "$(OX)\builtin_data.reslist" > $@
 
-default_css.h: mkcss$E $(SRCDIR)\default_css.txt
-	$** $@
+cleanx:
+	-del "$(OX)\*.obj" 2>NUL
+	-del "$(OBJDIR)\*.obj" 2>NUL
+	-del "$(OX)\*_.c" 2>NUL
+	-del "$(OX)\*.h" 2>NUL
+	-del "$(OX)\*.ilk" 2>NUL
+	-del "$(OX)\*.map" 2>NUL
+	-del "$(OX)\*.res" 2>NUL
+	-del "$(OX)\*.reslist" 2>NUL
+	-del "$(OX)\headers" 2>NUL
+	-del "$(OX)\linkopts" 2>NUL
+	-del "$(OX)\vc*.pdb" 2>NUL
 
-page_index.h: mkindex$E $(SRC)
-	$** > $@
-
-builtin_data.h:	mkbuiltin$E $(EXTRA_FILES)
-	mkbuiltin$E --prefix $(SRCDIR)/ $(EXTRA_FILES) > $@
-
-clean:
-	-del $(OX)\*.obj 2>NUL
-	-del *.obj 2>NUL
-	-del *_.c 2>NUL
-	-del *.h 2>NUL
-	-del *.ilk 2>NUL
-	-del *.map 2>NUL
-	-del *.res 2>NUL
-	-del headers 2>NUL
-	-del linkopts 2>NUL
-	-del vc*.pdb 2>NUL
+clean: cleanx
+	-del "$(APPNAME)" 2>NUL
+	-del "$(PDBNAME)" 2>NUL
+	-del "$(OBJDIR)\translate$E" 2>NUL
+	-del "$(OBJDIR)\translate$P" 2>NUL
+	-del "$(OBJDIR)\mkindex$E" 2>NUL
+	-del "$(OBJDIR)\mkindex$P" 2>NUL
+	-del "$(OBJDIR)\makeheaders$E" 2>NUL
+	-del "$(OBJDIR)\makeheaders$P" 2>NUL
+	-del "$(OBJDIR)\mkversion$E" 2>NUL
+	-del "$(OBJDIR)\mkversion$P" 2>NUL
+	-del "$(OBJDIR)\mkcss$E" 2>NUL
+	-del "$(OBJDIR)\mkcss$P" 2>NUL
+	-del "$(OBJDIR)\codecheck1$E" 2>NUL
+	-del "$(OBJDIR)\codecheck1$P" 2>NUL
+	-del "$(OBJDIR)\mkbuiltin$E" 2>NUL
+	-del "$(OBJDIR)\mkbuiltin$P" 2>NUL
 
 realclean: clean
-	-del $(APPNAME) 2>NUL
-	-del $(PDBNAME) 2>NUL
-	-del translate$E 2>NUL
-	-del translate$P 2>NUL
-	-del mkindex$E 2>NUL
-	-del mkindex$P 2>NUL
-	-del makeheaders$E 2>NUL
-	-del makeheaders$P 2>NUL
-	-del mkversion$E 2>NUL
-	-del mkversion$P 2>NUL
-	-del mkcss$E 2>NUL
-	-del mkcss$P 2>NUL
-	-del codecheck1$E 2>NUL
-	-del codecheck1$P 2>NUL
-	-del mkbuiltin$E 2>NUL
-	-del mkbuiltin$P 2>NUL
 
-$(OBJDIR)\json$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_artifact$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_branch$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_config$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_diff$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_dir$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_finfo$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_login$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_query$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_report$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_status$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_tag$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_timeline$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_user$O : $(SRCDIR)\json_detail.h
-$(OBJDIR)\json_wiki$O : $(SRCDIR)\json_detail.h
+"$(OBJDIR)\json$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_artifact$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_branch$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_config$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_diff$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_dir$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_finfo$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_login$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_query$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_report$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_status$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_tag$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_timeline$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_user$O" : "$(SRCDIR)\json_detail.h"
+"$(OBJDIR)\json_wiki$O" : "$(SRCDIR)\json_detail.h"
 }
+
+writeln {"$(OX)\builtin_data.reslist": $(EXTRA_FILES) "$(B)\win\Makefile.msc"}
+set redir {>}
+foreach s [lsort $extra_files] {
+  writeln "\techo \"\$(SRCDIR)\\${s}\" $redir \$@"
+  set redir {>>}
+}
+
+writeln ""
 foreach s [lsort $src] {
-  writeln "\$(OX)\\$s\$O : ${s}_.c ${s}.h"
-  writeln "\t\$(TCC) /Fo\$@ -c ${s}_.c\n"
-  writeln "${s}_.c : \$(SRCDIR)\\$s.c"
-  writeln "\ttranslate\$E \$** > \$@\n"
+  writeln "\"\$(OX)\\$s\$O\" : \"\$(OX)\\${s}_.c\" \"\$(OX)\\${s}.h\""
+  writeln "\t\$(TCC) /Fo\$@ /Fd\$(@D)\\ -c \"\$(OX)\\${s}_.c\"\n"
+  writeln "\"\$(OX)\\${s}_.c\" : \"\$(SRCDIR)\\$s.c\""
+  writeln "\t\"\$(OBJDIR)\\translate\$E\" \$** > \$@\n"
 }
 
-writeln "fossil.res : \$B\\win\\fossil.rc"
-writeln "\t\$(RCC)  /fo \$@ \$**\n"
+writeln "\"\$(OX)\\fossil.res\" : \"\$(B)\\win\\fossil.rc\""
+writeln "\t\$(RCC) /fo \$@ \$**\n"
 
-writeln "headers: makeheaders\$E page_index.h builtin_data.h default_css.h VERSION.h"
-writeln -nonewline "\tmakeheaders\$E "
+writeln "\"\$(OX)\\headers\": \"\$(OBJDIR)\\makeheaders\$E\" \"\$(OX)\\page_index.h\" \"\$(OX)\\builtin_data.h\" \"\$(OX)\\VERSION.h\""
+writeln -nonewline "\t\"\$(OBJDIR)\\makeheaders\$E\" "
 set i 0
 foreach s [lsort $src] {
   if {$i > 0} {
     writeln " \\"
     writeln -nonewline "\t\t\t"
   }
-  writeln -nonewline "${s}_.c:$s.h"; incr i
+  writeln -nonewline "\"\$(OX)\\${s}_.c\":\"\$(OX)\\$s.h\""; incr i
 }
-writeln " \\\n\t\t\t\$(SRCDIR)\\sqlite3.h \\"
-writeln "\t\t\t\$(SRCDIR)\\th.h \\"
-writeln "\t\t\tVERSION.h \\"
-writeln "\t\t\t\$(SRCDIR)\\cson_amalgamation.h"
-writeln "\t@copy /Y nul: headers"
+writeln " \\\n\t\t\t\"\$(SRCDIR)\\sqlite3.h\" \\"
+writeln "\t\t\t\"\$(SRCDIR)\\th.h\" \\"
+writeln "\t\t\t\"\$(OX)\\VERSION.h\" \\"
+writeln "\t\t\t\"\$(SRCDIR)\\cson_amalgamation.h\""
+writeln "\t@copy /Y nul: $@"
 
 
 close $output_file
@@ -2198,11 +2211,8 @@ builtin_data.h:	$(EXTRA_FILES) mkbuiltin.exe
 VERSION.h:	version.exe ..\manifest.uuid ..\manifest ..\VERSION
 	version.exe ..\manifest.uuid ..\manifest ..\VERSION  >$@
 
-default_css.h:	mkcss.exe default_css.txt
-	mkcss.exe default_css.txt $@
-
 # generate the simplified headers
-headers: makeheaders.exe page_index.h builtin_data.h default_css.h VERSION.h ../src/sqlite3.h ../src/th.h
+headers: makeheaders.exe page_index.h builtin_data.h VERSION.h ../src/sqlite3.h ../src/th.h
 	makeheaders.exe $(foreach ts,$(TRANSLATEDSRC),$(ts):$(ts:_.c=.h)) ../src/sqlite3.h ../src/th.h VERSION.h
 	echo Done >$@
 
