@@ -34,21 +34,80 @@ static int hasAnyCap(const char *zCap, const char *zTest){
   return 0;
 }
 
+/*
+** Parse the content-security-policy
+** into separate fields, and return a pointer to a null-terminated
+** array of pointers to strings, one entry for each field.  Or return
+** a NULL pointer if no CSP could be located in the header.
+**
+** Memory to hold the returned array and of the strings is obtained from
+** a single memory allocation, which the caller should free to avoid a
+** memory leak.
+*/
+static char **parse_content_security_policy(void){
+  char **azCSP = 0;
+  int nCSP = 0;
+  char *zAll;
+  char *zCopy;
+  int nAll = 0;
+  int jj;
+  int nSemi;
+
+  zAll = style_csp(0);
+  nAll = (int)strlen(zAll);
+  for(jj=nSemi=0; jj<nAll; jj++){ if( zAll[jj]==';' ) nSemi++; }
+  azCSP = fossil_malloc( nAll+1+(nSemi+2)*sizeof(char*) );
+  zCopy = (char*)&azCSP[nSemi+2];
+  memcpy(zCopy,zAll,nAll);
+  zCopy[nAll] = 0;
+  while( fossil_isspace(zCopy[0]) || zCopy[0]==';' ){ zCopy++; }
+  azCSP[0] = zCopy;
+  nCSP = 1;
+  for(jj=0; zCopy[jj]; jj++){
+    if( zCopy[jj]==';' ){
+      int k;
+      for(k=jj-1; k>0 && fossil_isspace(zCopy[k]); k--){ zCopy[k] = 0; }
+      zCopy[jj] = 0;
+      while( jj+1<nAll
+         && (fossil_isspace(zCopy[jj+1]) || zCopy[jj+1]==';')
+      ){
+        jj++;
+      }
+      assert( nCSP<nSemi+1 );
+      azCSP[nCSP++] = zCopy+jj;
+    }
+  }
+  assert( nCSP<=nSemi+2 );
+  azCSP[nCSP] = 0;
+  fossil_free(zAll);
+  return azCSP;
+}
 
 /*
 ** WEBPAGE: secaudit0
 **
-** Run a security audit of the current Fossil setup.
-** This page requires administrator access
+** Run a security audit of the current Fossil setup, looking
+** for configuration problems that might allow unauthorized
+** access to the repository.
+**
+** This page requires administrator access.  It is usually
+** accessed using the Admin/Security-Audit menu option
+** from any of the default skins.
 */
 void secaudit0_page(void){
   const char *zAnonCap;      /* Capabilities of user "anonymous" and "nobody" */
+  const char *zDevCap;       /* Capabilities of user group "developer" */
+  const char *zReadCap;      /* Capabilities of user group "reader" */
   const char *zPubPages;     /* GLOB pattern for public pages */
+  const char *zSelfCap;      /* Capabilities of self-registered users */
+  int hasSelfReg = 0;        /* True if able to self-register */
   char *z;
   int n;
+  CapabilityString *pCap;
+  char **azCSP;              /* Parsed content security policy */
 
   login_check_credentials();
-  if( !g.perm.Setup && !g.perm.Admin ){
+  if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
@@ -61,7 +120,14 @@ void secaudit0_page(void){
   ** though some content may be accessible anonymously.
   */
   zAnonCap = db_text("", "SELECT fullcap(NULL)");
+  zDevCap  = db_text("", "SELECT fullcap('v')");
+  zReadCap = db_text("", "SELECT fullcap('u')");
   zPubPages = db_get("public-pages",0);
+  hasSelfReg = db_get_boolean("self-register",0);
+  pCap = capability_add(0, db_get("default-perms","u"));
+  capability_expand(pCap);
+  zSelfCap = capability_string(pCap);
+  capability_free(pCap);
   if( hasAnyCap(zAnonCap,"as") ){
     @ <li><p>This repository is <big><b>Wildly INSECURE</b></big> because
     @ it grants administrator privileges to anonymous users.  You
@@ -69,43 +135,67 @@ void secaudit0_page(void){
     @ immediately!  Or, at least remove the Setup and Admin privileges
     @ for users "anonymous" and "login" on the
     @ <a href="setup_ulist">User Configuration</a> page.
+  }else if( hasAnyCap(zSelfCap,"as") && hasSelfReg ){
+    @ <li><p>This repository is <big><b>Wildly INSECURE</b></big> because
+    @ it grants administrator privileges to self-registered users.  You
+    @ should <a href="takeitprivate">take this repository private</a>
+    @ and/or disable self-registration
+    @ immediately!  Or, at least remove the Setup and Admin privileges
+    @ from the default permissions for new users.
   }else if( hasAnyCap(zAnonCap,"y") ){
     @ <li><p>This repository is <big><b>INSECURE</b></big> because
     @ it allows anonymous users to push unversioned files.
     @ <p>Fix this by <a href="takeitprivate">taking the repository private</a>
     @ or by removing the "y" permission from users "anonymous" and
     @ "nobody" on the <a href="setup_ulist">User Configuration</a> page.
+  }else if( hasAnyCap(zSelfCap,"y") ){
+    @ <li><p>This repository is <big><b>INSECURE</b></big> because
+    @ it allows self-registered users to push unversioned files.
+    @ <p>Fix this by <a href="takeitprivate">taking the repository private</a>
+    @ or by removing the "y" permission from the default permissions or
+    @ by disabling self-registration.
   }else if( hasAnyCap(zAnonCap,"goz") ){
     @ <li><p>This repository is <big><b>PUBLIC</b></big>. All
     @ checked-in content can be accessed by anonymous users.
     @ <a href="takeitprivate">Take it private</a>.<p>
+  }else if( hasAnyCap(zSelfCap,"goz") && hasSelfReg ){
+    @ <li><p>This repository is <big><b>PUBLIC</b></big> because all
+    @ checked-in content can be accessed by self-registered users.
+    @ This repostory would be private if you disabled self-registration.</p>
   }else if( !hasAnyCap(zAnonCap, "jrwy234567")
+         && (!hasSelfReg || !hasAnyCap(zSelfCap, "jrwy234567"))
          && (zPubPages==0 || zPubPages[0]==0) ){
     @ <li><p>This repository is <big><b>Completely PRIVATE</b></big>.
     @ A valid login and password is required to access any content.
   }else{
     @ <li><p>This repository is <big><b>Mostly PRIVATE</b></big>.
     @ A valid login and password is usually required, however some
-    @ content can be accessed anonymously:
+    @ content can be accessed either anonymously or by self-registered
+    @ users:
     @ <ul>
-    if( hasAnyCap(zAnonCap,"j") ){
-      @ <li> Wiki pages
-    }
-    if( hasAnyCap(zAnonCap,"r") ){
-      @ <li> Tickets
-    }
-    if( hasAnyCap(zAnonCap,"234567") ){
-      @ <li> Forum posts
+    if( hasSelfReg ){
+      if( hasAnyCap(zAnonCap,"j") || hasAnyCap(zSelfCap,"j") ){
+        @ <li> Wiki pages
+      }
+      if( hasAnyCap(zAnonCap,"r") || hasAnyCap(zSelfCap,"r") ){
+        @ <li> Tickets
+      }
+      if( hasAnyCap(zAnonCap,"234567") || hasAnyCap(zSelfCap,"234567") ){
+        @ <li> Forum posts
+      }
     }
     if( zPubPages && zPubPages[0] ){
       Glob *pGlob = glob_create(zPubPages);
       int i;
-      @ <li> URLs that match any of these GLOB patterns:
-      @ <ul>
+      @ <li> "Public Pages" are URLs that match any of these GLOB patterns:
+      @ <p><ul>
       for(i=0; i<pGlob->nPattern; i++){
         @ <li> %h(pGlob->azPattern[i])
       }
       @ </ul>
+      @ <p>Anoymous users are vested with capabilities "%h(zSelfCap)" on
+      @ public pages. See the "Public Pages" entry in the
+      @ "User capability summary" below.
     }
     @ </ul>
     if( zPubPages && zPubPages[0] ){
@@ -114,16 +204,48 @@ void secaudit0_page(void){
     }
   }
 
-  /* Make sure the HTTPS is required for login, so that the password
-  ** does not go across the internet in the clear.
+  /* Make sure the HTTPS is required for login, at least, so that the
+  ** password does not go across the Internet in the clear.
   */
-  if( db_get_boolean("redirect-to-https",0)==0 ){
+  if( db_get_int("redirect-to-https",0)==0 ){
     @ <li><p><b>WARNING:</b>
-    @ Login passwords can be sent over an unencrypted connection.
-    @ <p>Fix this by activating the "Redirect to HTTPS on the Login page"
-    @ setting on the <a href="setup_access">Access Control</a> page.
+    @ Sensitive material such as login passwords can be sent over an
+    @ unencrypted connection.
+    @ <p>Fix this by changing the "Redirect to HTTPS" setting on the
+    @ <a href="setup_access">Access Control</a> page. If you were using
+    @ the old "Redirect to HTTPS on Login Page" setting, switch to the
+    @ new setting: it has a more secure implementation.
   }
 
+#ifdef FOSSIL_ENABLE_TH1_DOCS
+  /* The use of embedded TH1 is dangerous.  Warn if it is possible.
+  */
+  if( !Th_AreDocsEnabled() ){
+    @ <li><p>
+    @ This server is compiled with -DFOSSIL_ENABLE_TH1_DOCS. TH1 docs
+    @ are disabled for this particular repository, so you are safe for
+    @ now.  However, to prevent future problems caused by accidentally
+    @ enabling TH1 docs in the future, it is recommended that you
+    @ recompile Fossil without the -DFOSSIL_ENABLE_TH1_DOCS flag.</p>
+  }else{
+    @ <li><p><b>DANGER:</b>
+    @ This server is compiled with -DFOSSIL_ENABLE_TH1_DOCS and TH1 docs
+    @ are enabled for this repository.  Anyone who can check-in or push
+    @ to this repository can create a malicious TH1 script and then cause
+    @ that script to be run on the server. This is a serious security concern.
+    @ TH1 docs should only be enabled for repositories with a very limited
+    @ number of trusted committers, and the repository should be monitored
+    @ closely to ensure no hostile content sneaks in.  If a bad TH1 script
+    @ does make it into the repository, the only want to prevent it from
+    @ being run is to shun it.</p>
+    @
+    @ <p>Disable TH1 docs by recompiling Fossil without the
+    @ -DFOSSIL_ENABLE_TH1_DOCS flag, and/or clear the th1-docs setting
+    @ and ensure that the TH1_ENABLE_DOCS environment variable does not
+    @ exist in the environment.</p>
+  }
+#endif
+
   /* Anonymous users should not be able to harvest email addresses
   ** from tickets.
   */
@@ -162,15 +284,35 @@ void secaudit0_page(void){
     @ on the <a href="setup_ulist">User Configuration</a> page.
   }
 
-  /* Anonymous users probably should not be allowed to delete
-  ** wiki or tickets.
+  /* Check to see if any TH1 scripts are configured to run on a sync
   */
-  if( hasAnyCap(zAnonCap, "d") ){
+  if( db_exists("SELECT 1 FROM config WHERE name GLOB 'xfer-*-script'"
+                " AND length(value)>0") ){
     @ <li><p><b>WARNING:</b>
-    @ Anonymous users can delete wiki and tickets.
-    @ <p>Fix this by removing the "Delete"
-    @ privilege from users "anonymous" and "nobody" on the
-    @ <a href="setup_ulist">User Configuration</a> page.
+    @ TH1 scripts might be configured to run on any sync, push, pull, or
+    @ clone operation.  See the the <a href="%R/xfersetup">/xfersetup</a>
+    @ page for more information.  These TH1 scripts are a potential
+    @ security concern and so should be carefully audited by a human.
+  } 
+
+  /* The strict-manifest-syntax setting should be on. */
+  if( db_get_boolean("strict-manifest-syntax",1)==0 ){
+    @ <li><p><b>WARNING:</b>
+    @ The "strict-manifest-syntax"  flag is off.  This is a security
+    @ risk.  Turn this setting on (its default) to protect the users
+    @ of this repository.
+  }
+
+  /* Obsolete:  */
+  if( hasAnyCap(zAnonCap, "d") ||
+      hasAnyCap(zDevCap,  "d") ||
+      hasAnyCap(zReadCap, "d") ){
+    @ <li><p><b>WARNING:</b>
+    @ One or more users has the <a
+    @ href="https://fossil-scm.org/forum/forumpost/43c78f4bef">obsolete</a>
+    @ "d" capability. You should remove it using the
+    @ <a href="setup_ulist">User Configuration</a> page in case we
+    @ ever reuse the letter for another purpose.
   }
 
   /* If anonymous users are allowed to create new Wiki, then
@@ -258,7 +400,7 @@ void secaudit0_page(void){
       @ <li><p><b>WARNING:</b>
       @ Administrator privilege is granted to
       @ <a href='setup_ulist?with=as'>%d(n) users</a>.
-      @ Ideally, administator privilege ('s' or 'a') should only
+      @ Ideally, administrator privilege ('s' or 'a') should only
       @ be granted to one or two users.
     }
   }
@@ -345,7 +487,7 @@ void secaudit0_page(void){
     @ filesystem is mounted within the jail, so that the load average
     @ can be obtained from the /proc/loadavg file.
   }else {
-    double r = atof(db_get("max-loadavg", "0"));
+    double r = atof(db_get("max-loadavg", 0));
     if( r<=0.0 ){
       @ <li><p>
       @ Load average limiting is turned off.  This can cause the server
@@ -367,13 +509,14 @@ void secaudit0_page(void){
   if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
     @ <li><p>
     @ The server error log is disabled.
-    @ To set up an error log:
-    @ <ul>
-    @ <li>If running from CGI, make an entry "errorlog: <i>FILENAME</i>"
-    @ in the CGI script.
-    @ <li>If running the "fossil server" or "fossil http" commands,
-    @ add the "--errorlog <i>FILENAME</i>" command-line option.
-    @ </ul>
+    @ To set up an error log,
+    if( fossil_strcmp(g.zCmdName, "cgi")==0 ){
+      @ make an entry like "errorlog: <i>FILENAME</i>" in the
+      @ CGI script at %h(P("SCRIPT_FILENAME")).
+    }else{
+      @ add the "--errorlog <i>FILENAME</i>" option to the 
+      @ "%h(g.argv[0]) %h(g.zCmdName)" command that launched this server.
+    }
   }else{
     FILE *pTest = fossil_fopen(g.zErrlog,"a");
     if( pTest==0 ){
@@ -384,14 +527,52 @@ void secaudit0_page(void){
     }else{
       fclose(pTest);
       @ <li><p>
-      @ The error log at "<a href='%R/errorlog'>%h(g.zErrlog)</a>" that is
+      @ The error log at "<a href='%R/errorlog'>%h(g.zErrlog)</a>" is
       @ %,lld(file_size(g.zErrlog, ExtFILE)) bytes in size.
     }
   }
 
+  if( g.zExtRoot ){
+    int nFile;
+    int nCgi;
+    ext_files();
+    nFile = db_int(0, "SELECT count(*) FROM sfile");
+    nCgi = nFile==0 ? 0 : db_int(0,"SELECT count(*) FROM sfile WHERE isexe");
+    @ <li><p> CGI Extensions are enabled with a document root
+    @ at <a href='%R/extfilelist'>%h(g.zExtRoot)</a> holding
+    @ %d(nCgi) CGIs and %d(nFile-nCgi) static content and data files.
+  }
+
+  if( fileedit_glob()!=0 ){
+    @ <li><p><a href='%R/fileedit'>Online File Editing</a> is enabled
+    @ for this repository.  Clear the 
+    @ <a href='%R/setup_settings'>"fileedit-glob" setting</a> to
+    @ disable online editing.</p>
+  }
+
   @ <li><p> User capability summary:
   capability_summary();
 
+
+  azCSP = parse_content_security_policy();
+  if( azCSP==0 ){
+    @ <li><p> WARNING: No Content Security Policy (CSP) is specified in the
+    @ header. Though not required, a strong CSP is recommended. Fossil will
+    @ automatically insert an appropriate CSP if you let it generate the
+    @ HTML <tt>&lt;head&gt;</tt> element by omitting <tt>&lt;body&gt;</tt>
+    @ from the header configuration in your customized skin.
+    @ 
+  }else{
+    int ii;
+    @ <li><p> Content Security Policy:
+    @ <ol type="a">
+    for(ii=0; azCSP[ii]; ii++){
+      @ <li>%h(azCSP[ii])
+    }
+    @ </ol>
+  }
+  fossil_free(azCSP);
+
   if( alert_enabled() ){
     @ <li><p> Email alert configuration summary:
     @ <table class="label-value">
@@ -401,8 +582,29 @@ void secaudit0_page(void){
     @ <li><p> Email alerts are disabled
   }
 
+  n = db_int(0,"SELECT count(*) FROM ("
+               "SELECT rid FROM phantom EXCEPT SELECT rid FROM private)");
+  if( n>0 ){
+    @ <li><p>\
+    @ There exists public phantom artifacts in this repository, shown below.
+    @ Phantom artifacts are artifacts whose hash name is referenced by some
+    @ other artifact but whose content is unknown.  Some phantoms are marked
+    @ private and those are ignored.  But public phantoms cause unnecessary
+    @ sync traffic and might represent malicious attempts to corrupt the
+    @ repository structure.
+    @ </p><p>
+    @ To suppress unnecessary sync traffic caused by phantoms, add the RID
+    @ of each phantom to the "private" table.  Example:
+    @ <blockquote><pre>
+    @    INSERT INTO private SELECT rid FROM blob WHERE content IS NULL;
+    @ </pre></blockquote>
+    @ </p>
+    table_of_public_phantoms();
+    @ </li>
+  }
+
   @ </ol>
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -412,7 +614,7 @@ void secaudit0_page(void){
 */
 void takeitprivate_page(void){
   login_check_credentials();
-  if( !g.perm.Setup && !g.perm.Admin ){
+  if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
@@ -421,11 +623,14 @@ void takeitprivate_page(void){
     cgi_redirect("secaudit0");
   }
   if( P("apply") ){
+    db_unprotect(PROTECT_ALL);
     db_multi_exec(
       "UPDATE user SET cap=''"
       " WHERE login IN ('nobody','anonymous');"
       "DELETE FROM config WHERE name='public-pages';"
     );
+    db_set("self-register","0",0);
+    db_protect_pop();
     cgi_redirect("secaudit0");
   }
   style_header("Make This Website Private");
@@ -441,7 +646,7 @@ void takeitprivate_page(void){
   @ <input type="submit" name="cancel" value="Cancel">
   @ </form>
 
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -460,7 +665,7 @@ void errorlog_page(void){
   FILE *in;
   char z[10000];
   login_check_credentials();
-  if( !g.perm.Setup && !g.perm.Admin ){
+  if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
@@ -482,7 +687,7 @@ void errorlog_page(void){
     @ a command-line option "--errorlog <i>FILENAME</i>" to that
     @ command.
     @ </ol>
-    style_footer();
+    style_finish_page();
     return;
   }
   if( P("truncate1") && cgi_csrf_safe(1) ){
@@ -502,7 +707,7 @@ void errorlog_page(void){
     @ <input type="submit" name="truncate1" value="Confirm">
     @ <input type="submit" name="cancel" value="Cancel">
     @ </form>
-    style_footer();
+    style_finish_page();
     return;
   }
   @ <p>The server error log at "%h(g.zErrlog)" is %,lld(szFile) bytes in size.
@@ -510,8 +715,8 @@ void errorlog_page(void){
   style_submenu_element("Truncate", "%R/errorlog?truncate");
   in = fossil_fopen(g.zErrlog, "rb");
   if( in==0 ){
-    @ <p class='generalError'>Unable top open that file for reading!</p>
-    style_footer();
+    @ <p class='generalError'>Unable to open that file for reading!</p>
+    style_finish_page();
     return;
   }
   if( szFile>MXSHOWLOG && P("all")==0 ){
@@ -528,5 +733,5 @@ void errorlog_page(void){
   }
   fclose(in);
   @ </pre>
-  style_footer();
+  style_finish_page();
 }
