@@ -54,6 +54,84 @@ void approxSizeName(int nOut, char *zOut, sqlite3_int64 v){
   }
 }
 
+/*
+** Generate stats for the email notification subsystem.
+*/
+void stats_for_email(void){
+  const char *zDest = db_get("email-send-method",0);
+  int nSub, nASub, nPend, nDPend;
+  const char *zDir, *zDb, *zCmd, *zRelay;
+  int iCutoff;
+  double rDigest;
+  @ <tr><th>Outgoing&nbsp;Email:</th><td>
+  if( fossil_strcmp(zDest,"pipe")==0
+   && (zCmd = db_get("email-send-command",0))!=0
+  ){
+    @ Piped to command "%h(zCmd)"
+  }else
+  if( fossil_strcmp(zDest,"db")==0
+   && (zDb = db_get("email-send-db",0))!=0
+  ){
+    sqlite3 *db;
+    sqlite3_stmt *pStmt;
+    int rc;
+    @ Queued to database "%h(zDb)"
+    rc = sqlite3_open(zDb, &db);
+    if( rc==SQLITE_OK ){
+      rc = sqlite3_prepare_v2(db, "SELECT count(*) FROM email",-1,&pStmt,0);
+      if( rc==SQLITE_OK && sqlite3_step(pStmt)==SQLITE_ROW ){
+        @ (%,d(sqlite3_column_int(pStmt,0)) messages,
+        @ %,d(file_size(zDb,ExtFILE)) bytes)
+      }
+      sqlite3_finalize(pStmt);
+    }
+    sqlite3_close(db);
+  }else
+  if( fossil_strcmp(zDest,"dir")==0
+   && (zDir = db_get("email-send-dir",0))!=0
+  ){
+    @ Written to files in "%h(zDir)"
+    @ (%,d(file_directory_size(zDir,0,1)) messages)
+  }else
+  if( fossil_strcmp(zDest,"relay")==0
+   && (zRelay = db_get("email-send-relayhost",0))!=0
+  ){
+    @ Relay to %h(zRelay) using SMTP
+  }
+  else{
+    @ Off
+  }
+  @ </td></tr>
+  nPend = db_int(0,"SELECT count(*) FROM pending_alert WHERE NOT sentSep");
+  nDPend = db_int(0,"SELECT count(*) FROM pending_alert"
+                    " WHERE NOT sentDigest");
+  @ <tr><th>Pending&nbsp;Alerts:</th><td>
+  @ %,d(nPend) normal, %,d(nDPend) digest
+  @ </td></tr>
+  if( g.perm.Admin ){
+    @ <tr><th><a href="%R/subscribers">Subscribers:</a></th><td>
+  }else{
+    @ <tr><th>Subscribers:</th><td>
+  }
+  nSub = db_int(0, "SELECT count(*) FROM subscriber");
+  iCutoff = db_get_int("email-renew-cutoff",0);
+  nASub = db_int(0, "SELECT count(*) FROM subscriber WHERE sverified"
+                   " AND NOT sdonotcall AND length(ssub)>1"
+                   " AND lastContact>=%d;", iCutoff);
+  @ %,d(nASub) active, %,d(nSub) total
+  @ </td></tr>
+  rDigest = db_double(-1.0, "SELECT (julianday('now') - value)*24.0"
+                            " FROM config WHERE name='email-last-digest'");
+  if( rDigest>0.0 ){
+    @ <tr><th>Last Digest:</th><td>Approximately \
+    if( rDigest>48.0 ){
+      @ %.1f(rDigest/24.0) days ago</td>
+    }else{
+      @ %.1f(rDigest) hours ago</td>
+    }
+  }
+}
+
 /*
 ** WEBPAGE: stat
 **
@@ -91,9 +169,9 @@ void stat_page(void){
   @ </td></tr>
   if( !brief ){
     @ <tr><th>Number&nbsp;Of&nbsp;Artifacts:</th><td>
-    n = db_int(0, "SELECT count(*) FROM blob");
+    n = db_int(0, "SELECT count(*) FROM blob WHERE content IS NOT NULL");
     m = db_int(0, "SELECT count(*) FROM delta");
-    @ %.d(n) (%,d(n-m) fulltext and %,d(m) deltas)
+    @ %,d(n) (%,d(n-m) fulltext and %,d(m) deltas)
     if( g.perm.Write ){
       @ <a href='%R/artifact_stats'>Details</a>
     }
@@ -103,7 +181,7 @@ void stat_page(void){
       Stmt q;
       @ <tr><th>Uncompressed&nbsp;Artifact&nbsp;Size:</th><td>
       db_prepare(&q, "SELECT total(size), avg(size), max(size)"
-                     " FROM blob WHERE size>0 /*scan*/");
+                     " FROM blob WHERE content IS NOT NULL /*scan*/");
       db_step(&q);
       t = db_column_int64(&q, 0);
       szAvg = db_column_int(&q, 1);
@@ -155,11 +233,31 @@ void stat_page(void){
                   " WHERE +tagname GLOB 'wiki-*'");
     @ %,d(n)
     @ </td></tr>
-    @ <tr><th>Number&nbsp;Of&nbsp;Tickets:</th><td>
+    if( db_table_exists("repository","chat") ){
+      sqlite3_int64 sz = 0;
+      char zSz[100];
+      n = db_int(0, "SELECT max(msgid) FROM chat");
+      m = db_int(0, "SELECT count(*) FROM chat WHERE mdel IS NOT TRUE");
+      sz = db_int64(0, "SELECT sum(coalesce(length(xmsg),0)+"
+                                  "coalesce(length(file),0)) FROM chat");
+      approxSizeName(sizeof(zSz), zSz, sz);
+      @ <tr><th>Number&nbsp;Of&nbsp;Chat&nbsp;Messages:</th>
+      @ <td>%,d(n) (%,d(m) still alive, %s(zSz) in size)</td></tr>
+    }
     n = db_int(0, "SELECT count(*) FROM tag  /*scan*/"
                   " WHERE +tagname GLOB 'tkt-*'");
-    @ %,d(n)
-    @ </td></tr>
+    if( n>0 ){
+      @ <tr><th>Number&nbsp;Of&nbsp;Tickets:</th><td>%,d(n)</td></tr>
+    }
+    if( db_table_exists("repository","forumpost") ){
+      n = db_int(0, "SELECT count(*) FROM forumpost/*scan*/");
+      if( n>0 ){
+        int nThread = db_int(0, "SELECT count(*) FROM forumpost"
+                                " WHERE froot=fpid");
+        @ <tr><th>Number&nbsp;Of&nbsp;Forum&nbsp;Posts:</th>
+        @ <td>%,d(n) on %d(nThread) threads</td></tr>
+      }
+    }
   }
   @ <tr><th>Duration&nbsp;Of&nbsp;Project:</th><td>
   n = db_int(0, "SELECT julianday('now') - (SELECT min(mtime) FROM event)"
@@ -200,23 +298,42 @@ void stat_page(void){
   @ %s(db_text(0, "PRAGMA repository.encoding")),
   @ %s(db_text(0, "PRAGMA repository.journal_mode")) mode
   @ </td></tr>
+  if( g.perm.Admin && g.zErrlog && g.zErrlog[0] ){
+    i64 szFile = file_size(g.zErrlog, ExtFILE);
+    if( szFile>=0 ){
+      @ <tr><th>Error Log:</th>
+      @ <td><a href='%R/errorlog'>%h(g.zErrlog)</a> (%,lld(szFile) bytes)
+    }
+    @ </td></tr>
+  }
+  if( g.perm.Admin ){
+    @ <tr><th>Backoffice:</th>
+    @ <td>Last run: %z(backoffice_last_run())</td></tr>
+  }
+  if( g.perm.Admin && alert_enabled() ){
+    stats_for_email();
+  }
 
   @ </table>
-  style_footer();
+  style_finish_page();
 }
 
 /*
-** COMMAND: dbstat*
+** COMMAND: dbstat
 **
 ** Usage: %fossil dbstat OPTIONS
 **
-** Shows statistics and global information about the repository.
+** Shows statistics and global information about the repository and/or
+** verify the integrity of a repository.
 **
 ** Options:
 **
-**   --brief|-b           Only show essential elements
-**   --db-check           Run a PRAGMA quick_check on the repository database
-**   --omit-version-info  Omit the SQLite and Fossil version information
+**   -b|--brief           Only show essential elements.
+**   --db-check           Run "PRAGMA quick_check" on the repository database.
+**   --db-verify          Run a full verification of the repository integrity.
+**                        This involves decoding and reparsing all artifacts
+**                        and can take significant time.
+**   --omit-version-info  Omit the SQLite and Fossil version information.
 */
 void dbstat_cmd(void){
   i64 t, fsize;
@@ -231,6 +348,7 @@ void dbstat_cmd(void){
   brief = find_option("brief", "b",0)!=0;
   omitVers = find_option("omit-version-info", 0, 0)!=0;
   dbCheck = find_option("db-check",0,0)!=0;
+  if( find_option("db-verify",0,0)!=0 ) dbCheck = 2;
   db_find_and_open_repository(0,0);
 
   /* We should be done with options.. */
@@ -244,7 +362,7 @@ void dbstat_cmd(void){
   fsize = file_size(g.zRepositoryName, ExtFILE);
   fossil_print( "%*s%,lld bytes\n", colWidth, "repository-size:", fsize);
   if( !brief ){
-    n = db_int(0, "SELECT count(*) FROM blob");
+    n = db_int(0, "SELECT count(*) FROM blob WHERE content IS NOT NULL");
     m = db_int(0, "SELECT count(*) FROM delta");
     fossil_print("%*s%,d (stored as %,d full text and %,d deltas)\n",
                  colWidth, "artifact-count:",
@@ -285,6 +403,15 @@ void dbstat_cmd(void){
     m = db_int(0, "SELECT COUNT(*) FROM event WHERE type='t'");
     fossil_print("%*s%,d (%,d changes)\n", colWidth, "tickets:", n, m);
     n = db_int(0, "SELECT COUNT(*) FROM event WHERE type='e'");
+    if( db_table_exists("repository","forumpost") ){
+      n = db_int(0, "SELECT count(*) FROM forumpost/*scan*/");
+      if( n>0 ){
+        int nThread = db_int(0, "SELECT count(*) FROM forumpost"
+                                " WHERE froot=fpid");
+        fossil_print("%*s%,d (on %,d threads)\n", colWidth, "forum-posts:",
+                     n, nThread);
+      }
+    }
     fossil_print("%*s%,d\n", colWidth, "events:", n);
     n = db_int(0, "SELECT COUNT(*) FROM event WHERE type='g'");
     fossil_print("%*s%,d\n", colWidth, "tag-changes:", n);
@@ -298,9 +425,11 @@ void dbstat_cmd(void){
                 " + 0.99");
   fossil_print("%*s%,d days or approximately %.2f years.\n",
                colWidth, "project-age:", n, n/365.2425);
-  p = db_get("project-code", 0);
-  if( p ){
-    fossil_print("%*s%s\n", colWidth, "project-id:", p);
+  if( !brief ){
+    p = db_get("project-code", 0);
+    if( p ){
+      fossil_print("%*s%s\n", colWidth, "project-id:", p);
+    }
   }
 #if 0
   /* Server-id is not useful information any more */
@@ -326,8 +455,19 @@ void dbstat_cmd(void){
                db_text(0, "PRAGMA repository.encoding"),
                db_text(0, "PRAGMA repository.journal_mode"));
   if( dbCheck ){
-    fossil_print("%*s%s\n", colWidth, "database-check:",
-                 db_text(0, "PRAGMA quick_check(1)"));
+    if( dbCheck<2 ){
+      char *zRes = db_text(0, "PRAGMA repository.quick_check(1)");
+      fossil_print("%*s%s\n", colWidth, "database-check:", zRes);
+    }else{
+      char *newArgv[3];
+      newArgv[0] = g.argv[0];
+      newArgv[1] = "test-integrity";
+      newArgv[2] = 0;
+      g.argv = newArgv;
+      g.argc = 2;
+      fossil_print("Full repository verification follows:\n");
+      test_integrity();
+    }
   }
 }
 
@@ -342,16 +482,17 @@ void urllist_page(void){
   int showAll = P("all")!=0;
   int nOmitted;
   sqlite3_int64 iNow;
-  char *zRemote;
+
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
+  style_set_current_feature("stat");
   style_header("URLs and Checkouts");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_element("Stat", "stat");
   style_submenu_element("Schema", "repo_schema");
   iNow = db_int64(0, "SELECT strftime('%%s','now')");
-  @ <div class="section">URLs</div>
+  @ <div class="section">URLs used to access this repository</div>
   @ <table border="0" width='100%%'>
   db_prepare(&q, "SELECT substr(name,9), datetime(mtime,'unixepoch'), mtime"
                  "  FROM config WHERE name GLOB 'baseurl:*' ORDER BY 3 DESC");
@@ -373,37 +514,56 @@ void urllist_page(void){
     @ <tr><td><a href="urllist?all"><i>Show %d(nOmitted) more...</i></a>
   }
   @ </table>
-  @ <div class="section">Checkouts</div>
-  @ <table border="0" width='100%%'>
   db_prepare(&q, "SELECT substr(name,7), datetime(mtime,'unixepoch')"
                  "  FROM config WHERE name GLOB 'ckout:*' ORDER BY 2 DESC");
   cnt = 0;
   while( db_step(&q)==SQLITE_ROW ){
     const char *zPath = db_column_text(&q,0);
     if( vfile_top_of_checkout(zPath) ){
+      if( cnt==0 ){
+        @ <div class="section">Checkouts</div>
+        @ <table border="0" width='100%%'>
+      }
       @ <tr><td width='100%%'>%h(zPath)</td>
       @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+      cnt++;
     }
-    cnt++;
   }
   db_finalize(&q);
-  if( cnt==0 ){
-    @ <tr><td>(none)</td>
+  if( cnt ){
+    @ </table>
   }
-  @ </table>
-  zRemote = db_text(0, "SELECT value FROM config WHERE name='last-sync-url'");
-  if( zRemote ){
-    @ <div class="section">Last Sync URL</div>
-    if( sqlite3_strlike("http%", zRemote, 0)==0 ){
-      UrlData x;
-      url_parse_local(zRemote, URL_OMIT_USER, &x);
-      @ <p><a href='%h(x.canonical)'>%h(zRemote)</a>
-    }else{
-      @ <p>%h(zRemote)</p>
+  cnt = 0;
+  db_prepare(&q,
+    "SELECT substr(name,10), datetime(mtime,'unixepoch')"
+    "  FROM config WHERE name GLOB 'syncwith:*'"
+    "UNION ALL "
+    "SELECT substr(name,10), datetime(mtime,'unixepoch')"
+    "  FROM config WHERE name GLOB 'syncfrom:*'"
+    "UNION ALL "
+    "SELECT substr(name,9), datetime(mtime,'unixepoch')"
+    "  FROM config WHERE name GLOB 'gitpush:*'"
+    "ORDER BY 2 DESC"
+  );
+  while( db_step(&q)==SQLITE_ROW ){
+    const char *zURL = db_column_text(&q,0);
+    UrlData x;
+    if( cnt==0 ){
+      @ <div class="section">Sync with these URLs</div>
+      @ <table border='0' width='100%%'>
     }
-    @ </div>
+    memset(&x, 0, sizeof(x));
+    url_parse_local(zURL, URL_OMIT_USER, &x);
+    @ <tr><td width='100%%'><a href='%h(x.canonical)'>%h(x.canonical)</a>
+    @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+    cnt++;
+    url_unparse(&x);
   }
-  style_footer();
+  db_finalize(&q);
+  if( cnt ){
+    @ </table>
+  }
+  style_finish_page();
 }
 
 /*
@@ -418,6 +578,7 @@ void repo_schema_page(void){
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
+  style_set_current_feature("stat");
   style_header("Repository Schema");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_element("Stat", "stat");
@@ -426,7 +587,7 @@ void repo_schema_page(void){
     style_submenu_element("Table Sizes", "repo-tabsize");
   }
   blob_init(&sql,
-    "SELECT sql FROM repository.sqlite_master WHERE sql IS NOT NULL", -1);
+    "SELECT sql FROM repository.sqlite_schema WHERE sql IS NOT NULL", -1);
   if( zArg ){
     style_submenu_element("All", "repo_schema");
     blob_appendf(&sql, " AND (tbl_name=%Q OR name=%Q)", zArg, zArg);
@@ -461,7 +622,7 @@ void repo_schema_page(void){
       style_submenu_element("Stat1","repo_stat1");
     }
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -473,6 +634,7 @@ void repo_stat1_page(void){
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
+  style_set_current_feature("stat");
   style_header("Repository STAT1 Table");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_element("Stat", "stat");
@@ -493,7 +655,7 @@ void repo_stat1_page(void){
     @ </pre>
     db_finalize(&q);
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -508,6 +670,7 @@ void repo_tabsize_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  style_set_current_feature("stat");
   style_header("Repository Table Sizes");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_element("Stat", "stat");
@@ -517,12 +680,12 @@ void repo_tabsize_page(void){
   db_multi_exec(
     "CREATE TEMP TABLE trans(name TEXT PRIMARY KEY,tabname TEXT)WITHOUT ROWID;"
     "INSERT INTO trans(name,tabname)"
-    "   SELECT name, tbl_name FROM repository.sqlite_master;"
+    "   SELECT name, tbl_name FROM repository.sqlite_schema;"
     "CREATE TEMP TABLE piechart(amt REAL, label TEXT);"
     "INSERT INTO piechart(amt,label)"
-    "  SELECT count(*), "
+    "  SELECT sum(pageno),"
     "  coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
-    "    FROM dbstat('repository')"
+    "    FROM dbstat('repository',TRUE)"
     "   GROUP BY 2 ORDER BY 2;"
   );
   nPageFree = db_int(0, "PRAGMA repository.freelist_count");
@@ -543,12 +706,12 @@ void repo_tabsize_page(void){
     db_multi_exec(
       "DELETE FROM trans;"
       "INSERT INTO trans(name,tabname)"
-      "   SELECT name, tbl_name FROM localdb.sqlite_master;"
+      "   SELECT name, tbl_name FROM localdb.sqlite_schema;"
       "DELETE FROM piechart;"
       "INSERT INTO piechart(amt,label)"
-      "  SELECT count(*), "
+      "  SELECT sum(pageno), "
       " coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
-      "    FROM dbstat('localdb')"
+      "    FROM dbstat('localdb',TRUE)"
       "   GROUP BY 2 ORDER BY 2;"
     );
     nPageFree = db_int(0, "PRAGMA localdb.freelist_count");
@@ -565,7 +728,7 @@ void repo_tabsize_page(void){
     piechart_render(800,500,PIE_OTHER|PIE_PERCENT);
     @ </svg></center>
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -573,7 +736,7 @@ void repo_tabsize_page(void){
 **
 ** Only populate the artstat.atype field if the bWithTypes parameter is true.
 */
-static void gather_artifact_stats(int bWithTypes){
+void gather_artifact_stats(int bWithTypes){
   static const char zSql[] = 
     @ CREATE TEMP TABLE artstat(
     @   id INTEGER PRIMARY KEY,   -- Corresponds to BLOB.RID
@@ -584,17 +747,18 @@ static void gather_artifact_stats(int bWithTypes){
     @ );
     @ INSERT INTO artstat(id,atype,isDelta,szExp,szCmpr)
     @    SELECT blob.rid, NULL,
-    @           EXISTS(SELECT 1 FROM delta WHERE delta.rid=blob.rid),
+    @           delta.rid IS NOT NULL,
     @           size, length(content)
-    @      FROM blob
+    @      FROM blob LEFT JOIN delta ON blob.rid=delta.rid
     @     WHERE content IS NOT NULL;
   ;
   static const char zSql2[] = 
     @ UPDATE artstat SET atype='file'
-    @  WHERE id IN (SELECT fid FROM mlink)
-    @    AND atype IS NULL;
+    @  WHERE +id IN (SELECT fid FROM mlink);
     @ UPDATE artstat SET atype='manifest'
     @  WHERE id IN (SELECT objid FROM event WHERE type='ci') AND atype IS NULL;
+    @ UPDATE artstat SET atype='forum'
+    @  WHERE id IN (SELECT objid FROM event WHERE type='f') AND atype IS NULL;
     @ UPDATE artstat SET atype='cluster'
     @  WHERE atype IS NULL 
     @    AND id IN (SELECT rid FROM tagxref
@@ -678,11 +842,13 @@ void artifact_stats_page(void){
   ** user without check-in privileges, to prevent excessive usage by
   ** robots and random passers-by on the internet
   */
-  if( !g.perm.Write ){
-    login_needed(g.anon.Admin);
+  if( !g.perm.Write && !db_get_boolean("artifact_stats_enable",0) ){
+    login_needed(g.anon.Write);
     return;
   }
+  load_control();
 
+  style_set_current_feature("stat");
   style_header("Artifact Statistics");
   style_submenu_element("Repository Stats", "stat");
   style_submenu_element("Artifact List", "bloblist");
@@ -704,7 +870,7 @@ void artifact_stats_page(void){
   db_finalize(&q);
   if( nTotal==0 ){
     @ No artifacts in this repository!
-    style_footer();
+    style_finish_page();
     return;
   }
   avgCmpr = (double)sumCmpr/nTotal;
@@ -759,7 +925,7 @@ void artifact_stats_page(void){
   @ <td>largest: %,d(mxCmpr), average: %,d((int)r), median: %,d(med)</td>
   @ </table>
 
-  @ <h1>Artifact size distribution facts:</h1>
+  @ <h1>Artifact Size Distribution Facts:</h1>
   @ <ol>
   @ <li><p>The largest %.2f(n50pct*100.0/nTotal)%% of artifacts
   largest_n_artifacts(n50pct);
@@ -808,8 +974,8 @@ void artifact_stats_page(void){
     @ <td data-sortkey='%08x(nTotal)' align='right'>%,d(nTotal)</td>
     @ <td data-sortkey='%08x(nFull)' align='right'>%,d(nFull)</td>
     @ <td data-sortkey='%08x(nDelta)' align='right'>%,d(nDelta)</td>
-    @ <td data-sortkey='%016x(szCmpr)' align='right'>%,lld(szCmpr)</td>
-    @ <td data-sortkey='%016x(szExp)' align='right'>%,lld(szExp)</td>
+    @ <td data-sortkey='%016llx(szCmpr)' align='right'>%,lld(szCmpr)</td>
+    @ <td data-sortkey='%016llx(szExp)' align='right'>%,lld(szExp)</td>
   }
   @ </tbody></table>
   db_finalize(&q);
@@ -849,5 +1015,5 @@ void artifact_stats_page(void){
     db_finalize(&q);
   }
   style_table_sorter();
-  style_footer();
+  style_finish_page();
 }
