@@ -23,10 +23,22 @@
 
 /*
 ** If the repository is configured for autosyncing, then do an
-** autosync.  This will be a pull if the argument is true or a push
-** if the argument is false.
+** autosync.  Bits of the "flags" parameter determine details of behavior:
+**
+**   SYNC_PULL           Pull content from the server to the local repo
+**   SYNC_PUSH           Push content from local up to the server
+**   SYNC_CKIN_LOCK      Take a check-in lock on the current checkout.
+**   SYNC_VERBOSE        Extra output
 **
 ** Return the number of errors.
+**
+** The autosync setting can be a boolean or "pullonly".  No autosync
+** is attempted if the autosync setting is off, and only auto-pull is
+** attempted if autosync is set to "pullonly".  The check-in lock is
+** not acquired unless autosync is set to "on".
+**
+** If dont-push setting is true, that is the same as having autosync
+** set to pullonly.
 */
 int autosync(int flags){
   const char *zAutosync;
@@ -35,19 +47,12 @@ int autosync(int flags){
   if( g.fNoSync ){
     return 0;
   }
-  if( flags==SYNC_PUSH && db_get_boolean("dont-push",0) ){
-    return 0;
-  }
   zAutosync = db_get("autosync", 0);
-  if( zAutosync ){
-    if( (flags & SYNC_PUSH)!=0 && fossil_strncmp(zAutosync,"pull",4)==0 ){
-      return 0;   /* Do not auto-push when autosync=pullonly */
-    }
-    if( is_false(zAutosync) ){
-      return 0;   /* Autosync is completely off */
-    }
-  }else{
-    /* Autosync defaults on.  To make it default off, "return" here. */
+  if( zAutosync==0 ) zAutosync = "on";  /* defend against misconfig */
+  if( is_false(zAutosync) ) return 0;
+  if( db_get_boolean("dont-push",0) || fossil_strncmp(zAutosync,"pull",4)==0 ){
+    flags &= ~SYNC_CKIN_LOCK;
+    if( flags & SYNC_PUSH ) return 0;
   }
   url_parse(0, URL_REMEMBER);
   if( g.url.protocol==0 ) return 0;
@@ -58,22 +63,10 @@ int autosync(int flags){
   }
   g.zHttpAuth = get_httpauth();
   url_remember();
-#if 0 /* Disabled for now */
-  if( (flags & AUTOSYNC_PULL)!=0 && db_get_boolean("auto-shun",1) ){
-    /* When doing an automatic pull, also automatically pull shuns from
-    ** the server if pull_shuns is enabled.
-    **
-    ** TODO:  What happens if the shun list gets really big?
-    ** Maybe the shunning list should only be pulled on every 10th
-    ** autosync, or something?
-    */
-    configSync = CONFIGSET_SHUN;
-  }
-#endif
   if( find_option("verbose","v",0)!=0 ) flags |= SYNC_VERBOSE;
   fossil_print("Autosync:  %s\n", g.url.canonical);
   url_enable_proxy("via proxy: ");
-  rc = client_sync(flags, configSync, 0);
+  rc = client_sync(flags, configSync, 0, 0);
   return rc;
 }
 
@@ -123,7 +116,8 @@ int autosync_loop(int flags, int nTries, int doPrompt){
 static void process_sync_args(
   unsigned *pConfigFlags,      /* Write configuration flags here */
   unsigned *pSyncFlags,        /* Write sync flags here */
-  int uvOnly                   /* Special handling flags for UV sync */
+  int uvOnly,                  /* Special handling flags for UV sync */
+  unsigned urlOmitFlags        /* Omit these URL flags */
 ){
   const char *zUrl = 0;
   const char *zHttpAuth = 0;
@@ -154,12 +148,15 @@ static void process_sync_args(
   if( find_option("verbose","v",0)!=0 ){
     *pSyncFlags |= SYNC_VERBOSE;
   }
+  if( find_option("no-http-compression",0,0)!=0 ){
+    *pSyncFlags |= SYNC_NOHTTPCOMPRESS;
+  }
   url_proxy_options();
   clone_ssh_find_options();
   if( !uvOnly ) db_find_and_open_repository(0, 0);
-  db_open_config(0, 0);
+  db_open_config(0, 1);
   if( g.argc==2 ){
-    if( db_get_boolean("auto-shun",1) ) configSync = CONFIGSET_SHUN;
+    if( db_get_boolean("auto-shun",0) ) configSync = CONFIGSET_SHUN;
   }else if( g.argc==3 ){
     zUrl = g.argv[2];
   }
@@ -168,6 +165,7 @@ static void process_sync_args(
   ){
     *pSyncFlags |= SYNC_UNVERSIONED;
   }
+  urlFlags &= ~urlOmitFlags;
   if( urlFlags & URL_REMEMBER ){
     clone_ssh_db_set_options();
   }
@@ -179,7 +177,7 @@ static void process_sync_args(
     usage("URL");
   }
   user_select();
-  if( g.argc==2 ){
+  if( g.url.isAlias ){
     if( ((*pSyncFlags) & (SYNC_PUSH|SYNC_PULL))==(SYNC_PUSH|SYNC_PULL) ){
       fossil_print("Sync with %s\n", g.url.canonical);
     }else if( (*pSyncFlags) & SYNC_PUSH ){
@@ -197,13 +195,14 @@ static void process_sync_args(
 **
 ** Usage: %fossil pull ?URL? ?options?
 **
-** Pull all sharable changes from a remote repository into the local repository.
-** Sharable changes include public check-ins, and wiki, ticket, and tech-note
-** edits.  Add the --private option to pull private branches.  Use the
+** Pull all sharable changes from a remote repository into the local
+** repository.  Sharable changes include public check-ins, edits to
+** wiki pages, tickets, tech-notes, and forum posts.  Add
+** the --private option to pull private branches.  Use the
 ** "configuration pull" command to pull website configuration details.
 **
 ** If URL is not specified, then the URL from the most recent clone, push,
-** pull, remote-url, or sync command is used.  See "fossil help clone" for
+** pull, remote, or sync command is used.  See "fossil help clone" for
 ** details on the URL formats.
 **
 ** Options:
@@ -212,9 +211,11 @@ static void process_sync_args(
 **                              if required by the remote website
 **   --from-parent-project      Pull content from the parent project
 **   --ipv4                     Use only IPv4, not IPv6
+**   --no-http-compression      Do not compress HTTP traffic
 **   --once                     Do not remember URL for subsequent syncs
-**   --proxy PROXY              Use the specified HTTP proxy
 **   --private                  Pull private branches too
+**   --project-code CODE        Use CODE as the project code
+**   --proxy PROXY              Use the specified HTTP proxy
 **   -R|--repository REPO       Local repository to pull into
 **   --ssl-identity FILE        Local SSL credentials, if requested by remote
 **   --ssh-command SSH          Use SSH as the "ssh" command
@@ -222,20 +223,23 @@ static void process_sync_args(
 **   --verily                   Exchange extra information with the remote
 **                              to ensure no content is overlooked
 **
-** See also: clone, config pull, push, remote-url, sync
+** See also: [[clone]], [[config]], [[push]], [[remote]], [[sync]]
 */
 void pull_cmd(void){
   unsigned configFlags = 0;
   unsigned syncFlags = SYNC_PULL;
+  unsigned urlOmitFlags = 0;
+  const char *zAltPCode = find_option("project-code",0,1);
   if( find_option("from-parent-project",0,0)!=0 ){
     syncFlags |= SYNC_FROMPARENT;
   }
-  process_sync_args(&configFlags, &syncFlags, 0);
+  if( zAltPCode ) urlOmitFlags = URL_REMEMBER;
+  process_sync_args(&configFlags, &syncFlags, 0, urlOmitFlags);
 
   /* We should be done with options.. */
   verify_all_options();
 
-  client_sync(syncFlags, configFlags, 0);
+  client_sync(syncFlags, configFlags, 0, zAltPCode);
 }
 
 /*
@@ -243,13 +247,14 @@ void pull_cmd(void){
 **
 ** Usage: %fossil push ?URL? ?options?
 **
-** Push all sharable changes from the local repository to a remote repository.
-** Sharable changes include public check-ins, and wiki, ticket, and tech-note
-** edits.  Use --private to also push private branches.  Use the
-** "configuration push" command to push website configuration details.
+** Push all sharable changes from the local repository to a remote
+** repository.  Sharable changes include public check-ins, edits to
+** wiki pages, tickets, tech-notes, and forum posts.  Use
+** --private to also push private branches.  Use the "configuration
+** push" command to push website configuration details.
 **
 ** If URL is not specified, then the URL from the most recent clone, push,
-** pull, remote-url, or sync command is used.  See "fossil help clone" for
+** pull, remote, or sync command is used.  See "fossil help clone" for
 ** details on the URL formats.
 **
 ** Options:
@@ -257,6 +262,7 @@ void pull_cmd(void){
 **   -B|--httpauth USER:PASS    Credentials for the simple HTTP auth protocol,
 **                              if required by the remote website
 **   --ipv4                     Use only IPv4, not IPv6
+**   --no-http-compression      Do not compress HTTP traffic
 **   --once                     Do not remember URL for subsequent syncs
 **   --proxy PROXY              Use the specified HTTP proxy
 **   --private                  Push private branches too
@@ -267,12 +273,12 @@ void pull_cmd(void){
 **   --verily                   Exchange extra information with the remote
 **                              to ensure no content is overlooked
 **
-** See also: clone, config push, pull, remote-url, sync
+** See also: [[clone]], [[config]], [[pull]], [[remote]], [[sync]]
 */
 void push_cmd(void){
   unsigned configFlags = 0;
   unsigned syncFlags = SYNC_PUSH;
-  process_sync_args(&configFlags, &syncFlags, 0);
+  process_sync_args(&configFlags, &syncFlags, 0, 0);
 
   /* We should be done with options.. */
   verify_all_options();
@@ -280,7 +286,7 @@ void push_cmd(void){
   if( db_get_boolean("dont-push",0) ){
     fossil_fatal("pushing is prohibited: the 'dont-push' option is set");
   }
-  client_sync(syncFlags, 0, 0);
+  client_sync(syncFlags, 0, 0, 0);
 }
 
 
@@ -291,10 +297,10 @@ void push_cmd(void){
 **
 ** Synchronize all sharable changes between the local repository and a
 ** remote repository.  Sharable changes include public check-ins and
-** edits to wiki pages, tickets, and technical notes.
+** edits to wiki pages, tickets, forum posts, and technical notes.
 **
 ** If URL is not specified, then the URL from the most recent clone, push,
-** pull, remote-url, or sync command is used.  See "fossil help clone" for
+** pull, remote, or sync command is used.  See "fossil help clone" for
 ** details on the URL formats.
 **
 ** Options:
@@ -302,6 +308,7 @@ void push_cmd(void){
 **   -B|--httpauth USER:PASS    Credentials for the simple HTTP auth protocol,
 **                              if required by the remote website
 **   --ipv4                     Use only IPv4, not IPv6
+**   --no-http-compression      Do not compress HTTP traffic
 **   --once                     Do not remember URL for subsequent syncs
 **   --proxy PROXY              Use the specified HTTP proxy
 **   --private                  Sync private branches too
@@ -313,7 +320,7 @@ void push_cmd(void){
 **   --verily                   Exchange extra information with the remote
 **                              to ensure no content is overlooked
 **
-** See also: clone, pull, push, remote-url
+** See also: [[clone]], [[pull]], [[push]], [[remote]]
 */
 void sync_cmd(void){
   unsigned configFlags = 0;
@@ -321,13 +328,13 @@ void sync_cmd(void){
   if( find_option("unversioned","u",0)!=0 ){
     syncFlags |= SYNC_UNVERSIONED;
   }
-  process_sync_args(&configFlags, &syncFlags, 0);
+  process_sync_args(&configFlags, &syncFlags, 0, 0);
 
   /* We should be done with options.. */
   verify_all_options();
 
   if( db_get_boolean("dont-push",0) ) syncFlags &= ~SYNC_PUSH;
-  client_sync(syncFlags, configFlags, 0);
+  client_sync(syncFlags, configFlags, 0, 0);
   if( (syncFlags & SYNC_PUSH)==0 ){
     fossil_warning("pull only: the 'dont-push' option is set");
   }
@@ -340,52 +347,215 @@ void sync_cmd(void){
 void sync_unversioned(unsigned syncFlags){
   unsigned configFlags = 0;
   (void)find_option("uv-noop",0,0);
-  process_sync_args(&configFlags, &syncFlags, 1);
+  process_sync_args(&configFlags, &syncFlags, 1, 0);
   verify_all_options();
-  client_sync(syncFlags, 0, 0);
+  client_sync(syncFlags, 0, 0, 0);
 }
 
 /*
-** COMMAND: remote-url
+** COMMAND: remote
+** COMMAND: remote-url*
 **
-** Usage: %fossil remote-url ?URL|off?
+** Usage: %fossil remote ?SUBCOMMAND ...?
 **
-** Query and/or change the default server URL used by the "pull", "push",
-** and "sync" commands.
+** View or modify the set of remote repository sync URLs used as the
+** target in any command that uses the sync protocol: "sync", "push",
+** and "pull", plus all other commands that trigger Fossil's autosync
+** feature.  (Collectively, "sync operations".)
 **
-** The remote-url is set automatically by a "clone" command or by any
-** "sync", "push", or "pull" command that specifies an explicit URL.
-** The default remote-url is used by auto-syncing and by "sync", "push",
-** "pull" that omit the server URL.
+** See "fossil help clone" for the format of these sync URLs.
 **
-** See "fossil help clone" for further information about URL formats
+** Fossil implicitly sets the default remote sync URL from the initial
+** "clone" or "open URL" command for a repository, then may subsequently
+** change it when given a URL in commands that take a sync URL, except
+** when given the --once flag.  Fossil uses this new sync URL as its
+** default when not explicitly given one in subsequent sync operations.
 **
-** See also: clone, push, pull, sync
+** Named remotes added by "remote add" allow use of those names in place
+** of a sync URL in any command that takes one.
+**
+** The full name of this command is "remote-url", but we anticipate no
+** future collision from use of its shortened form "remote".
+**
+** > fossil remote
+**
+**     With no arguments, this command shows the current default remote
+**     URL.  If there is no default, it shows "off".
+**
+** > fossil remote add NAME URL
+**
+**     Add a new named URL to the set of remote sync URLs for use in
+**     place of a sync URL in commands that take one.
+**
+** > fossil remote delete NAME
+**
+**     Delete a sync URL previously added by the "add" subcommand.
+**
+** > fossil remote list|ls
+**
+**     Show all remote repository sync URLs.
+**
+** > fossil remote off
+**
+**     Forget the default sync URL, disabling autosync.  Combined with
+**     named sync URLs, it allows canceling this "airplane mode" with
+**     "fossil remote NAME" to select a previously-set named URL.
+**
+**     To disable use of the default remote without forgetting its URL,
+**     say "fossil set autosync 0" instead.
+**
+** > fossil remote REF
+**
+**     Make REF the new default URL, replacing the prior default.
+**     REF may be a URL or a NAME from a prior "add".
 */
 void remote_url_cmd(void){
-  char *zUrl;
+  char *zUrl, *zArg;
+  int nArg;
   db_find_and_open_repository(0, 0);
 
   /* We should be done with options.. */
   verify_all_options();
 
-  if( g.argc!=2 && g.argc!=3 ){
-    usage("?URL|off?");
+  if( g.argc==2 ){
+    /* "fossil remote" with no arguments:  Show the last sync URL. */
+    zUrl = db_get("last-sync-url", 0);
+    if( zUrl==0 ){
+      fossil_print("off\n");
+    }else{
+      url_parse(zUrl, 0);
+      fossil_print("%s\n", g.url.canonical);
+    }
+    return;
   }
-  if( g.argc==3 ){
+  zArg = g.argv[2];
+  nArg = (int)strlen(zArg);
+  if( strcmp(zArg,"off")==0 ){
+    /* fossil remote off
+    ** Forget the last-sync-URL and its password
+    */
+    if( g.argc!=3 ) usage("off");
+remote_delete_default:
+    db_unprotect(PROTECT_CONFIG);
+    db_multi_exec(
+      "DELETE FROM config WHERE name GLOB 'last-sync-*';"
+    );
+    db_protect_pop();
+    return;
+  }
+  if( strncmp(zArg, "list", nArg)==0 || strcmp(zArg,"ls")==0 ){
+    Stmt q;
+    if( g.argc!=3 ) usage("list");
+    db_prepare(&q,
+      "SELECT 'default', value FROM config WHERE name='last-sync-url'"
+      " UNION ALL "
+      "SELECT substr(name,10), value FROM config"
+      " WHERE name GLOB 'sync-url:*'"
+      " ORDER BY 1"
+    );
+    while( db_step(&q)==SQLITE_ROW ){
+      fossil_print("%-18s %s\n", db_column_text(&q,0), db_column_text(&q,1));
+    }
+    db_finalize(&q);
+    return;
+  }
+  if( strcmp(zArg, "add")==0 ){
+    char *zName;
+    char *zUrl;
+    UrlData x;
+    if( g.argc!=5 ) usage("add NAME URL");
+    memset(&x, 0, sizeof(x));
+    zName = g.argv[3];
+    zUrl = g.argv[4];
+    if( strcmp(zName,"default")==0 ) goto remote_add_default;
+    url_parse_local(zUrl, URL_PROMPT_PW, &x);
+    db_begin_write();
+    db_unprotect(PROTECT_CONFIG);
+    db_multi_exec(
+       "REPLACE INTO config(name, value, mtime)"
+       " VALUES('sync-url:%q',%Q,now())",
+       zName, x.canonical
+    );
+    db_multi_exec(
+       "REPLACE INTO config(name, value, mtime)"
+       " VALUES('sync-pw:%q',obscure(%Q),now())",
+       zName, x.passwd
+    );
+    db_protect_pop();
+    db_commit_transaction();
+    return;
+  }
+  if( strncmp(zArg, "delete", nArg)==0 ){
+    char *zName;
+    if( g.argc!=4 ) usage("delete NAME");
+    zName = g.argv[3];
+    if( strcmp(zName,"default")==0 ) goto remote_delete_default;
+    db_begin_write();
+    db_unprotect(PROTECT_CONFIG);
+    db_multi_exec("DELETE FROM config WHERE name glob 'sync-url:%q'", zName);
+    db_multi_exec("DELETE FROM config WHERE name glob 'sync-pw:%q'", zName);
+    db_protect_pop();
+    db_commit_transaction();
+    return;
+  }
+  if( sqlite3_strlike("http://%",zArg,0)==0
+   || sqlite3_strlike("https://%",zArg,0)==0
+   || sqlite3_strlike("ssh:%",zArg,0)==0
+   || sqlite3_strlike("file:%",zArg,0)==0
+   || db_exists("SELECT 1 FROM config WHERE name='sync-url:%q'",zArg)
+  ){
+remote_add_default:
     db_unset("last-sync-url", 0);
     db_unset("last-sync-pw", 0);
-    db_unset("http-auth", 0);
-    if( is_false(g.argv[2]) ) return;
     url_parse(g.argv[2], URL_REMEMBER|URL_PROMPT_PW|URL_ASK_REMEMBER_PW);
-  }
-  url_remember();
-  zUrl = db_get("last-sync-url", 0);
-  if( zUrl==0 ){
-    fossil_print("off\n");
+    url_remember();
     return;
-  }else{
-    url_parse(zUrl, 0);
-    fossil_print("%s\n", g.url.canonical);
   }
+  fossil_fatal("unknown command \"%s\" - should be a URL or one of: "
+               "add delete list off", zArg);
+}
+
+/*
+** COMMAND: backup*
+**
+** Usage: %fossil backup ?OPTIONS? FILE|DIRECTORY
+**
+** Make a backup of the repository into the named file or into the named
+** directory.  This backup is guaranteed to be consistent even if there are
+** concurrent changes taking place on the repository.  In other words, it
+** is safe to run "fossil backup" on a repository that is in active use.
+**
+** Only the main repository database is backed up by this command.  The
+** open checkout file (if any) is not saved.  Nor is the global configuration
+** database.
+**
+** Options:
+**
+**    --overwrite              OK to overwrite an existing file
+**    -R NAME                  Filename of the repository to backup
+*/
+void backup_cmd(void){
+  char *zDest;
+  int bOverwrite = 0;
+  db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
+  bOverwrite = find_option("overwrite",0,0)!=0;
+  verify_all_options();
+  if( g.argc!=3 ){
+    usage("FILE|DIRECTORY");
+  }
+  zDest = g.argv[2];
+  if( file_isdir(zDest, ExtFILE)==1 ){
+    zDest = mprintf("%s/%s", zDest, file_tail(g.zRepositoryName));
+  }
+  if( file_isfile(zDest, ExtFILE) ){
+    if( bOverwrite ){
+      if( file_delete(zDest) ){
+        fossil_fatal("unable to delete old copy of \"%s\"", zDest);
+      }
+    }else{
+      fossil_fatal("backup \"%s\" already exists", zDest);
+    }
+  }
+  db_unprotect(PROTECT_ALL);
+  db_multi_exec("VACUUM repository INTO %Q", zDest);
 }
