@@ -254,7 +254,7 @@ static void add_pax_header(
   blob_appendf(&tball.pax, "%d %s=%*.*s\n", blen, zField, nValue, nValue, zValue);
   /* this _must_ be right */
   if(blob_size(&tball.pax) != blen){
-    fossil_fatal("internal error: PAX tar header has bad length");
+    fossil_panic("internal error: PAX tar header has bad length");
   }
 }
 
@@ -427,7 +427,7 @@ static void tar_finish(Blob *pOut){
 ** Generate a GZIP-compressed tarball in the file given by the first argument
 ** that contains files given in the second and subsequent arguments.
 **
-**   -h, --dereference   Follow symlinks; archive the files they point to.
+**   -h|--dereference   Follow symlinks and archive the files they point to
 */
 void test_tarball_cmd(void){
   int i;
@@ -445,7 +445,7 @@ void test_tarball_cmd(void){
     Blob file;
     blob_zero(&file);
     blob_read_from_file(&file, g.argv[i], eFType);
-    tar_add_file(g.argv[i], &file, file_perm(0,0), file_mtime(0,0));
+    tar_add_file(g.argv[i], &file, file_perm(0,eFType), file_mtime(0,eFType));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -468,7 +468,7 @@ void test_tarball_cmd(void){
 ** added to as part of the tarball. It may be 0 or an empty string, in
 ** which case it is ignored. The intention is to create a tarball which
 ** politely expands into a subdir instead of filling your current dir
-** with source files. For example, pass a UUID or "ProjectName".
+** with source files. For example, pass an artifact hash or "ProjectName".
 **
 */
 void tarball_of_checkin(
@@ -476,7 +476,8 @@ void tarball_of_checkin(
   Blob *pTar,          /* Write the tarball into this blob */
   const char *zDir,    /* Directory prefix for all file added to tarball */
   Glob *pInclude,      /* Only add files matching this pattern */
-  Glob *pExclude       /* Exclude files matching this pattern */
+  Glob *pExclude,      /* Exclude files matching this pattern */
+  int listFlag         /* Show filenames on stdout */
 ){
   Blob mfile, hash, file;
   Manifest *pManifest;
@@ -503,7 +504,7 @@ void tarball_of_checkin(
   if( pManifest ){
     int flg, eflg = 0;
     mTime = (pManifest->rDate - 2440587.5)*86400.0;
-    tar_begin(mTime);
+    if( pTar ) tar_begin(mTime);
     flg = db_get_manifest_setting();
     if( flg ){
       /* eflg is the effective flags, taking include/exclude into account */
@@ -527,29 +528,36 @@ void tarball_of_checkin(
         if( eflg & MFESTFLG_RAW ){
           blob_append(&filename, "manifest", -1);
           zName = blob_str(&filename);
-        }
-        if( eflg & MFESTFLG_RAW ) {
-          sterilize_manifest(&mfile);
-          tar_add_file(zName, &mfile, 0, mTime);
+          if( listFlag ) fossil_print("%s\n", zName);
+          if( pTar ){
+            sterilize_manifest(&mfile, CFTYPE_MANIFEST);
+            tar_add_file(zName, &mfile, 0, mTime);
+          }
         }
       }
       blob_reset(&mfile);
       if( eflg & MFESTFLG_UUID ){
-        blob_append(&hash, "\n", 1);
         blob_resize(&filename, nPrefix);
         blob_append(&filename, "manifest.uuid", -1);
         zName = blob_str(&filename);
-        tar_add_file(zName, &hash, 0, mTime);
+        if( listFlag ) fossil_print("%s\n", zName);
+        if( pTar ){
+          blob_append(&hash, "\n", 1);
+          tar_add_file(zName, &hash, 0, mTime);
+        }
       }
       if( eflg & MFESTFLG_TAGS ){
-        Blob tagslist;
-        blob_zero(&tagslist);
-        get_checkin_taglist(rid, &tagslist);
         blob_resize(&filename, nPrefix);
         blob_append(&filename, "manifest.tags", -1);
         zName = blob_str(&filename);
-        tar_add_file(zName, &tagslist, 0, mTime);
-        blob_reset(&tagslist);
+        if( listFlag ) fossil_print("%s\n", zName);
+        if( pTar ){
+          Blob tagslist;
+          blob_zero(&tagslist);
+          get_checkin_taglist(rid, &tagslist);
+          tar_add_file(zName, &tagslist, 0, mTime);
+          blob_reset(&tagslist);
+        }
       }
     }
     manifest_file_rewind(pManifest);
@@ -559,26 +567,32 @@ void tarball_of_checkin(
       if( glob_match(pExclude, pFile->zName) ) continue;
       fid = uuid_to_rid(pFile->zUuid, 0);
       if( fid ){
-        content_get(fid, &file);
         blob_resize(&filename, nPrefix);
         blob_append(&filename, pFile->zName, -1);
         zName = blob_str(&filename);
-        tar_add_file(zName, &file, manifest_file_mperm(pFile), mTime);
-        blob_reset(&file);
+        if( listFlag ) fossil_print("%s\n", zName);
+        if( pTar ){
+          content_get(fid, &file);
+          tar_add_file(zName, &file, manifest_file_mperm(pFile), mTime);
+          blob_reset(&file);
+        }
       }
     }
   }else{
     blob_append(&filename, blob_str(&hash), 16);
     zName = blob_str(&filename);
-    mTime = db_int64(0, "SELECT (julianday('now') -  2440587.5)*86400.0;");
-    tar_begin(mTime);
-    tar_add_file(zName, &mfile, 0, mTime);
+    if( listFlag ) fossil_print("%s\n", zName);
+    if( pTar ){
+      mTime = db_int64(0, "SELECT (julianday('now') -  2440587.5)*86400.0;");
+      tar_begin(mTime);
+      tar_add_file(zName, &mfile, 0, mTime);
+    }
   }
   manifest_destroy(pManifest);
   blob_reset(&mfile);
   blob_reset(&hash);
   blob_reset(&filename);
-  tar_finish(pTar);
+  if( pTar ) tar_finish(pTar);
 }
 
 /*
@@ -597,9 +611,16 @@ void tarball_of_checkin(
 ** in "..." or '...' so that it may contain commas.  If a file matches both
 ** --include and --exclude then it is excluded.
 **
+** If OUTPUTFILE is an empty string or "/dev/null" then no tarball is
+** actually generated.  This feature can be used in combination with
+** the --list option to get a list of the filename that would be in the
+** tarball had it actually been generated.  Note that --list shows only
+** filenames.  "tar tzf" shows both filesnames and subdirectory names.
+**
 ** Options:
 **   -X|--exclude GLOBLIST   Comma-separated list of GLOBs of files to exclude
 **   --include GLOBLIST      Comma-separated list of GLOBs of files to include
+**   -l|--list               Show archive content on stdout
 **   --name DIRECTORYNAME    The name of the top-level directory in the archive
 **   -R REPOSITORY           Specify a Fossil repository
 */
@@ -611,12 +632,15 @@ void tarball_cmd(void){
   Glob *pExclude = 0;
   const char *zInclude;
   const char *zExclude;
+  int listFlag = 0;
+  const char *zOut;
   zName = find_option("name", 0, 1);
   zExclude = find_option("exclude", "X", 1);
   if( zExclude ) pExclude = glob_create(zExclude);
   zInclude = find_option("include", 0, 1);
   if( zInclude ) pInclude = glob_create(zInclude);
   db_find_and_open_repository(0, 0);
+  listFlag = find_option("list","l",0)!=0;
 
   /* We should be done with options.. */
   verify_all_options();
@@ -630,6 +654,10 @@ void tarball_cmd(void){
     fossil_fatal("Check-in not found: %s", g.argv[2]);
     return;
   }
+  zOut = g.argv[3];
+  if( fossil_strcmp("/dev/null",zOut)==0 || fossil_strcmp("",zOut)==0 ){
+    zOut = 0;
+  }
 
   if( zName==0 ){
     zName = db_text("default-name",
@@ -642,11 +670,44 @@ void tarball_cmd(void){
        db_get("project-name", "unnamed"), rid, rid
     );
   }
-  tarball_of_checkin(rid, &tarball, zName, pInclude, pExclude);
+  tarball_of_checkin(rid, zOut ? &tarball : 0,
+                     zName, pInclude, pExclude, listFlag);
   glob_free(pInclude);
   glob_free(pExclude);
-  blob_write_to_file(&tarball, g.argv[3]);
-  blob_reset(&tarball);
+  if( listFlag ) fflush(stdout);
+  if( zOut ){
+    blob_write_to_file(&tarball, zOut);
+    blob_reset(&tarball);
+  }
+}
+
+/*
+** Check to see if the input string is of the form:
+**
+**        checkin-name/filename.ext
+**
+** In other words, check to see if the input contains a single '/'
+** character that separates a valid check-in name from a filename.
+**
+** If the condition is true, return the check-in name and set the
+** input string to be the filename.
+**
+** If the condition is false, return NULL
+*/
+char *tar_uuid_from_name(char **pzName){
+  char *zName = *pzName;
+  int i, n;
+  for(i=n=0; zName[i]; i++){
+    if( zName[i]=='/' ){
+      if( n==0 ) n = i;
+      else return 0;
+    }
+  }
+  if( n==0 ) return 0;
+  if( zName[n+1]==0 ) return 0;
+  zName[n] = 0;
+  *pzName = fossil_strdup(&zName[n+1]);
+  return zName;
 }
 
 /*
@@ -657,6 +718,14 @@ void tarball_cmd(void){
 ** query parameter.  Return that compressed tarball as the HTTP reply
 ** content.
 **
+** The r= and name= query parameters can be specified as extensions to the
+** URI.  Example, the following URIs are all equivalent:
+**
+**      /tarball/release/xyz.tar.gz
+**      /tarball?r=release&name=xyz.tar.gz
+**      /tarball/xyz.tar.gz?r=release
+**      /tarball?name=release/xyz.tar.gz
+**
 ** Query parameters:
 **
 **   name=NAME[.tar.gz]  The base name of the output file.  The default
@@ -667,8 +736,11 @@ void tarball_cmd(void){
 **   r=TAG               The check-in that is turned into a compressed tarball.
 **                       Defaults to "trunk".  This query parameter used to
 **                       be called "uuid" and "uuid" is still accepted for
-**                       backwards compatibility.  If omitted, the default
-**                       check-in name is "trunk".
+**                       backwards compatibility.  If the name= query parameter
+**                       contains one "/" character then the part before the /
+**                       is the TAG and the part after the / is the true name.
+**                       If no TAG is specified by any of the above means, then
+**                       "trunk" is used as the default.
 **
 **   in=PATTERN          Only include files that match the comma-separate
 **                       list of GLOB patterns in PATTERN, as with ex=
@@ -693,10 +765,10 @@ void tarball_page(void){
   login_check_credentials();
   if( !g.perm.Zip ){ login_needed(g.anon.Zip); return; }
   load_control();
-  zName = mprintf("%s", PD("name",""));
-  nName = strlen(zName);
+  zName = fossil_strdup(PD("name",""));
   z = P("r");
   if( z==0 ) z = P("uuid");
+  if( z==0 ) z = tar_uuid_from_name(&zName);
   if( z==0 ) z = "trunk";
   g.zOpenRevision = zRid = fossil_strdup(z);
   nRid = strlen(zRid);
@@ -704,6 +776,10 @@ void tarball_page(void){
   if( zInclude ) pInclude = glob_create(zInclude);
   zExclude = P("ex");
   if( zExclude ) pExclude = glob_create(zExclude);
+  if( zInclude==0 && zExclude==0 ){
+    etag_check_for_invariant_name(z);
+  }
+  nName = strlen(zName);
   if( nName>7 && fossil_strcmp(&zName[nName-7], ".tar.gz")==0 ){
     /* Special case:  Remove the ".tar.gz" suffix.  */
     nName -= 7;
@@ -733,6 +809,7 @@ void tarball_page(void){
   if( zInclude ) blob_appendf(&cacheKey, ",in=%Q", zInclude);
   if( zExclude ) blob_appendf(&cacheKey, ",ex=%Q", zExclude);
   zKey = blob_str(&cacheKey);
+  etag_check(ETAG_HASH, zKey);
 
   if( P("debug")!=0 ){
     style_header("Tarball Generator Debug Screen");
@@ -745,7 +822,7 @@ void tarball_page(void){
       @ zExclude = "%h(zExclude)"<br />
     }
     @ zKey = "%h(zKey)"
-    style_footer();
+    style_finish_page();
     return;
   }
   if( referred_from_login() ){
@@ -756,18 +833,19 @@ void tarball_page(void){
     @ of check-in <b>%h(zRid)</b>:
     @ <input type="submit" value="Download" />
     @ </form>
-    style_footer();
+    style_finish_page();
     return;
   }
   blob_zero(&tarball);
   if( cache_read(&tarball, zKey)==0 ){
-    tarball_of_checkin(rid, &tarball, zName, pInclude, pExclude);
+    tarball_of_checkin(rid, &tarball, zName, pInclude, pExclude, 0);
     cache_write(&tarball, zKey);
   }
   glob_free(pInclude);
   glob_free(pExclude);
   fossil_free(zName);
   fossil_free(zRid);
+  g.zOpenRevision = 0;
   blob_reset(&cacheKey);
   cgi_set_content(&tarball);
   cgi_set_content_type("application/x-compressed");
