@@ -23,13 +23,14 @@
 #include <assert.h>
 
 /*
-** Main sub-menu for configuring the ticketing system.
 ** WEBPAGE: tktsetup
+** Main sub-menu for configuring the ticketing system.
 */
 void tktsetup_page(void){
   login_check_credentials();
   if( !g.perm.Setup ){
-    login_needed();
+    login_needed(0);
+    return;
   }
 
   style_header("Ticket Setup");
@@ -55,7 +56,7 @@ void tktsetup_page(void){
   setup_menu_entry("Key Template", "tktsetup_keytplt",
     "The default color key for reports.");
   @ </table>
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -69,7 +70,8 @@ static const char zDefaultTicketTable[] =
 @   tkt_id INTEGER PRIMARY KEY,
 @   tkt_uuid TEXT UNIQUE,
 @   tkt_mtime DATE,
-@   -- Add as many field as required below this line
+@   tkt_ctime DATE,
+@   -- Add as many fields as required below this line
 @   type TEXT,
 @   status TEXT,
 @   subsystem TEXT,
@@ -81,13 +83,26 @@ static const char zDefaultTicketTable[] =
 @   title TEXT,
 @   comment TEXT
 @ );
+@ CREATE TABLE ticketchng(
+@   -- Do not change any column that begins with tkt_
+@   tkt_id INTEGER REFERENCES ticket,
+@   tkt_rid INTEGER REFERENCES blob,
+@   tkt_mtime DATE,
+@   -- Add as many fields as required below this line
+@   login TEXT,
+@   username TEXT,
+@   mimetype TEXT,
+@   icomment TEXT
+@ );
+@ CREATE INDEX ticketchng_idx1 ON ticketchng(tkt_id, tkt_mtime);
 ;
 
 /*
-** Return the ticket table definition
+** Return the ticket table definition in heap-allocated
+** memory owned by the caller.
 */
-const char *ticket_table_schema(void){
-  return db_get("ticket-table", (char*)zDefaultTicketTable);
+char *ticket_table_schema(void){
+  return db_get("ticket-table", zDefaultTicketTable);
 }
 
 /*
@@ -98,43 +113,46 @@ static void tktsetup_generic(
   const char *zDbField,         /* Configuration field being edited */
   const char *zDfltValue,       /* Default text value */
   const char *zDesc,            /* Description of this field */
-  char *(*xText)(const char*),  /* Validitity test or NULL */
-  void (*xRebuild)(void),       /* Run after successulf update */
+  char *(*xText)(const char*),  /* Validity test or NULL */
+  void (*xRebuild)(void),       /* Run after successful update */
   int height                    /* Height of the edit box */
 ){
   const char *z;
   int isSubmit;
-  
+
   login_check_credentials();
   if( !g.perm.Setup ){
-    login_needed();
+    login_needed(0);
+    return;
   }
-  if( P("setup") ){
+  style_set_current_feature("tktsetup");
+  if( PB("setup") ){
     cgi_redirect("tktsetup");
   }
   isSubmit = P("submit")!=0;
   z = P("x");
   if( z==0 ){
-    z = db_get(zDbField, (char*)zDfltValue);
+    z = db_get(zDbField, zDfltValue);
   }
+  style_set_current_feature("tktsetup");
   style_header("Edit %s", zTitle);
   if( P("clear")!=0 ){
     login_verify_csrf_secret();
-    db_unset(zDbField, 0);
+    db_unset(zDbField/*works-like:"x"*/, 0);
     if( xRebuild ) xRebuild();
-    z = zDfltValue;
+    cgi_redirect("tktsetup");
   }else if( isSubmit ){
     char *zErr = 0;
     login_verify_csrf_secret();
     if( xText && (zErr = xText(z))!=0 ){
       @ <p class="tktsetupError">ERROR: %h(zErr)</p>
     }else{
-      db_set(zDbField, z, 0);
+      db_set(zDbField/*works-like:"x"*/, z, 0);
       if( xRebuild ) xRebuild();
       cgi_redirect("tktsetup");
     }
   }
-  @ <form action="%s(g.zTop)/%s(g.zPath)" method="post"><div>
+  @ <form action="%R/%s(g.zPath)" method="post"><div>
   login_insert_csrf_secret();
   @ <p>%s(zDesc)</p>
   @ <textarea name="x" rows="%d(height)" cols="80">%h(z)</textarea>
@@ -149,11 +167,13 @@ static void tktsetup_generic(
   @ <blockquote><pre>
   @ %h(zDfltValue)
   @ </pre></blockquote>
-  style_footer();
+  style_finish_page();
 }
 
 /*
 ** WEBPAGE: tktsetup_tab
+** Administrative page for defining the "ticket" table used
+** to hold ticket information.
 */
 void tktsetup_tab_page(void){
   static const char zDesc[] =
@@ -225,11 +245,13 @@ static const char zDefaultTicketCommon[] =
 ** Return the ticket common code.
 */
 const char *ticket_common_code(void){
-  return db_get("ticket-common", (char*)zDefaultTicketCommon);
+  return db_get("ticket-common", zDefaultTicketCommon);
 }
 
 /*
 ** WEBPAGE: tktsetup_com
+** Administrative page used to define TH1 script that is
+** common to all ticket screens.
 */
 void tktsetup_com_page(void){
   static const char zDesc[] =
@@ -255,11 +277,13 @@ static const char zDefaultTicketChange[] =
 ** Return the ticket change code.
 */
 const char *ticket_change_code(void){
-  return db_get("ticket-change", (char*)zDefaultTicketChange);
+  return db_get("ticket-change", zDefaultTicketChange);
 }
 
 /*
 ** WEBPAGE: tktsetup_change
+** Administrative screen used to view or edit the TH1 script
+** that shows ticket changes.
 */
 void tktsetup_change_page(void){
   static const char zDesc[] =
@@ -279,80 +303,118 @@ void tktsetup_change_page(void){
 
 static const char zDefaultNew[] =
 @ <th1>
+@   if {![info exists mutype]} {set mutype Markdown}
 @   if {[info exists submit]} {
 @      set status Open
+@      if {$mutype eq "HTML"} {
+@        set mimetype "text/html"
+@      } elseif {$mutype eq "Wiki"} {
+@        set mimetype "text/x-fossil-wiki"
+@      } elseif {$mutype eq "Markdown"} {
+@        set mimetype text/x-markdown
+@      } elseif {$mutype eq {[links only]}} {
+@        set mimetype "text/x-fossil-plain"
+@      } else {
+@        set mimetype "text/plain"
+@      }
 @      submit_ticket
+@      set preview 1
 @   }
 @ </th1>
 @ <h1 style="text-align: center;">Enter A New Ticket</h1>
 @ <table cellpadding="5">
 @ <tr>
-@ <td colspan="2">
+@ <td colspan="3">
 @ Enter a one-line summary of the ticket:<br />
 @ <input type="text" name="title" size="60" value="$<title>" />
 @ </td>
 @ </tr>
-@ 
+@
 @ <tr>
-@ <td style="text-align: center;">Type:
-@ <th1>combobox type $type_choices 1</th1>
-@ </td>
-@ <td>What type of ticket is this?</td>
+@ <td align="right">Type:</td>
+@ <td align="left"><th1>combobox type $type_choices 1</th1></td>
+@ <td align="left">What type of ticket is this?</td>
 @ </tr>
-@ 
+@
 @ <tr>
-@ <td style="text-align: center;">Version: 
+@ <td align="right">Version:</td>
+@ <td align="left">
 @ <input type="text" name="foundin" size="20" value="$<foundin>" />
 @ </td>
-@ <td>In what version or build number do you observe the problem?</td>
+@ <td align="left">In what version or build number do you observe
+@ the problem?</td>
 @ </tr>
-@ 
+@
 @ <tr>
-@ <td style="text-align: center;">Severity:
-@ <th1>combobox severity $severity_choices 1</th1>
-@ </td>
-@ <td>How debilitating is the problem?  How badly does the problem
+@ <td align="right">Severity:</td>
+@ <td align="left"><th1>combobox severity $severity_choices 1</th1></td>
+@ <td align="left">How debilitating is the problem?  How badly does the problem
 @ affect the operation of the product?</td>
 @ </tr>
-@ 
+@
 @ <tr>
-@ <td style="text-align: center;">EMail:
-@ <input type="text" name="private_contact" value="$<private_contact>" size="30" />
+@ <td align="right">EMail:</td>
+@ <td align="left">
+@ <input type="text" name="private_contact" value="$<private_contact>"
+@  size="30" />
 @ </td>
-@ <td><span style="text-decoration: underline;">Not publicly visible</span>.
+@ <td align="left"><u>Not publicly visible</u>
 @ Used by developers to contact you with questions.</td>
 @ </tr>
-@ 
+@
 @ <tr>
-@ <td colspan="2">
+@ <td colspan="3">
 @ Enter a detailed description of the problem.
 @ For code defects, be sure to provide details on exactly how
 @ the problem can be reproduced.  Provide as much detail as
-@ possible.
+@ possible.  Format:
+@ <th1>combobox mutype {HTML {[links only]} Markdown {Plain Text} Wiki} 1</th1>
 @ <br />
 @ <th1>set nline [linecount $comment 50 10]</th1>
-@ <textarea name="comment" cols="80" rows="$nline"
-@  wrap="virtual" class="wikiedit">$<comment></textarea><br />
-@ <input type="submit" name="preview" value="Preview" /></td>
+@ <textarea name="icomment" cols="80" rows="$nline"
+@  wrap="virtual" class="wikiedit">$<icomment></textarea><br />
 @ </tr>
 @
 @ <th1>enable_output [info exists preview]</th1>
-@ <tr><td colspan="2">
+@ <tr><td colspan="3">
 @ Description Preview:<br /><hr />
-@ <th1>wiki $comment</th1>
-@ <hr />
-@ </td></tr>
+@ <th1>
+@ if {$mutype eq "Wiki"} {
+@   wiki $icomment
+@ } elseif {$mutype eq "Plain Text"} {
+@   set r [randhex]
+@   wiki "<verbatim-$r>[string trimright $icomment]\n</verbatim-$r>"
+@ } elseif {$mutype eq "Markdown"} {
+@   html [lindex [markdown "$icomment\n"] 1]
+@ } elseif {$mutype eq {[links only]}} {
+@   set r [randhex]
+@   wiki "<verbatim-$r links>[string trimright $icomment]\n</verbatim-$r>"
+@ } else {
+@   wiki "<nowiki>$icomment\n</nowiki>"
+@ }
+@ </th1>
+@ <hr /></td></tr>
 @ <th1>enable_output 1</th1>
-@ 
+@
 @ <tr>
-@ <td style="text-align: center;">
+@ <td><td align="left">
+@ <input type="submit" name="preview" value="Preview" />
+@ </td>
+@ <td align="left">See how the description will appear after formatting.</td>
+@ </tr>
+@
+@ <th1>enable_output [info exists preview]</th1>
+@ <tr>
+@ <td><td align="left">
 @ <input type="submit" name="submit" value="Submit" />
 @ </td>
-@ <td>After filling in the information above, press this button to create
-@ the new ticket</td>
+@ <td align="left">After filling in the information above, press this
+@ button to create the new ticket</td>
 @ </tr>
+@ <th1>enable_output 1</th1>
+@
 @ <tr>
-@ <td style="text-align: center;">
+@ <td><td align="left">
 @ <input type="submit" name="cancel" value="Cancel" />
 @ </td>
 @ <td>Abandon and forget this ticket</td>
@@ -364,11 +426,13 @@ static const char zDefaultNew[] =
 ** Return the code used to generate the new ticket page
 */
 const char *ticket_newpage_code(void){
-  return db_get("ticket-newpage", (char*)zDefaultNew);
+  return db_get("ticket-newpage", zDefaultNew);
 }
 
 /*
 ** WEBPAGE: tktsetup_newpage
+** Administrative page used to view or edit the TH1 script used
+** to enter new tickets.
 */
 void tktsetup_newpage_page(void){
   static const char zDesc[] =
@@ -388,11 +452,27 @@ void tktsetup_newpage_page(void){
 
 static const char zDefaultView[] =
 @ <table cellpadding="5">
-@ <tr><td class="tktDspLabel">Ticket&nbsp;UUID:</td>
-@ <td class="tktDspValue" colspan="3">$<tkt_uuid></td></tr>
+@ <tr><td class="tktDspLabel">Ticket&nbsp;Hash:</td>
+@ <th1>
+@ if {[info exists tkt_uuid]} {
+@   html "<td class='tktDspValue' colspan='3'>"
+@   copybtn hash-tk 0 $tkt_uuid 2
+@   if {[hascap s]} {
+@     html " ($tkt_id)"
+@   }
+@   html "</td></tr>\n"
+@ } else {
+@   if {[hascap s]} {
+@     html "<td class='tktDspValue' colspan='3'>Deleted "
+@     html "(0)</td></tr>\n"
+@   } else {
+@     html "<td class='tktDspValue' colspan='3'>Deleted</td></tr>\n"
+@   }
+@ }
+@ </th1>
 @ <tr><td class="tktDspLabel">Title:</td>
 @ <td class="tktDspValue" colspan="3">
-@ <th1>wiki $title</th1>
+@ $<title>
 @ </td></tr>
 @ <tr><td class="tktDspLabel">Status:</td><td class="tktDspValue">
 @ $<status>
@@ -413,7 +493,11 @@ static const char zDefaultView[] =
 @ $<resolution>
 @ </td></tr>
 @ <tr><td class="tktDspLabel">Last&nbsp;Modified:</td><td class="tktDspValue">
-@ $<tkt_datetime>
+@ <th1>
+@ if {[info exists tkt_datetime]} {
+@   html $tkt_datetime
+@ }
+@ </th1>
 @ </td>
 @ <th1>enable_output [hascap e]</th1>
 @   <td class="tktDspLabel">Contact:</td><td class="tktDspValue">
@@ -425,10 +509,58 @@ static const char zDefaultView[] =
 @ <td colspan="3" valign="top" class="tktDspValue">
 @ $<foundin>
 @ </td></tr>
-@ <tr><td>Description &amp; Comments:</td></tr>
-@ <tr><td colspan="4" class="tktDspValue">
-@ <th1>wiki $comment</th1>
-@ </td></tr>
+@
+@ <th1>
+@ if {[info exists comment]} {
+@   if {[string length $comment]>10} {
+@     html {
+@       <tr><td class="tktDspLabel">Description:</td></tr>
+@       <tr><td colspan="5" class="tktDspValue">
+@     }
+@     if {[info exists plaintext]} {
+@       set r [randhex]
+@       wiki "<verbatim-$r links>\n$comment\n</verbatim-$r>"
+@     } else {
+@       wiki $comment
+@     }
+@   }
+@ }
+@ set seenRow 0
+@ set alwaysPlaintext [info exists plaintext]
+@ query {SELECT datetime(tkt_mtime) AS xdate, login AS xlogin,
+@               mimetype as xmimetype, icomment AS xcomment,
+@               username AS xusername
+@          FROM ticketchng
+@         WHERE tkt_id=$tkt_id AND length(icomment)>0} {
+@   if {$seenRow} {
+@     html "<hr />\n"
+@   } else {
+@     html "<tr><td class='tktDspLabel'>User Comments:</td></tr>\n"
+@     html "<tr><td colspan='5' class='tktDspValue'>\n"
+@     set seenRow 1
+@   }
+@   html "[htmlize $xlogin]"
+@   if {$xlogin ne $xusername && [string length $xusername]>0} {
+@     html " (claiming to be [htmlize $xusername])"
+@   }
+@   html " added on $xdate:\n"
+@   if {$alwaysPlaintext || $xmimetype eq "text/plain"} {
+@     set r [randhex]
+@     if {$xmimetype ne "text/plain"} {html "([htmlize $xmimetype])\n"}
+@     wiki "<verbatim-$r>[string trimright $xcomment]</verbatim-$r>\n"
+@   } elseif {$xmimetype eq "text/x-fossil-wiki"} {
+@     wiki "<p>\n[string trimright $xcomment]\n</p>\n"
+@   } elseif {$xmimetype eq "text/x-markdown"} {
+@     html [lindex [markdown $xcomment] 1]
+@   } elseif {$xmimetype eq "text/html"} {
+@     wiki "<p><nowiki>\n[string trimright $xcomment]\n</nowiki>\n"
+@   } else {
+@     set r [randhex]
+@     wiki "<verbatim-$r links>[string trimright $xcomment]</verbatim-$r>\n"
+@   }
+@ }
+@ if {$seenRow} {html "</td></tr>\n"}
+@ </th1>
 @ </table>
 ;
 
@@ -437,11 +569,13 @@ static const char zDefaultView[] =
 ** Return the code used to generate the view ticket page
 */
 const char *ticket_viewpage_code(void){
-  return db_get("ticket-viewpage", (char*)zDefaultView);
+  return db_get("ticket-viewpage", zDefaultView);
 }
 
 /*
 ** WEBPAGE: tktsetup_viewpage
+** Administrative page used to view or edit the TH1 script that
+** displays individual tickets.
 */
 void tktsetup_viewpage_page(void){
   static const char zDesc[] =
@@ -460,98 +594,119 @@ void tktsetup_viewpage_page(void){
 
 static const char zDefaultEdit[] =
 @ <th1>
+@   if {![info exists mutype]} {set mutype Markdown}
+@   if {![info exists icomment]} {set icomment {}}
 @   if {![info exists username]} {set username $login}
 @   if {[info exists submit]} {
-@     if {[info exists cmappnd]} {
-@       if {[string length $cmappnd]>0} {
-@         set ctxt "\n\n<hr /><i>[htmlize $login]"
-@         if {$username ne $login} {
-@           set ctxt "$ctxt claiming to be [htmlize $username]"
-@         }
-@         set ctxt "$ctxt added on [date] UTC:</i><br />\n$cmappnd"
-@         append_field comment $ctxt
-@       }
+@     if {$mutype eq "Wiki"} {
+@       set mimetype text/x-fossil-wiki
+@     } elseif {$mutype eq "Markdown"} {
+@       set mimetype text/x-markdown
+@     } elseif {$mutype eq "HTML"} {
+@       set mimetype text/html
+@     } elseif {$mutype eq {[links only]}} {
+@       set mimetype text/x-fossil-plain
+@     } else {
+@       set mimetype text/plain
 @     }
 @     submit_ticket
+@     set preview 1
 @   }
 @ </th1>
 @ <table cellpadding="5">
 @ <tr><td class="tktDspLabel">Title:</td><td>
 @ <input type="text" name="title" value="$<title>" size="60" />
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Status:</td><td>
 @ <th1>combobox status $status_choices 1</th1>
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Type:</td><td>
 @ <th1>combobox type $type_choices 1</th1>
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Severity:</td><td>
 @ <th1>combobox severity $severity_choices 1</th1>
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Priority:</td><td>
 @ <th1>combobox priority $priority_choices 1</th1>
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Resolution:</td><td>
 @ <th1>combobox resolution $resolution_choices 1</th1>
 @ </td></tr>
+@
 @ <tr><td class="tktDspLabel">Subsystem:</td><td>
 @ <th1>combobox subsystem $subsystem_choices 1</th1>
 @ </td></tr>
+@
 @ <th1>enable_output [hascap e]</th1>
 @   <tr><td class="tktDspLabel">Contact:</td><td>
 @   <input type="text" name="private_contact" size="40"
 @    value="$<private_contact>" />
 @   </td></tr>
 @ <th1>enable_output 1</th1>
+@
 @ <tr><td class="tktDspLabel">Version&nbsp;Found&nbsp;In:</td><td>
 @ <input type="text" name="foundin" size="50" value="$<foundin>" />
 @ </td></tr>
+@
 @ <tr><td colspan="2">
-@ <th1>
-@   if {![info exists eall]} {set eall 0}
-@   if {[info exists aonlybtn]} {set eall 0}
-@   if {[info exists eallbtn]} {set eall 1}
-@   if {![hascap w]} {set eall 0}
-@   if {![info exists cmappnd]} {set cmappnd {}}
-@   set nline [linecount $comment 15 10]
-@   enable_output $eall
-@ </th1>
-@   Description And Comments:<br />
-@   <textarea name="comment" cols="80" rows="$nline"
-@    wrap="virtual" class="wikiedit">$<comment></textarea><br />
-@   <input type="hidden" name="eall" value="1" />
-@   <input type="submit" name="aonlybtn" value="Append Remark" />
-@   <input type="submit" name="preview1btn" value="Preview" />
-@ <th1>enable_output [expr {!$eall}]</th1>
-@   Append Remark from 
+@   Append Remark with format
+@  <th1>combobox mutype {HTML {[links only]} Markdown {Plain Text} Wiki} 1</th1>
+@   from
 @   <input type="text" name="username" value="$<username>" size="30" />:<br />
-@   <textarea name="cmappnd" cols="80" rows="15"
-@    wrap="virtual" class="wikiedit">$<cmappnd></textarea><br />
-@ <th1>enable_output [expr {[hascap w] && !$eall}]</th1>
-@   <input type="submit" name="eallbtn" value="Edit All" />
-@ <th1>enable_output [expr {!$eall}]</th1>
-@   <input type="submit" name="preview2btn" value="Preview" />
-@ <th1>enable_output 1</th1>
+@   <textarea name="icomment" cols="80" rows="15"
+@    wrap="virtual" class="wikiedit">$<icomment></textarea>
 @ </td></tr>
 @
-@ <th1>enable_output [info exists preview1btn]</th1>
+@ <th1>enable_output [info exists preview]</th1>
 @ <tr><td colspan="2">
 @ Description Preview:<br /><hr />
-@ <th1>wiki $comment</th1>
-@ <hr />
-@ </td></tr>
-@ <th1>enable_output [info exists preview2btn]</th1>
-@ <tr><td colspan="2">
-@ Description Preview:<br /><hr />
-@ <th1>wiki $cmappnd</th1>
+@ <th1>
+@ if {$mutype eq "Wiki"} {
+@   wiki $icomment
+@ } elseif {$mutype eq "Plain Text"} {
+@   set r [randhex]
+@   wiki "<verbatim-$r>\n[string trimright $icomment]\n</verbatim-$r>"
+@ } elseif {$mutype eq "Markdown"} {
+@   html [lindex [markdown "$icomment\n"] 1]
+@ } elseif {$mutype eq {[links only]}} {
+@   set r [randhex]
+@   wiki "<verbatim-$r links>\n[string trimright $icomment]</verbatim-$r>"
+@ } else {
+@   wiki "<nowiki>\n[string trimright $icomment]\n</nowiki>"
+@ }
+@ </th1>
 @ <hr />
 @ </td></tr>
 @ <th1>enable_output 1</th1>
 @
-@ <tr><td align="right"></td><td>
-@ <input type="submit" name="submit" value="Submit Changes" />
+@ <tr>
+@ <td align="right">
+@ <input type="submit" name="preview" value="Preview" />
+@ </td>
+@ <td align="left">See how the description will appear after formatting.</td>
+@ </tr>
+@
+@ <th1>enable_output [info exists preview]</th1>
+@ <tr>
+@ <td align="right">
+@ <input type="submit" name="submit" value="Submit" />
+@ </td>
+@ <td align="left">Apply the changes shown above</td>
+@ </tr>
+@ <th1>enable_output 1</th1>
+@
+@ <tr>
+@ <td align="right">
 @ <input type="submit" name="cancel" value="Cancel" />
-@ </td></tr>
+@ </td>
+@ <td>Abandon this edit</td>
+@ </tr>
+@
 @ </table>
 ;
 
@@ -559,11 +714,13 @@ static const char zDefaultEdit[] =
 ** Return the code used to generate the edit ticket page
 */
 const char *ticket_editpage_code(void){
-  return db_get("ticket-editpage", (char*)zDefaultEdit);
+  return db_get("ticket-editpage", zDefaultEdit);
 }
 
 /*
 ** WEBPAGE: tktsetup_editpage
+** Administrative page for viewing or editing the TH1 script that
+** drives the ticket editing page.
 */
 void tktsetup_editpage_page(void){
   static const char zDesc[] =
@@ -585,21 +742,26 @@ void tktsetup_editpage_page(void){
 */
 static const char zDefaultReportList[] =
 @ <th1>
-@ if {[hascap n]} {
+@ if {[anoncap n]} {
 @   html "<p>Enter a new ticket:</p>"
 @   html "<ul><li><a href='tktnew'>New ticket</a></li></ul>"
 @ }
 @ </th1>
-@ 
+@
 @ <p>Choose a report format from the following list:</p>
 @ <ol>
 @ <th1>html $report_items</th1>
 @ </ol>
-@ 
+@
 @ <th1>
-@ if {[hascap t]} {
-@   html "<p>Create a new ticket display format:</p>"
-@   html "<ul><li><a href='rptnew'>New report format</a></li></ul>"
+@ if {[anoncap t q]} {
+@   html "<p>Other options:</p>\n<ul>\n"
+@   if {[anoncap t]} {
+@     html "<li><a href='rptnew'>New report format</a></li>\n"
+@   }
+@   if {[anoncap q]} {
+@     html "<li><a href='modreq'>Tend to pending moderation requests</a></li>\n"
+@   }
 @ }
 @ </th1>
 ;
@@ -608,11 +770,13 @@ static const char zDefaultReportList[] =
 ** Return the code used to generate the report list
 */
 const char *ticket_reportlist_code(void){
-  return db_get("ticket-reportlist", (char*)zDefaultReportList);
+  return db_get("ticket-reportlist", zDefaultReportList);
 }
 
 /*
 ** WEBPAGE: tktsetup_reportlist
+** Administrative page used to view or edit the TH1 script that
+** defines the "report list" page.
 */
 void tktsetup_reportlist(void){
   static const char zDesc[] =
@@ -632,7 +796,7 @@ void tktsetup_reportlist(void){
 /*
 ** The default template ticket report format:
 */
-static char zDefaultReport[] = 
+static char zDefaultReport[] =
 @ SELECT
 @   CASE WHEN status IN ('Open','Verified') THEN '#f2dcdc'
 @        WHEN status='Review' THEN '#e8e8e8'
@@ -660,6 +824,9 @@ char *ticket_report_template(void){
 
 /*
 ** WEBPAGE: tktsetup_rpttplt
+**
+** Administrative page used to view or edit the ticket report
+** template.
 */
 void tktsetup_rpttplt_page(void){
   static const char zDesc[] =
@@ -681,7 +848,7 @@ void tktsetup_rpttplt_page(void){
 /*
 ** The default template ticket key:
 */
-static const char zDefaultKey[] = 
+static const char zDefaultKey[] =
 @ #ffffff Key:
 @ #f2dcdc Active
 @ #e8e8e8 Review
@@ -696,11 +863,14 @@ static const char zDefaultKey[] =
 ** Return the template ticket report format:
 */
 const char *ticket_key_template(void){
-  return db_get("ticket-key-template", (char*)zDefaultKey);
+  return db_get("ticket-key-template", zDefaultKey);
 }
 
 /*
 ** WEBPAGE: tktsetup_keytplt
+**
+** Administrative page used to view or edit the Key template
+** for tickets.
 */
 void tktsetup_keytplt_page(void){
   static const char zDesc[] =
@@ -721,36 +891,46 @@ void tktsetup_keytplt_page(void){
 
 /*
 ** WEBPAGE: tktsetup_timeline
+**
+** Administrative page used ot configure how tickets are
+** rendered on timeline views.
 */
 void tktsetup_timeline_page(void){
   login_check_credentials();
   if( !g.perm.Setup ){
-    login_needed();
+    login_needed(0);
+    return;
   }
 
   if( P("setup") ){
     cgi_redirect("tktsetup");
   }
+  style_set_current_feature("tktsetup");
   style_header("Ticket Display On Timelines");
   db_begin_transaction();
-  @ <form action="%s(g.zTop)/tktsetup_timeline" method="post"><div>
+  @ <form action="%R/tktsetup_timeline" method="post"><div>
   login_insert_csrf_secret();
 
   @ <hr />
-  entry_attribute("Ticket Title", 40, "ticket-title-expr", "t", "title");
+  entry_attribute("Ticket Title", 40, "ticket-title-expr", "t",
+                  "title", 0);
   @ <p>An SQL expression in a query against the TICKET table that will
-  @ return the title of the ticket for display purposes.</p>
+  @ return the title of the ticket for display purposes.
+  @ (Property: ticket-title-expr)</p>
 
   @ <hr />
-  entry_attribute("Ticket Status", 40, "ticket-status-column", "s", "status");
+  entry_attribute("Ticket Status", 40, "ticket-status-column", "s",
+                  "status", 0);
   @ <p>The name of the column in the TICKET table that contains the ticket
-  @ status in human-readable form.  Case sensitive.</p>
+  @ status in human-readable form.  Case sensitive.
+  @ (Property: ticket-status-column)</p>
 
   @ <hr />
   entry_attribute("Ticket Closed", 40, "ticket-closed-expr", "c",
-                  "status='Closed'");
+                  "status='Closed'", 0);
   @ <p>An SQL expression that evaluates to true in a TICKET table query if
-  @ the ticket is closed.</p>
+  @ the ticket is closed.
+  @ (Property: ticket-closed-expr)</p>
 
   @ <hr />
   @ <p>
@@ -759,6 +939,6 @@ void tktsetup_timeline_page(void){
   @ </p>
   @ </div></form>
   db_end_transaction(0);
-  style_footer();
-  
+  style_finish_page();
+
 }
