@@ -64,7 +64,10 @@ void unversioned_schema(void){
 */
 const char *unversioned_content_hash(int debugFlag){
   const char *zHash = debugFlag ? 0 : db_get("uv-hash", 0);
-  if( zHash==0 ){
+  if( zHash ) return zHash;
+  if( !db_table_exists("repository","unversioned") ){
+    return "da39a3ee5e6b4b0d3255bfef95601890afd80709";
+  }else{
     Stmt q;
     db_prepare(&q,
       "SELECT printf('%%s %%s %%s\n',name,datetime(mtime,'unixepoch'),hash)"
@@ -87,21 +90,36 @@ const char *unversioned_content_hash(int debugFlag){
 /*
 ** Initialize pContent to be the content of an unversioned file zName.
 **
-** Return 0 on success.  Return 1 if zName is not found.
+** Return 0 on failures.
+** Return 1 if the file is found by name.
+** Return 2 if the file is found by hash.
 */
 int unversioned_content(const char *zName, Blob *pContent){
   Stmt q;
-  int rc = 1;
+  int rc = 0;
   blob_init(pContent, 0, 0);
-  db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE name=%Q", zName);
+  db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE name=%Q",
+                 zName);
   if( db_step(&q)==SQLITE_ROW ){
     db_column_blob(&q, 1, pContent);
     if( db_column_int(&q, 0)==1 ){
       blob_uncompress(pContent, pContent);
     }
-    rc = 0;
+    rc = 1;
   }
   db_finalize(&q);
+  if( rc==0 && validate16(zName,-1) ){
+    db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE hash=%Q",
+                   zName);
+    if( db_step(&q)==SQLITE_ROW ){
+      db_column_blob(&q, 1, pContent);
+      if( db_column_int(&q, 0)==1 ){
+        blob_uncompress(pContent, pContent);
+      }
+      rc = 2;
+    }
+    db_finalize(&q);
+  }
   return rc;
 }
 
@@ -121,7 +139,7 @@ static void unversioned_write(
     "REPLACE INTO unversioned(name,rcvid,mtime,hash,sz,encoding,content)"
     " VALUES(:name,:rcvid,:mtime,:hash,:sz,:encoding,:content)"
   );
-  sha1sum_blob(pContent, &hash);
+  hname_hash(pContent, 0, &hash);
   blob_compress(pContent, &compressed);
   db_bind_text(&ins, ":name", zUVFile);
   db_bind_int(&ins, ":rcvid", g.rcvid);
@@ -145,7 +163,7 @@ static void unversioned_write(
 
 /*
 ** Check the status of unversioned file zName.  "mtime" and "zHash" are the
-** time of last change and SHA1 hash of a copy of this file on a remote
+** time of last change and hash of a copy of this file on a remote
 ** server.  Return an integer status code as follows:
 **
 **    0:     zName does not exist in the unversioned table.
@@ -155,7 +173,11 @@ static void unversioned_write(
 **    4:     zName exists and is the same as zHash but has a newer mtime.
 **    5:     zName exists and should override the mtime/zHash remote.
 */
-int unversioned_status(const char *zName, sqlite3_int64 mtime, const char *zHash){
+int unversioned_status(
+  const char *zName,
+  sqlite3_int64 mtime,
+  const char *zHash
+){
   int iStatus = 0;
   Stmt q;
   db_prepare(&q, "SELECT mtime, hash FROM unversioned WHERE name=%Q", zName);
@@ -216,50 +238,65 @@ static int contains_whitespace(const char *zName){
 **
 ** Subcommands:
 **
-**    add FILE ...         Add or update an unversioned files in the local
-**                         repository so that it matches FILE on disk.
-**                         Use "--as UVFILE" to give the file a different name
-**                         in the repository than what it called on disk.
-**                         Changes are not pushed to other repositories until
-**                         the next sync.
+**    add FILE ...           Add or update one or more unversioned files in
+**                           the local repository so that they match FILEs
+**                           on disk. Changes are not pushed to other
+**                           repositories until the next sync.
 **
-**    cat FILE ...         Concatenate the content of FILEs to stdout.
+**    add FILE --as UVFILE   Add or update a single file named FILE on disk
+**                           and UVFILE in the repository unversioned file
+**                           namespace. This variant of the 'add' command allows
+**                           the name to be different in the repository versus
+**                           what appears on disk, but it only allows adding
+**                           a single file at a time.
 **
-**    edit FILE            Bring up FILE in a text editor for modification.
+**    cat FILE ...           Concatenate the content of FILEs to stdout.
 **
-**    export FILE OUTPUT   Write the content of FILE into OUTPUT on disk
+**    edit FILE              Bring up FILE in a text editor for modification.
 **
-**    list | ls            Show all unversioned files held in the local
-**                         repository.
+**    export FILE OUTPUT     Write the content of FILE into OUTPUT on disk
 **
-**    revert ?URL?         Restore the state of all unversioned files in the
-**                         local repository to match the remote repository
-**                         URL.
+**    list | ls              Show all unversioned files held in the local
+**                           repository. Options:
 **
-**                         Options:
-**                            -v|--verbose     Extra diagnostic output
-**                            -n|--dryrun      Show what would have happened
+**                              --glob PATTERN   Show only files that match
+**                              --like PATTERN   Show only files that match
+**                              -l               Show additional details for
+**                                               files that match. Implied
+**                                               when 'list' is used.
+**
+**    revert ?URL?           Restore the state of all unversioned files in the
+**                           local repository to match the remote repository
+**                           URL.
+**
+**                           Options:
+**                              -v|--verbose     Extra diagnostic output
+**                              -n|--dryrun      Show what would have happened
 **
 **    remove|rm|delete FILE ...
-**                         Remove unversioned files from the local repository.
-**                         Changes are not pushed to other repositories until
-**                         the next sync.
+**                           Remove unversioned files from the local repository.
+**                           Changes are not pushed to other repositories until
+**                           the next sync.  Options:
 **
-**    sync ?URL?           Synchronize the state of all unversioned files with
-**                         the remote repository URL.  The most recent version
-**                         of each file is propagate to all repositories and
-**                         all prior versions are permanently forgotten.
+**                              --glob PATTERN   Remove files that match
+**                              --like PATTERN   Remove files that match
 **
-**                         Options:
-**                            -v|--verbose     Extra diagnostic output
-**                            -n|--dryrun      Show what would have happened
+**    sync ?URL?             Synchronize the state of all unversioned files with
+**                           the remote repository URL.  The most recent version
+**                           of each file is propagated to all repositories and
+**                           all prior versions are permanently forgotten.
 **
-**    touch FILE ...       Update the TIMESTAMP on all of the listed files
+**                           Options:
+**                              -v|--verbose     Extra diagnostic output
+**                              -n|--dryrun      Show what would have happened
+**
+**    touch FILE ...         Update the TIMESTAMP on all of the listed files
 **
 ** Options:
 **
-**   --mtime TIMESTAMP     Use TIMESTAMP instead of "now" for the "add",
-**                         "edit", "remove", and "touch" subcommands.
+**   --mtime TIMESTAMP       Use TIMESTAMP instead of "now" for the "add",
+**                           "edit", "remove", and "touch" subcommands.
+**   -R|--repository REPO    Use FILE as the repository
 */
 void unversioned_cmd(void){
   const char *zCmd;
@@ -284,8 +321,8 @@ void unversioned_cmd(void){
     int i;
 
     zAs = find_option("as",0,1);
-    if( zAs && g.argc!=4 ) usage("add DISKFILE --as UVFILE");
     verify_all_options();
+    if( zAs && g.argc!=4 ) usage("add DISKFILE --as UVFILE");
     db_begin_transaction();
     content_rcvid_init("#!fossil unversioned add");
     for(i=3; i<g.argc; i++){
@@ -314,7 +351,7 @@ void unversioned_cmd(void){
     db_begin_transaction();
     for(i=3; i<g.argc; i++){
       Blob content;
-      if( unversioned_content(g.argv[i], &content)==0 ){
+      if( unversioned_content(g.argv[i], &content)!=0 ){
         blob_write_to_file(&content, "-");
       }
       blob_reset(&content);
@@ -331,12 +368,14 @@ void unversioned_cmd(void){
     if( g.argc!=4) usage("edit UVFILE");
     zUVFile = g.argv[3];
     zEditor = fossil_text_editor();
-    if( zEditor==0 ) fossil_fatal("no text editor - set the VISUAL env variable");
+    if( zEditor==0 ){
+      fossil_fatal("no text editor - set the VISUAL env variable");
+    }
     zTFile = fossil_temp_filename();
     if( zTFile==0 ) fossil_fatal("cannot find a temporary filename");
     db_begin_transaction();
     content_rcvid_init("#!fossil unversioned edit");
-    if( unversioned_content(zUVFile, &content) ){
+    if( unversioned_content(zUVFile, &content)==0 ){
       fossil_fatal("no such uv-file: %Q", zUVFile);
     }
     if( looks_like_binary(&content) ){
@@ -346,7 +385,7 @@ void unversioned_cmd(void){
     blob_add_cr(&content);
 #endif
     blob_write_to_file(&content, zTFile);
-    zCmd = mprintf("%s \"%s\"", zEditor, zTFile);
+    zCmd = mprintf("%s %$", zEditor, zTFile);
     if( fossil_system(zCmd) ){
       fossil_fatal("editor aborted: %Q", zCmd);
     }
@@ -365,7 +404,7 @@ void unversioned_cmd(void){
     Blob content;
     verify_all_options();
     if( g.argc!=5 ) usage("export UVFILE OUTPUT");
-    if( unversioned_content(g.argv[3], &content) ){
+    if( unversioned_content(g.argv[3], &content)==0 ){
       fossil_fatal("no such uv-file: %Q", g.argv[3]);
     }
     blob_write_to_file(&content, g.argv[4]);
@@ -378,13 +417,27 @@ void unversioned_cmd(void){
     Stmt q;
     int allFlag = find_option("all","a",0)!=0;
     int longFlag = find_option("l",0,0)!=0 || (nCmd>1 && zCmd[1]=='i');
+    char *zPattern = sqlite3_mprintf("true");
+    const char *zGlob;
+    zGlob = find_option("glob",0,1);
+    if( zGlob ){
+      sqlite3_free(zPattern);
+      zPattern = sqlite3_mprintf("(name GLOB %Q)", zGlob);
+    }
+    zGlob = find_option("like",0,1);
+    if( zGlob ){
+      sqlite3_free(zPattern);
+      zPattern = sqlite3_mprintf("(name LIKE %Q)", zGlob);
+    }
     verify_all_options();
     if( !longFlag ){
       if( allFlag ){
-        db_prepare(&q, "SELECT name FROM unversioned ORDER BY name");
+        db_prepare(&q, "SELECT name FROM unversioned WHERE %s ORDER BY name",
+                   zPattern/*safe-for-%s*/);
       }else{
-        db_prepare(&q, "SELECT name FROM unversioned WHERE hash IS NOT NULL"
-                       " ORDER BY name");
+        db_prepare(&q, "SELECT name FROM unversioned"
+                       " WHERE %s AND hash IS NOT NULL"
+                       " ORDER BY name", zPattern/*safe-for-%s*/);
       }
       while( db_step(&q)==SQLITE_ROW ){
         fossil_print("%s\n", db_column_text(&q,0));
@@ -392,8 +445,8 @@ void unversioned_cmd(void){
     }else{
       db_prepare(&q,
         "SELECT hash, datetime(mtime,'unixepoch'), sz, length(content), name"
-        "   FROM unversioned"
-        "  ORDER BY name;"
+        "   FROM unversioned WHERE %s"
+        "  ORDER BY name;", zPattern/*safe-for-%s*/
       );
       while( db_step(&q)==SQLITE_ROW ){
         const char *zHash = db_column_text(&q, 0);
@@ -415,16 +468,33 @@ void unversioned_cmd(void){
       }
     }
     db_finalize(&q);
+    sqlite3_free(zPattern);
   }else if( memcmp(zCmd, "revert", nCmd)==0 ){
-    unsigned syncFlags = unversioned_sync_flags(SYNC_UNVERSIONED|SYNC_UV_REVERT);
+    unsigned syncFlags = 
+        unversioned_sync_flags(SYNC_UNVERSIONED|SYNC_UV_REVERT);
     g.argv[1] = "sync";
     g.argv[2] = "--uv-noop";
     sync_unversioned(syncFlags);
   }else if( memcmp(zCmd, "remove", nCmd)==0 || memcmp(zCmd, "rm", nCmd)==0
          || memcmp(zCmd, "delete", nCmd)==0 ){
     int i;
-    verify_all_options();
+    const char *zGlob;
     db_begin_transaction();
+    while( (zGlob = find_option("glob",0,1))!=0 ){
+      db_multi_exec(
+        "UPDATE unversioned"
+        "   SET hash=NULL, content=NULL, mtime=%lld, sz=0 WHERE name GLOB %Q",
+        mtime, zGlob
+      );
+    }
+    while( (zGlob = find_option("like",0,1))!=0 ){
+      db_multi_exec(
+        "UPDATE unversioned"
+        "   SET hash=NULL, content=NULL, mtime=%lld, sz=0 WHERE name LIKE %Q",
+        mtime, zGlob
+      );
+    }
+    verify_all_options();
     for(i=3; i<g.argc; i++){
       db_multi_exec(
         "UPDATE unversioned"
@@ -477,10 +547,11 @@ void uvlist_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  etag_check(ETAG_DATA,0);
   style_header("Unversioned Files");
   if( !db_table_exists("repository","unversioned") ){
     @ No unversioned files on this server
-    style_footer();
+    style_finish_page();
     return;
   }
   if( PB("byage") ) zOrderBy = "mtime DESC";
@@ -519,7 +590,7 @@ void uvlist_page(void){
       @   <th> Age
       @   <th> Size
       @   <th> User
-      @   <th> SHA1
+      @   <th> Hash
       if( g.perm.Admin ){
         @ <th> rcvid
       }
@@ -556,7 +627,7 @@ void uvlist_page(void){
   if( n ){
     approxSizeName(sizeof(zSzName), zSzName, iTotalSz);
     @ </tbody>
-    @ <tfoot><tr><td><b>Total over %d(cnt) files</b><td><td>%s(zSzName)
+    @ <tfoot><tr><td><b>Total for %d(cnt) files</b><td><td>%s(zSzName)
     @ <td><td>
     if( g.perm.Admin ){
       @ <td>
@@ -566,7 +637,7 @@ void uvlist_page(void){
   }else{
     @ No unversioned files on this server.
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -588,7 +659,8 @@ void uvlist_json_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  cgi_set_content_type("text/json");
+  cgi_set_content_type("application/json");
+  etag_check(ETAG_DATA,0);
   if( !db_table_exists("repository","unversioned") ){
     blob_init(&json, "[]", -1);
     cgi_set_content(&json);
@@ -612,14 +684,12 @@ void uvlist_json_page(void){
      int fullSize = db_column_int(&q, 3);
      const char *zLogin = db_column_text(&q, 4);
      if( zLogin==0 ) zLogin = "";
-     blob_appendf(&json, "%s{\"name\":\"", zSep);
+     blob_appendf(&json, "%s{\"name\":\"%j\",\n", zSep, zName);
      zSep = ",\n ";
-     blob_append_json_string(&json, zName);
-     blob_appendf(&json, "\",\n  \"mtime\":%lld,\n  \"hash\":\"", mtime);
-     blob_append_json_string(&json, zHash);
-     blob_appendf(&json, "\",\n  \"size\":%d,\n  \"user\":\"", fullSize);
-     blob_append_json_string(&json, zLogin);
-     blob_appendf(&json, "\"}");
+     blob_appendf(&json, "  \"mtime\":%lld,\n", mtime);
+     blob_appendf(&json, "  \"hash\":\"%j\",\n", zHash);
+     blob_appendf(&json, "  \"size\":%d,\n", fullSize);
+     blob_appendf(&json, "  \"user\":\"%j\"}", zLogin);
    }
    db_finalize(&q);
    blob_appendf(&json,"]\n");
