@@ -21,10 +21,6 @@
 #include "config.h"
 #include "user.h"
 
-#if defined(_WIN32)
-#include <conio.h>
-#endif
-
 /*
 ** Strip leading and trailing space from a string and add the string
 ** onto the end of a blob.
@@ -44,60 +40,78 @@ static void strip_string(Blob *pBlob, char *z){
   blob_append(pBlob, z, -1);
 }
 
-#if defined(_WIN32) || defined(__BIONIC__)
-#ifdef __MINGW32__
+#if defined(_WIN32) || (defined(__BIONIC__) && !defined(FOSSIL_HAVE_GETPASS))
+#ifdef _WIN32
 #include <conio.h>
 #endif
-/*
-** getpass for Windows and Android
-*/
-static char *getpass(const char *prompt){
-  static char pwd[64];
-  size_t i;
 
+/*
+** getpass() for Windows and Android.
+*/
+static char *zPwdBuffer = 0;
+static size_t nPwdBuffer = 0;
+
+static char *getpass(const char *prompt){
+  char *zPwd;
+  size_t nPwd;
+  size_t i;
+#if defined(_WIN32)
+  int useGetch = _isatty(_fileno(stderr));
+#endif
+
+  if( zPwdBuffer==0 ){
+    zPwdBuffer = fossil_secure_alloc_page(&nPwdBuffer);
+    assert( zPwdBuffer );
+  }else{
+    fossil_secure_zero(zPwdBuffer, nPwdBuffer);
+  }
+  zPwd = zPwdBuffer;
+  nPwd = nPwdBuffer;
   fputs(prompt,stderr);
   fflush(stderr);
-  for(i=0; i<sizeof(pwd)-1; ++i){
+  assert( zPwd!=0 );
+  assert( nPwd>0 );
+  for(i=0; i<nPwd-1; ++i){
 #if defined(_WIN32)
-    pwd[i] = _getch();
+    zPwd[i] = useGetch ? _getch() : getc(stdin);
 #else
-    pwd[i] = getc(stdin);
+    zPwd[i] = getc(stdin);
 #endif
-    if(pwd[i]=='\r' || pwd[i]=='\n'){
+    if(zPwd[i]=='\r' || zPwd[i]=='\n'){
       break;
     }
     /* BS or DEL */
-    else if(i>0 && (pwd[i]==8 || pwd[i]==127)){
+    else if(i>0 && (zPwd[i]==8 || zPwd[i]==127)){
       i -= 2;
       continue;
     }
     /* CTRL-C */
-    else if(pwd[i]==3) {
+    else if(zPwd[i]==3) {
       i=0;
       break;
     }
     /* ESC */
-    else if(pwd[i]==27){
+    else if(zPwd[i]==27){
       i=0;
       break;
     }
     else{
+#if defined(_WIN32)
+      if( useGetch )
+#endif
       fputc('*',stderr);
     }
   }
-  pwd[i]='\0';
+  zPwd[i]='\0';
   fputs("\n", stderr);
-  return pwd;
+  assert( zPwd==zPwdBuffer );
+  return zPwd;
+}
+void freepass(){
+  if( !zPwdBuffer ) return;
+  assert( nPwdBuffer>0 );
+  fossil_secure_free_page(zPwdBuffer, nPwdBuffer);
 }
-#endif
-
-#if defined(_WIN32) || defined(WIN32)
-# include <io.h>
-# include <fcntl.h>
-# undef popen
-# define popen _popen
-# undef pclose
-# define pclose _pclose
 #endif
 
 /*
@@ -305,32 +319,36 @@ void prompt_user(const char *zPrompt, Blob *pIn){
 /*
 ** COMMAND: user*
 **
-** Usage: %fossil user SUBCOMMAND ...  ?-R|--repository FILE?
+** Usage: %fossil user SUBCOMMAND ...  ?-R|--repository REPO?
 **
 ** Run various subcommands on users of the open repository or of
 ** the repository identified by the -R or --repository option.
 **
-**    %fossil user capabilities USERNAME ?STRING?
+** > fossil user capabilities USERNAME ?STRING?
 **
 **        Query or set the capabilities for user USERNAME
 **
-**    %fossil user default ?USERNAME?
+** > fossil user contact USERNAME ?CONTACT-INFO?
+**
+**        Query or set contact information for user USERNAME
+**
+** > fossil user default ?USERNAME?
 **
 **        Query or set the default user.  The default user is the
 **        user for command-line interaction.
 **
-**    %fossil user list
-**    %fossil user ls
+** > fossil user list
+** > fossil user ls
 **
 **        List all users known to the repository
 **
-**    %fossil user new ?USERNAME? ?CONTACT-INFO? ?PASSWORD?
+** > fossil user new ?USERNAME? ?CONTACT-INFO? ?PASSWORD?
 **
 **        Create a new user in the repository.  Users can never be
 **        deleted.  They can be denied all access but they must continue
 **        to exist in the database.
 **
-**    %fossil user password USERNAME ?PASSWORD?
+** > fossil user password USERNAME ?PASSWORD?
 **
 **        Change the web access password for a user.
 */
@@ -338,13 +356,13 @@ void user_cmd(void){
   int n;
   db_find_and_open_repository(0, 0);
   if( g.argc<3 ){
-    usage("capabilities|default|list|new|password ...");
+    usage("capabilities|contact|default|list|new|password ...");
   }
   n = strlen(g.argv[2]);
   if( n>=2 && strncmp(g.argv[2],"new",n)==0 ){
     Blob passwd, login, caps, contact;
     char *zPw;
-    blob_init(&caps, db_get("default-perms", "u"), -1);
+    blob_init(&caps, db_get("default-perms", 0), -1);
 
     if( g.argc>=4 ){
       blob_init(&login, g.argv[3], -1);
@@ -365,11 +383,13 @@ void user_cmd(void){
       prompt_for_password("password: ", &passwd, 1);
     }
     zPw = sha1_shared_secret(blob_str(&passwd), blob_str(&login), 0);
+    db_unprotect(PROTECT_USER);
     db_multi_exec(
       "INSERT INTO user(login,pw,cap,info,mtime)"
       "VALUES(%B,%Q,%B,%B,now())",
       &login, zPw, &caps, &contact
     );
+    db_protect_pop();
     free(zPw);
   }else if( n>=2 && strncmp(g.argv[2],"default",n)==0 ){
     if( g.argc==3 ){
@@ -411,8 +431,10 @@ void user_cmd(void){
       fossil_print("password unchanged\n");
     }else{
       char *zSecret = sha1_shared_secret(blob_str(&pw), g.argv[3], 0);
+      db_unprotect(PROTECT_USER);
       db_multi_exec("UPDATE user SET pw=%Q, mtime=now() WHERE uid=%d",
                     zSecret, uid);
+      db_protect_pop();
       free(zSecret);
     }
   }else if( n>=2 && strncmp(g.argv[2],"capabilities",2)==0 ){
@@ -425,15 +447,35 @@ void user_cmd(void){
       fossil_fatal("no such user: %s", g.argv[3]);
     }
     if( g.argc==5 ){
+      db_unprotect(PROTECT_USER);
       db_multi_exec(
         "UPDATE user SET cap=%Q, mtime=now() WHERE uid=%d",
         g.argv[4], uid
       );
+      db_protect_pop();
     }
     fossil_print("%s\n", db_text(0, "SELECT cap FROM user WHERE uid=%d", uid));
+  }else if( n>=2 && strncmp(g.argv[2], "contact", 2)==0 ){
+    int uid;
+    if( g.argc!=4 && g.argc!=5 ){
+      usage("contact USERNAME ?CONTACT-INFO?");
+    }
+    uid = db_int(0, "SELECT uid FROM user WHERE login=%Q", g.argv[3]);
+    if( uid==0 ){
+      fossil_fatal("no such user: %s", g.argv[3]);
+    }
+    if( g.argc==5 ){
+      db_unprotect(PROTECT_USER);
+      db_multi_exec(
+        "UPDATE user SET info=%Q, mtime=now() WHERE uid=%d",
+        g.argv[4], uid
+      );
+      db_protect_pop();
+    }
+    fossil_print("%s\n", db_text(0, "SELECT info FROM user WHERE uid=%d", uid));
   }else{
     fossil_fatal("user subcommand should be one of: "
-                 "capabilities default list new password");
+                 "capabilities contact default list new password");
   }
 }
 
@@ -477,6 +519,7 @@ static int attempt_user(const char *zLogin){
 ** The user name is stored in g.zLogin.  The uid is in g.userUid.
 */
 void user_select(void){
+  UrlData url;
   if( g.userUid ) return;
   if( g.zLogin ){
     if( attempt_user(g.zLogin)==0 ){
@@ -498,8 +541,9 @@ void user_select(void){
 
   if( attempt_user(fossil_getenv("USERNAME")) ) return;
 
-  url_parse(0, 0);
-  if( g.url.user && attempt_user(g.url.user) ) return;
+  memset(&url, 0, sizeof(url));
+  url_parse_local(0, 0, &url);
+  if( url.user && attempt_user(url.user) ) return;
 
   fossil_print(
     "Cannot figure out who you are!  Consider using the --user\n"
@@ -550,6 +594,7 @@ void user_hash_passwords_cmd(void){
   db_open_repository(g.argv[2]);
   sqlite3_create_function(g.db, "shared_secret", 2, SQLITE_UTF8, 0,
                           sha1_shared_secret_sql_function, 0, 0);
+  db_unprotect(PROTECT_ALL);
   db_multi_exec(
     "UPDATE user SET pw=shared_secret(pw,login), mtime=now()"
     " WHERE length(pw)>0 AND length(pw)!=40"
@@ -568,7 +613,33 @@ void test_prompt_user_cmd(void){
   Blob answer;
   if( g.argc!=3 ) usage("PROMPT");
   prompt_user(g.argv[2], &answer);
-  fossil_print("%s", blob_str(&answer));
+  fossil_print("%s\n", blob_str(&answer));
+}
+
+/*
+** COMMAND: test-prompt-password
+**
+** Usage: %fossil test-prompt-password PROMPT VERIFY
+**
+** Prompts the user for a password and then prints it verbatim.
+**
+** Behavior is controlled by the VERIFY parameter:
+**
+**     0     Just ask once.
+**
+**     1     If the first answer is a non-empty string, ask for
+**           verification.  Repeat if the two strings do not match.
+**
+**     2     Ask twice, repeat if the strings do not match.
+
+*/
+void test_prompt_password_cmd(void){
+  Blob answer;
+  int iVerify = 0;
+  if( g.argc!=4 ) usage("PROMPT VERIFY");
+  iVerify = atoi(g.argv[3]);
+  prompt_for_password(g.argv[2], &answer, iVerify);
+  fossil_print("[%s]\n", blob_str(&answer));
 }
 
 /*
@@ -587,6 +658,7 @@ void access_log_page(void){
   int y = atoi(PD("y","3"));
   int n = atoi(PD("n","200"));
   int skip = atoi(PD("o","0"));
+  const char *zUser = P("u");
   Blob sql;
   Stmt q;
   int cnt = 0;
@@ -600,24 +672,24 @@ void access_log_page(void){
 
   if( P("delall") && P("delallbtn") ){
     db_multi_exec("DELETE FROM accesslog");
-    cgi_redirectf("%s/access_log?y=%d&n=%d&o=%o", g.zTop, y, n, skip);
+    cgi_redirectf("%R/access_log?y=%d&n=%d&o=%o", y, n, skip);
     return;
   }
   if( P("delanon") && P("delanonbtn") ){
     db_multi_exec("DELETE FROM accesslog WHERE uname='anonymous'");
-    cgi_redirectf("%s/access_log?y=%d&n=%d&o=%o", g.zTop, y, n, skip);
+    cgi_redirectf("%R/access_log?y=%d&n=%d&o=%o", y, n, skip);
     return;
   }
   if( P("delfail") && P("delfailbtn") ){
     db_multi_exec("DELETE FROM accesslog WHERE NOT success");
-    cgi_redirectf("%s/access_log?y=%d&n=%d&o=%o", g.zTop, y, n, skip);
+    cgi_redirectf("%R/access_log?y=%d&n=%d&o=%o", y, n, skip);
     return;
   }
   if( P("delold") && P("deloldbtn") ){
     db_multi_exec("DELETE FROM accesslog WHERE rowid in"
                   "(SELECT rowid FROM accesslog ORDER BY rowid DESC"
                   " LIMIT -1 OFFSET 200)");
-    cgi_redirectf("%s/access_log?y=%d&n=%d", g.zTop, y, n);
+    cgi_redirectf("%R/access_log?y=%d&n=%d", y, n);
     return;
   }
   style_header("Access Log");
@@ -626,22 +698,26 @@ void access_log_page(void){
     "SELECT uname, ipaddr, datetime(mtime,toLocal()), success"
     "  FROM accesslog"
   );
-  if( y==1 ){
+  if( zUser ){
+    blob_append_sql(&sql, "  WHERE uname=%Q", zUser);
+    n = 1000000000;
+    skip = 0;
+  }else if( y==1 ){
     blob_append(&sql, "  WHERE success", -1);
   }else if( y==2 ){
     blob_append(&sql, "  WHERE NOT success", -1);
   }
   blob_append_sql(&sql,"  ORDER BY rowid DESC LIMIT %d OFFSET %d", n+1, skip);
   if( skip ){
-    style_submenu_element("Newer", "Newer entries",
-              "%s/access_log?o=%d&n=%d&y=%d", g.zTop, skip>=n ? skip-n : 0,
-              n, y);
+    style_submenu_element("Newer", "%R/access_log?o=%d&n=%d&y=%d",
+              skip>=n ? skip-n : 0, n, y);
   }
   rc = db_prepare_ignore_error(&q, "%s", blob_sql_text(&sql));
   fLogEnabled = db_get_boolean("access-log", 0);
   @ <div align="center">Access logging is %s(fLogEnabled?"on":"off").
   @ (Change this on the <a href="setup_settings">settings</a> page.)</div>
-  @ <table border="1" cellpadding="5" id="logtable" align="center">
+  @ <table border="1" cellpadding="5" class="sortable" align="center" \
+  @  data-column-types='Ttt' data-init-sort='1'>
   @ <thead><tr><th width="33%%">Date</th><th width="34%%">User</th>
   @ <th width="33%%">IP Address</th></tr></thead><tbody>
   while( rc==SQLITE_OK && db_step(&q)==SQLITE_ROW ){
@@ -651,8 +727,8 @@ void access_log_page(void){
     int bSuccess = db_column_int(&q, 3);
     cnt++;
     if( cnt>n ){
-      style_submenu_element("Older", "Older entries",
-                  "%s/access_log?o=%d&n=%d&y=%d", g.zTop, skip+n, n, y);
+      style_submenu_element("Older", "%R/access_log?o=%d&n=%d&y=%d",
+                  skip+n, n, y);
       break;
     }
     if( bSuccess ){
@@ -663,32 +739,31 @@ void access_log_page(void){
     @ <td>%s(zDate)</td><td>%h(zName)</td><td>%h(zIP)</td></tr>
   }
   if( skip>0 || cnt>n ){
-    style_submenu_element("All", "All entries",
-          "%s/access_log?n=10000000", g.zTop);
+    style_submenu_element("All", "%R/access_log?n=10000000");
   }
   @ </tbody></table>
   db_finalize(&q);
   @ <hr />
-  @ <form method="post" action="%s(g.zTop)/access_log">
+  @ <form method="post" action="%R/access_log">
   @ <label><input type="checkbox" name="delold">
   @ Delete all but the most recent 200 entries</input></label>
   @ <input type="submit" name="deloldbtn" value="Delete"></input>
   @ </form>
-  @ <form method="post" action="%s(g.zTop)/access_log">
+  @ <form method="post" action="%R/access_log">
   @ <label><input type="checkbox" name="delanon">
   @ Delete all entries for user "anonymous"</input></label>
   @ <input type="submit" name="delanonbtn" value="Delete"></input>
   @ </form>
-  @ <form method="post" action="%s(g.zTop)/access_log">
+  @ <form method="post" action="%R/access_log">
   @ <label><input type="checkbox" name="delfail">
   @ Delete all failed login attempts</input></label>
   @ <input type="submit" name="delfailbtn" value="Delete"></input>
   @ </form>
-  @ <form method="post" action="%s(g.zTop)/access_log">
+  @ <form method="post" action="%R/access_log">
   @ <label><input type="checkbox" name="delall">
   @ Delete all entries</input></label>
   @ <input type="submit" name="delallbtn" value="Delete"></input>
   @ </form>
-  output_table_sorting_javascript("logtable", "Ttt", 1);
-  style_footer();
+  style_table_sorter();
+  style_finish_page();
 }
