@@ -22,6 +22,10 @@
 #include "config.h"
 #ifdef _WIN32
 /* This code is for win32 only */
+# if !defined(_WIN32_WINNT)
+#  define _WIN32_WINNT 0x0501
+# endif
+#include <winsock2.h>
 #include <ws2tcpip.h>
 #include <windows.h>
 #include <process.h>
@@ -201,22 +205,22 @@ static int DualSocket_listen(DualSocket* ds, const char* zIp, int iPort){
 */
 static void DualSocket_accept(DualSocket* pListen, DualSocket* pClient,
                               DualAddr* pClientAddr){
-	fd_set rs;
+  fd_set rs;
   int rs_count = 0;
   assert( pListen!=NULL && pClient!=NULL && pClientAddr!= NULL );
   DualSocket_init(pClient);
   DualAddr_init(pClientAddr);
   FD_ZERO(&rs);
-	if( pListen->s4!=INVALID_SOCKET ){
+  if( pListen->s4!=INVALID_SOCKET ){
     FD_SET(pListen->s4, &rs);
     ++rs_count;
   }
-	if( pListen->s6!=INVALID_SOCKET ){
+  if( pListen->s6!=INVALID_SOCKET ){
     FD_SET(pListen->s6, &rs);
     ++rs_count;
   }
-	if( select(rs_count, &rs, 0, 0, 0 /*blocking*/)==SOCKET_ERROR ){
-		return;
+  if( select(rs_count, &rs, 0, 0, 0 /*blocking*/)==SOCKET_ERROR ){
+    return;
   }
   if( FD_ISSET(pListen->s4, &rs) ){
     pClient->s4 = accept(pListen->s4, (struct sockaddr*)&pClientAddr->a4.addr,
@@ -389,11 +393,11 @@ static void win32_http_request(void *pAppData){
   zIp = SocketAddr_toString(&p->addr);
   if( (p->flags & HTTP_SERVER_HAD_CHECKOUT)==0 ){
     assert( g.zRepositoryName && g.zRepositoryName[0] );
-    sqlite3_snprintf(sizeof(zCmd), zCmd, "%s%s\n%s\n%s\n%s",
+    sqlite3_snprintf(sizeof(zCmd), zCmd, "%s--in %s\n--out %s\n--ipaddr %s\n%s",
       get_utf8_bom(0), zRequestFName, zReplyFName, zIp, g.zRepositoryName
     );
   }else{
-    sqlite3_snprintf(sizeof(zCmd), zCmd, "%s%s\n%s\n%s",
+    sqlite3_snprintf(sizeof(zCmd), zCmd, "%s--in %s\n--out %s\n--ipaddr %s",
       get_utf8_bom(0), zRequestFName, zReplyFName, zIp
     );
   }
@@ -402,7 +406,8 @@ static void win32_http_request(void *pAppData){
   if( aux==0 ) goto end_request;
   fwrite(zCmd, 1, strlen(zCmd), aux);
 
-  sqlite3_snprintf(sizeof(zCmd), zCmd, "\"%s\" http -args \"%s\" --nossl%s",
+  sqlite3_snprintf(sizeof(zCmd), zCmd,
+    "\"%s\" http -args \"%s\" --nossl%s",
     g.nameOfExe, zCmdFName, p->zOptions
   );
   in = fossil_fopen(zReplyFName, "w+b");
@@ -472,7 +477,8 @@ static void win32_scgi_request(void *pAppData){
   assert( g.zRepositoryName && g.zRepositoryName[0] );
   zIp = SocketAddr_toString(&p->addr);
   sqlite3_snprintf(sizeof(zCmd), zCmd,
-    "\"%s\" http \"%s\" \"%s\" %s \"%s\" --scgi --nossl%s",
+    "\"%s\" http --in \"%s\" --out \"%s\" --ipaddr %s \"%s\""
+    " --scgi --nossl%s",
     g.nameOfExe, zRequestFName, zReplyFName, zIp,
     g.zRepositoryName, p->zOptions
   );
@@ -531,11 +537,20 @@ void win32_http_server(
 #endif
 
   blob_zero(&options);
+  if( PB("HTTPS") ){
+    blob_appendf(&options, " --https");
+  }
   if( zBaseUrl ){
-    blob_appendf(&options, " --baseurl %s", zBaseUrl);
+    blob_appendf(&options, " --baseurl ");
+    blob_append_escaped_arg(&options, zBaseUrl, 0);
   }
   if( zNotFound ){
-    blob_appendf(&options, " --notfound %s", zNotFound);
+    blob_appendf(&options, " --notfound ");
+    blob_append_escaped_arg(&options, zNotFound, 1);
+  }
+  if( g.zCkoutAlias ){
+    blob_appendf(&options, " --ckout-alias ");
+    blob_append_escaped_arg(&options, g.zCkoutAlias, 0);
   }
   if( zFileGlob ){
     blob_appendf(&options, " --files-urlenc %T", zFileGlob);
@@ -549,10 +564,18 @@ void win32_http_server(
   if( flags & HTTP_SERVER_REPOLIST ){
     blob_appendf(&options, " --repolist");
   }
+  if( g.zExtRoot && g.zExtRoot[0] ){
+    blob_appendf(&options, " --extroot");
+    blob_append_escaped_arg(&options, g.zExtRoot, 1);
+  }
   zSkin = skin_in_use();
   if( zSkin ){
     blob_appendf(&options, " --skin %s", zSkin);
   }
+  if( g.zMainMenuFile ){
+    blob_appendf(&options, " --mainmenu ");
+    blob_append_escaped_arg(&options, g.zMainMenuFile, 1);
+  }
 #if USE_SEE
   zSavedKey = db_get_saved_encryption_key();
   savedKeySize = db_get_saved_encryption_key_size();
@@ -562,7 +585,7 @@ void win32_http_server(
   }
 #endif
   if( WSAStartup(MAKEWORD(2,0), &wd) ){
-    fossil_fatal("unable to initialize winsock");
+    fossil_panic("unable to initialize winsock");
   }
   DualSocket_init(&ds);
   while( iPort<=mxPort ){
@@ -583,6 +606,7 @@ void win32_http_server(
     break;
   }
   if( iPort>mxPort ){
+    /* These exits are merely fatal because firewall settings can cause them. */
     if( mnPort==mxPort ){
       fossil_fatal("unable to open listening socket on port %d", mnPort);
     }else{
@@ -591,7 +615,7 @@ void win32_http_server(
     }
   }
   if( !GetTempPathW(MAX_PATH, zTmpPath) ){
-    fossil_fatal("unable to get path to the temporary directory.");
+    fossil_panic("unable to get path to the temporary directory.");
   }
   zTempPrefix = mprintf("%sfossil_server_P%d",
                         fossil_unicode_to_utf8(zTmpPath), iPort);
@@ -642,7 +666,7 @@ void win32_http_server(
         return;
       }else{
         WSACleanup();
-        fossil_fatal("error from accept()");
+        fossil_panic("error from accept()");
       }
     }
     if( client.s4!=INVALID_SOCKET ){
@@ -742,7 +766,7 @@ static char *win32_get_last_errmsg(void){
   if( nMsg ){
     zMsg = fossil_unicode_to_utf8(tmp);
   }else{
-    fossil_fatal("unable to get system error message.");
+    fossil_panic("unable to get system error message.");
   }
   if( tmp ){
     LocalFree((HLOCAL) tmp);
@@ -951,7 +975,7 @@ int win32_http_service(
 **              Specifies the TCP port (default port is 8080) on which the
 **              server should listen.
 **
-**         -R|--repository REPOSITORY
+**         -R|--repository REPO
 **
 **              Specifies the name of the repository to be served.
 **              The repository option may be omitted if the working directory
